@@ -15,7 +15,19 @@
 #
 set -eu
 
-trap "kill `jobs -p` &> /dev/null || true" ERR
+# Cleanup to run whenever we exit
+function cleanup()
+{
+    for n in `jobs -p`
+    do
+        kill $n 2> /dev/null || true
+    done
+    wait
+}
+
+# Install cleanup handler (except for kill -9)
+trap cleanup EXIT
+
 
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
@@ -31,7 +43,13 @@ BASEDB=${1:-"auditor-basedb"}
 # elsewhere
 TARGET_DB=taler-auditor-basedb
 
-# Configuation file will be edited, so we create one
+WALLET_DB=${BASEDB:-"wallet"}.wdb
+
+# delete existing wallet database
+rm -f $WALLET_DB
+
+
+# Configuration file will be edited, so we create one
 # from the template.
 CONF=generate-auditor-basedb-prod.conf
 cp generate-auditor-basedb-template.conf $CONF
@@ -47,6 +65,7 @@ echo " FOUND"
 
 
 # Clean up
+
 DATA_DIR=`taler-config -f -c $CONF -s PATHS -o TALER_HOME`
 rm -rf $DATA_DIR || true
 
@@ -54,6 +73,7 @@ rm -rf $DATA_DIR || true
 dropdb $TARGET_DB >/dev/null 2>/dev/null || true
 createdb $TARGET_DB || exit_skip "Could not create database $TARGET_DB"
 
+
 # obtain key configuration data
 MASTER_PRIV_FILE=`taler-config -f -c $CONF -s EXCHANGE -o MASTER_PRIV_FILE`
 MASTER_PRIV_DIR=`dirname $MASTER_PRIV_FILE`
@@ -95,7 +115,7 @@ mv a2e.dat $ABD
 
 # Launch services
 echo "Launching services"
-taler-bank-manage-testing $CONF postgres:///$TARGET_DB serve-http &
+taler-bank-manage-testing $CONF postgres:///$TARGET_DB serve &
 taler-exchange-httpd -c $CONF 2> taler-exchange-httpd.log &
 taler-merchant-httpd -c $CONF -L INFO 2> taler-merchant-httpd.log &
 taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
@@ -121,19 +141,38 @@ done
 
 if [ 1 != $OK ]
 then
-    kill `jobs -p`
-    wait
     exit_skip "Failed to launch services"
 fi
+
+# Setup merchant
+
+curl -H "Content-Type: application/json" -X POST -d '{"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_ms" : 3600000},"default_pay_delay":{"d_ms": 3600000}}' http://localhost:9966/private/instances
+
+
 echo " DONE"
 
 # run wallet CLI
 echo "Running wallet"
-taler-wallet-cli testing integrationtest -e $EXCHANGE_URL -m $MERCHANT_URL -b $BANK_URL
+
+
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'runIntegrationTest' \
+  "$(jq -n '
+    {
+      amountToSpend: "TESTKUDOS:4",
+      amountToWithdraw: "TESTKUDOS:10",
+      bankBaseUrl: $BANK_URL,
+      exchangeBaseUrl: $EXCHANGE_URL,
+      merchantApiKey: "sandbox",
+      merchantBaseUrl: $MERCHANT_URL,
+    }' \
+    --arg MERCHANT_URL "$MERCHANT_URL" \
+    --arg EXCHANGE_URL "$EXCHANGE_URL" \
+    --arg BANK_URL "$BANK_URL"
+  )"
+
 
 echo "Shutting down services"
-kill `jobs -p`
-wait
+cleanup
 
 # Dump database
 echo "Dumping database"
