@@ -125,7 +125,7 @@ handle_signal (int signal_number)
                   1);
   /* While one might like to "handle errors" here, even logging via fprintf()
      isn't safe inside of a signal handler. So there is nothing we safely CAN
-     do. OTOH, also very little that can go wrong in pratice. Calling _exit()
+     do. OTOH, also very little that can go wrong in practice. Calling _exit()
      on errors might be a possibility, but that might do more harm than good. *///
 }
 
@@ -302,7 +302,7 @@ handle_mhd_completion_callback (void *cls,
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
   */
-static int
+static MHD_RESULT
 handle_version (struct TAH_RequestHandler *rh,
                 struct MHD_Connection *connection,
                 void **connection_cls,
@@ -347,7 +347,7 @@ handle_version (struct TAH_RequestHandler *rh,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_mhd_request (void *cls,
                     struct MHD_Connection *connection,
                     const char *url,
@@ -479,7 +479,7 @@ auditor_serve_process_config (void)
   {
     /* Fall back to trying to read private key */
     char *auditor_key_file;
-    struct GNUNET_CRYPTO_EddsaPrivateKey *eddsa_priv;
+    struct GNUNET_CRYPTO_EddsaPrivateKey eddsa_priv;
 
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_get_value_filename (cfg,
@@ -495,8 +495,10 @@ auditor_serve_process_config (void)
                                  "AUDITOR_PRIV_FILE");
       return GNUNET_SYSERR;
     }
-    eddsa_priv = GNUNET_CRYPTO_eddsa_key_create_from_file (auditor_key_file);
-    if (NULL == eddsa_priv)
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_key_from_file (auditor_key_file,
+                                           GNUNET_NO,
+                                           &eddsa_priv))
     {
       /* Both failed, complain! */
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
@@ -509,9 +511,8 @@ auditor_serve_process_config (void)
       return 1;
     }
     GNUNET_free (auditor_key_file);
-    GNUNET_CRYPTO_eddsa_key_get_public (eddsa_priv,
+    GNUNET_CRYPTO_eddsa_key_get_public (&eddsa_priv,
                                         &auditor_pub.eddsa_pub);
-    GNUNET_free (eddsa_priv);
   }
   return GNUNET_OK;
 }
@@ -578,10 +579,10 @@ main (int argc,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Malformed configuration file `%s', exiting ...\n",
                 cfgfile);
-    GNUNET_free_non_null (cfgfile);
+    GNUNET_free (cfgfile);
     return 1;
   }
-  GNUNET_free_non_null (cfgfile);
+  GNUNET_free (cfgfile);
 
   if (GNUNET_OK !=
       auditor_serve_process_config ())
