@@ -56,7 +56,7 @@ static pthread_mutex_t lock;
  * @param es information about the exchange's signing key
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 verify_and_execute_deposit_confirmation (
   struct MHD_Connection *connection,
   const struct TALER_AUDITORDB_DepositConfirmation *dc,
@@ -86,7 +86,7 @@ verify_and_execute_deposit_confirmation (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_FORBIDDEN,
                                        TALER_EC_DEPOSIT_CONFIRMATION_SIGNATURE_INVALID,
-                                       "master_sig (expired)");
+                                       "master signature expired");
   }
 
   /* check our cache */
@@ -105,14 +105,14 @@ verify_and_execute_deposit_confirmation (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_DB_SETUP_FAILED,
-                                       "failed to establish session with database");
+                                       NULL);
   }
   if (! cached)
   {
     /* Not in cache, need to verify the signature, persist it, and possibly cache it */
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
-                                    &skv.purpose,
+                                    &skv,
                                     &es->master_sig.eddsa_signature,
                                     &es->master_public_key.eddsa_pub))
     {
@@ -120,7 +120,7 @@ verify_and_execute_deposit_confirmation (
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_DEPOSIT_CONFIRMATION_SIGNATURE_INVALID,
-                                         "master_sig");
+                                         "master signature invalid");
     }
 
     /* execute transaction */
@@ -134,7 +134,7 @@ verify_and_execute_deposit_confirmation (
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_AUDITOR_EXCHANGE_STORE_DB_ERROR,
-                                         "failed to persist exchange signing key");
+                                         NULL);
     }
 
     /* Cache it, due to concurreny it might already be in the cache,
@@ -155,7 +155,7 @@ verify_and_execute_deposit_confirmation (
       .purpose.size = htonl (sizeof (struct TALER_DepositConfirmationPS)),
       .h_contract_terms = dc->h_contract_terms,
       .h_wire = dc->h_wire,
-      .timestamp = GNUNET_TIME_absolute_hton (dc->timestamp),
+      .exchange_timestamp = GNUNET_TIME_absolute_hton (dc->exchange_timestamp),
       .refund_deadline = GNUNET_TIME_absolute_hton (dc->refund_deadline),
       .coin_pub = dc->coin_pub,
       .merchant = dc->merchant
@@ -165,7 +165,7 @@ verify_and_execute_deposit_confirmation (
                        &dc->amount_without_fee);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
-                                    &dcs.purpose,
+                                    &dcs,
                                     &dc->exchange_sig.eddsa_signature,
                                     &dc->exchange_pub.eddsa_pub))
     {
@@ -174,7 +174,7 @@ verify_and_execute_deposit_confirmation (
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_DEPOSIT_CONFIRMATION_SIGNATURE_INVALID,
-                                         "exchange_sig");
+                                         "exchange signature invalid");
     }
   }
 
@@ -189,7 +189,7 @@ verify_and_execute_deposit_confirmation (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_DEPOSIT_CONFIRMATION_STORE_DB_ERROR,
-                                       "failed to persist deposit-confirmation data");
+                                       NULL);
   }
   return TALER_MHD_reply_json_pack (connection,
                                     MHD_HTTP_OK,
@@ -212,7 +212,7 @@ verify_and_execute_deposit_confirmation (
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
   */
-int
+MHD_RESULT
 TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
                                   struct MHD_Connection *connection,
                                   void **connection_cls,
@@ -224,17 +224,18 @@ TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("h_contract_terms", &dc.h_contract_terms),
     GNUNET_JSON_spec_fixed_auto ("h_wire", &dc.h_wire),
-    GNUNET_JSON_spec_absolute_time ("timestamp", &dc.timestamp),
-    GNUNET_JSON_spec_absolute_time ("refund_deadline", &dc.refund_deadline),
+    TALER_JSON_spec_absolute_time ("exchange_timestamp",
+                                   &dc.exchange_timestamp),
+    TALER_JSON_spec_absolute_time ("refund_deadline", &dc.refund_deadline),
     TALER_JSON_spec_amount ("amount_without_fee", &dc.amount_without_fee),
     GNUNET_JSON_spec_fixed_auto ("coin_pub", &dc.coin_pub),
     GNUNET_JSON_spec_fixed_auto ("merchant_pub", &dc.merchant),
     GNUNET_JSON_spec_fixed_auto ("exchange_sig",  &dc.exchange_sig),
     GNUNET_JSON_spec_fixed_auto ("exchange_pub",  &dc.exchange_pub),
     GNUNET_JSON_spec_fixed_auto ("master_pub",  &es.master_public_key),
-    GNUNET_JSON_spec_absolute_time ("ep_start",  &es.ep_start),
-    GNUNET_JSON_spec_absolute_time ("ep_expire",  &es.ep_expire),
-    GNUNET_JSON_spec_absolute_time ("ep_end",  &es.ep_end),
+    TALER_JSON_spec_absolute_time ("ep_start",  &es.ep_start),
+    TALER_JSON_spec_absolute_time ("ep_expire",  &es.ep_expire),
+    TALER_JSON_spec_absolute_time ("ep_end",  &es.ep_end),
     GNUNET_JSON_spec_fixed_auto ("master_sig",  &es.master_sig),
     GNUNET_JSON_spec_end ()
   };
@@ -245,7 +246,7 @@ TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
   (void) upload_data_size;
   {
     json_t *json;
-    int res;
+    enum GNUNET_GenericReturnValue res;
 
     res = TALER_MHD_parse_post_json (connection,
                                      connection_cls,
@@ -270,7 +271,7 @@ TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
   es.exchange_pub = dc.exchange_pub; /* used twice! */
   dc.master_public_key = es.master_public_key;
   {
-    int res;
+    MHD_RESULT res;
 
     res = verify_and_execute_deposit_confirmation (connection,
                                                    &dc,
