@@ -290,14 +290,12 @@ report_emergency_by_amount (
                                  issue->expire_deposit),
                                "value",
                                TALER_JSON_from_amount_nbo (&issue->value)));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&reported_emergency_risk_by_amount,
-                                   &reported_emergency_risk_by_amount,
-                                   risk));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&reported_emergency_loss,
-                                   &reported_emergency_loss,
-                                   loss));
+  TALER_ARL_amount_add (&reported_emergency_risk_by_amount,
+                        &reported_emergency_risk_by_amount,
+                        risk);
+  TALER_ARL_amount_add (&reported_emergency_loss,
+                        &reported_emergency_loss,
+                        loss);
 }
 
 
@@ -342,18 +340,15 @@ report_emergency_by_count (
                                  issue->expire_deposit),
                                "value",
                                TALER_JSON_from_amount_nbo (&issue->value)));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&reported_emergency_risk_by_count,
-                                   &reported_emergency_risk_by_count,
-                                   risk));
+  TALER_ARL_amount_add (&reported_emergency_risk_by_count,
+                        &reported_emergency_risk_by_count,
+                        risk);
   TALER_amount_ntoh (&denom_value,
                      &issue->value);
   for (uint64_t i = num_issued; i<num_known; i++)
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (
-                     &reported_emergency_loss_by_count,
-                     &reported_emergency_loss_by_count,
-                     &denom_value));
+    TALER_ARL_amount_add (&reported_emergency_loss_by_count,
+                          &reported_emergency_loss_by_count,
+                          &denom_value);
 
 }
 
@@ -388,19 +383,17 @@ report_amount_arithmetic_inconsistency (
                             auditor))
   {
     /* exchange > auditor */
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&delta,
-                                          exchange,
-                                          auditor));
+    TALER_ARL_amount_subtract (&delta,
+                               exchange,
+                               auditor);
   }
   else
   {
     /* auditor < exchange */
     profitable = -profitable;
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&delta,
-                                          auditor,
-                                          exchange));
+    TALER_ARL_amount_subtract (&delta,
+                               auditor,
+                               exchange);
   }
   TALER_ARL_report (report_amount_arithmetic_inconsistencies,
                     json_pack ("{s:s, s:I, s:o, s:o, s:I}",
@@ -414,10 +407,9 @@ report_amount_arithmetic_inconsistency (
     target = (1 == profitable)
              ? &total_arithmetic_delta_plus
              : &total_arithmetic_delta_minus;
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (target,
-                                     target,
-                                     &delta));
+    TALER_ARL_amount_add (target,
+                          target,
+                          &delta);
   }
 }
 
@@ -495,51 +487,44 @@ check_coin_history (const struct TALER_CoinSpendPublicKeyP *coin_pub,
     {
     case TALER_EXCHANGEDB_TT_DEPOSIT:
       /* spent += pos->amount_with_fee */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&spent,
-                                       &spent,
-                                       &pos->details.deposit->amount_with_fee));
+      TALER_ARL_amount_add (&spent,
+                            &spent,
+                            &pos->details.deposit->amount_with_fee);
       deposit_fee = pos->details.deposit->deposit_fee;
       break;
     case TALER_EXCHANGEDB_TT_MELT:
       /* spent += pos->amount_with_fee */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&spent,
-                                       &spent,
-                                       &pos->details.melt->amount_with_fee));
+      TALER_ARL_amount_add (&spent,
+                            &spent,
+                            &pos->details.melt->amount_with_fee);
       break;
     case TALER_EXCHANGEDB_TT_REFUND:
       /* refunded += pos->refund_amount - pos->refund_fee */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&refunded,
-                                       &refunded,
-                                       &pos->details.refund->refund_amount));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&spent,
-                                       &spent,
-                                       &pos->details.refund->refund_fee));
+      TALER_ARL_amount_add (&refunded,
+                            &refunded,
+                            &pos->details.refund->refund_amount);
+      TALER_ARL_amount_add (&spent,
+                            &spent,
+                            &pos->details.refund->refund_fee);
       have_refund = GNUNET_YES;
       break;
     case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
       /* refunded += pos->value */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&refunded,
-                                       &refunded,
-                                       &pos->details.old_coin_recoup->value));
+      TALER_ARL_amount_add (&refunded,
+                            &refunded,
+                            &pos->details.old_coin_recoup->value);
       break;
     case TALER_EXCHANGEDB_TT_RECOUP:
       /* spent += pos->value */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&spent,
-                                       &spent,
-                                       &pos->details.recoup->value));
+      TALER_ARL_amount_add (&spent,
+                            &spent,
+                            &pos->details.recoup->value);
       break;
     case TALER_EXCHANGEDB_TT_RECOUP_REFRESH:
       /* spent += pos->value */
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&spent,
-                                       &spent,
-                                       &pos->details.recoup_refresh->value));
+      TALER_ARL_amount_add (&spent,
+                            &spent,
+                            &pos->details.recoup_refresh->value);
       break;
     }
   }
@@ -547,16 +532,14 @@ check_coin_history (const struct TALER_CoinSpendPublicKeyP *coin_pub,
   if (have_refund)
   {
     /* If we gave any refund, also discount ONE deposit fee */
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&refunded,
-                                     &refunded,
-                                     &deposit_fee));
+    TALER_ARL_amount_add (&refunded,
+                          &refunded,
+                          &deposit_fee);
   }
   /* total coin value = original value plus refunds */
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total,
-                                   &refunded,
-                                   value));
+  TALER_ARL_amount_add (&total,
+                        &refunded,
+                        value);
   if (1 ==
       TALER_amount_cmp (&spent,
                         &total))
@@ -564,10 +547,9 @@ check_coin_history (const struct TALER_CoinSpendPublicKeyP *coin_pub,
     /* spent > total: bad */
     struct TALER_Amount loss;
 
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&loss,
-                                          &spent,
-                                          &total));
+    TALER_ARL_amount_subtract (&loss,
+                               &spent,
+                               &total);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Loss detected for coin %s - %s\n",
                 TALER_B2S (coin_pub),
@@ -743,7 +725,7 @@ init_denomination (const struct GNUNET_HashCode *denom_hash,
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (
           TALER_SIGNATURE_MASTER_DENOMINATION_KEY_REVOKED,
-          &rm.purpose,
+          &rm,
           &msig.eddsa_signature,
           &TALER_ARL_master_pub.eddsa_pub))
     {
@@ -844,10 +826,9 @@ sync_denomination (void *cls,
       /* The denomination expired and carried a balance; we can now
          book the remaining balance as profit, and reduce our risk
          exposure by the accumulated risk of the denomination. */
-      GNUNET_assert (GNUNET_SYSERR !=
-                     TALER_amount_subtract (&total_risk,
-                                            &total_risk,
-                                            &ds->denom_risk));
+      TALER_ARL_amount_subtract (&total_risk,
+                                 &total_risk,
+                                 &ds->denom_risk);
       /* If the above fails, our risk assessment is inconsistent!
          This is really, really bad (auditor-internal invariant
          would be violated). Hence we can "safely" assert.  If
@@ -1010,6 +991,8 @@ withdraw_cb (void *cls,
     report_row_inconsistency ("withdraw",
                               rowid,
                               "denomination key not found");
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
@@ -1035,26 +1018,24 @@ withdraw_cb (void *cls,
               GNUNET_h2s (&dh),
               TALER_amount2s (&value));
   ds->num_issued++;
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&ds->denom_balance,
-                                   &ds->denom_balance,
-                                   &value));
+  TALER_ARL_amount_add (&ds->denom_balance,
+                        &ds->denom_balance,
+                        &value);
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "New balance of denomination `%s' is %s\n",
               GNUNET_h2s (&dh),
               TALER_amount2s (&ds->denom_balance));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_escrow_balance,
-                                   &total_escrow_balance,
-                                   &value));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_risk,
-                                   &total_risk,
-                                   &value));
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&ds->denom_risk,
-                                   &ds->denom_risk,
-                                   &value));
+  TALER_ARL_amount_add (&total_escrow_balance,
+                        &total_escrow_balance,
+                        &value);
+  TALER_ARL_amount_add (&total_risk,
+                        &total_risk,
+                        &value);
+  TALER_ARL_amount_add (&ds->denom_risk,
+                        &ds->denom_risk,
+                        &value);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1220,11 +1201,9 @@ check_known_coin (const char *operation,
                                    loss_potential),
                                  "coin_pub", GNUNET_JSON_from_data_auto (
                                    coin_pub)));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_sig_loss,
-                                     &total_bad_sig_loss,
-                                     loss_potential));
-
+    TALER_ARL_amount_add (&total_bad_sig_loss,
+                          &total_bad_sig_loss,
+                          loss_potential);
   }
   GNUNET_CRYPTO_rsa_signature_free (ci.denom_sig.rsa_signature);
   return qs;
@@ -1277,6 +1256,8 @@ refresh_session_cb (void *cls,
     report_row_inconsistency ("melt",
                               rowid,
                               "denomination key not found");
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
@@ -1308,11 +1289,13 @@ refresh_session_cb (void *cls,
       .coin_pub = *coin_pub
     };
 
+    GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
+                                       &rmc.h_denom_pub);
     TALER_amount_hton (&rmc.amount_with_fee,
                        amount_with_fee);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_MELT,
-                                    &rmc.purpose,
+                                    &rmc,
                                     &coin_sig->eddsa_signature,
                                     &coin_pub->eddsa_pub))
     {
@@ -1324,10 +1307,9 @@ refresh_session_cb (void *cls,
                                      amount_with_fee),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount_with_fee);
     }
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -1367,10 +1349,11 @@ refresh_session_cb (void *cls,
                                      amount_with_fee),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_refresh_hanging,
-                                       &total_refresh_hanging,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&total_refresh_hanging,
+                            &total_refresh_hanging,
+                            amount_with_fee);
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
       return GNUNET_OK;
     }
     if (GNUNET_SYSERR == reveal_ctx.err)
@@ -1378,8 +1361,12 @@ refresh_session_cb (void *cls,
 
     if (GNUNET_OK != reveal_ctx.err)
     {
-      GNUNET_free_non_null (reveal_ctx.new_issues);
-      return (GNUNET_SYSERR == reveal_ctx.err) ? GNUNET_SYSERR : GNUNET_OK;
+      GNUNET_free (reveal_ctx.new_issues);
+      if (GNUNET_SYSERR == reveal_ctx.err)
+        return GNUNET_SYSERR;
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
+      return GNUNET_OK;
     }
 
     /* Check that the resulting amounts are consistent with the value being
@@ -1397,14 +1384,12 @@ refresh_session_cb (void *cls,
                          &reveal_ctx.new_issues[i]->fee_withdraw);
       TALER_amount_ntoh (&value,
                          &reveal_ctx.new_issues[i]->value);
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&refresh_cost,
-                                       &refresh_cost,
-                                       &fee));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&refresh_cost,
-                                       &refresh_cost,
-                                       &value));
+      TALER_ARL_amount_add (&refresh_cost,
+                            &refresh_cost,
+                            &fee);
+      TALER_ARL_amount_add (&refresh_cost,
+                            &refresh_cost,
+                            &value);
     }
 
     /* compute contribution of old coin */
@@ -1413,10 +1398,10 @@ refresh_session_cb (void *cls,
 
       TALER_amount_ntoh (&melt_fee,
                          &issue->fee_refresh);
-      if (GNUNET_OK !=
-          TALER_amount_subtract (&amount_without_fee,
-                                 amount_with_fee,
-                                 &melt_fee))
+      if (TALER_ARL_SR_POSITIVE !=
+          TALER_ARL_amount_subtract_neg (&amount_without_fee,
+                                         amount_with_fee,
+                                         &melt_fee))
       {
         /* Melt fee higher than contribution of melted coin; this makes
            no sense (exchange should never have accepted the operation) */
@@ -1469,29 +1454,25 @@ refresh_session_cb (void *cls,
                     GNUNET_h2s (&reveal_ctx.new_issues[i]->denom_hash),
                     TALER_amount2s (&value));
         dsi->num_issued++;
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&dsi->denom_balance,
-                                         &dsi->denom_balance,
-                                         &value));
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&dsi->denom_risk,
-                                         &dsi->denom_risk,
-                                         &value));
+        TALER_ARL_amount_add (&dsi->denom_balance,
+                              &dsi->denom_balance,
+                              &value);
+        TALER_ARL_amount_add (&dsi->denom_risk,
+                              &dsi->denom_risk,
+                              &value);
         GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                     "New balance of denomination `%s' is %s\n",
                     GNUNET_h2s (&reveal_ctx.new_issues[i]->denom_hash),
                     TALER_amount2s (&dsi->denom_balance));
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&total_escrow_balance,
-                                         &total_escrow_balance,
-                                         &value));
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_amount_add (&total_risk,
-                                         &total_risk,
-                                         &value));
+        TALER_ARL_amount_add (&total_escrow_balance,
+                              &total_escrow_balance,
+                              &value);
+        TALER_ARL_amount_add (&total_risk,
+                              &total_risk,
+                              &value);
       }
     }
-    GNUNET_free_non_null (reveal_ctx.new_issues);
+    GNUNET_free (reveal_ctx.new_issues);
   }
 
   /* update old coin's denomination balance */
@@ -1506,15 +1487,14 @@ refresh_session_cb (void *cls,
   }
   else
   {
-    if (GNUNET_SYSERR ==
-        TALER_amount_subtract (&tmp,
-                               &dso->denom_balance,
-                               amount_with_fee))
+    if (TALER_ARL_SR_INVALID_NEGATIVE ==
+        TALER_ARL_amount_subtract_neg (&tmp,
+                                       &dso->denom_balance,
+                                       amount_with_fee))
     {
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&dso->denom_loss,
-                                       &dso->denom_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&dso->denom_loss,
+                            &dso->denom_loss,
+                            amount_with_fee);
       dso->report_emergency = GNUNET_YES;
     }
     else
@@ -1539,10 +1519,9 @@ refresh_session_cb (void *cls,
     }
     else
     {
-      GNUNET_assert (GNUNET_SYSERR !=
-                     TALER_amount_subtract (&total_escrow_balance,
-                                            &total_escrow_balance,
-                                            amount_with_fee));
+      TALER_ARL_amount_subtract (&total_escrow_balance,
+                                 &total_escrow_balance,
+                                 amount_with_fee);
     }
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "New balance of denomination `%s' after melt is %s\n",
@@ -1556,11 +1535,12 @@ refresh_session_cb (void *cls,
 
     TALER_amount_ntoh (&rfee,
                        &issue->fee_refresh);
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_melt_fee_income,
-                                     &total_melt_fee_income,
-                                     &rfee));
+    TALER_ARL_amount_add (&total_melt_fee_income,
+                          &total_melt_fee_income,
+                          &rfee);
   }
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1571,16 +1551,17 @@ refresh_session_cb (void *cls,
  *
  * @param cls closure
  * @param rowid unique serial ID for the deposit in our DB
- * @param timestamp when did the deposit happen
+ * @param exchange_timestamp when did the exchange get the deposit
+ * @param wallet_timestamp when did the contract signing happen
  * @param merchant_pub public key of the merchant
  * @param denom_pub denomination public key of @a coin_pub
  * @param coin_pub public key of the coin
  * @param coin_sig signature from the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param refund_deadline by which the merchant adviced that he might want
+ * @param refund_deadline by which the merchant advised that he might want
  *        to get a refund
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
  * @param receiver_wire_account wire details for the merchant, NULL from iterate_matching_deposits()
  * @param done flag set if the deposit was already executed (or not)
@@ -1589,7 +1570,8 @@ refresh_session_cb (void *cls,
 static int
 deposit_cb (void *cls,
             uint64_t rowid,
-            struct GNUNET_TIME_Absolute timestamp,
+            struct GNUNET_TIME_Absolute exchange_timestamp,
+            struct GNUNET_TIME_Absolute wallet_timestamp,
             const struct TALER_MerchantPublicKeyP *merchant_pub,
             const struct TALER_DenominationPublicKey *denom_pub,
             const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -1606,7 +1588,6 @@ deposit_cb (void *cls,
   struct DenominationSummary *ds;
   enum GNUNET_DB_QueryStatus qs;
 
-  (void) wire_deadline;
   (void) done;
   GNUNET_assert (rowid >= ppc.last_deposit_serial_id); /* should be monotonically increasing */
   ppc.last_deposit_serial_id = rowid + 1;
@@ -1619,8 +1600,17 @@ deposit_cb (void *cls,
     report_row_inconsistency ("deposits",
                               rowid,
                               "denomination key not found");
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
+  if (refund_deadline.abs_value_us >
+      wire_deadline.abs_value_us)
+  {
+    report_row_inconsistency ("deposits",
+                              rowid,
+                              "refund deadline past wire deadline");
+  }
 
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
   {
@@ -1647,13 +1637,15 @@ deposit_cb (void *cls,
       .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT),
       .purpose.size = htonl (sizeof (dr)),
       .h_contract_terms = *h_contract_terms,
-      .timestamp = GNUNET_TIME_absolute_hton (timestamp),
+      .wallet_timestamp = GNUNET_TIME_absolute_hton (wallet_timestamp),
       .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
       .deposit_fee = issue->fee_deposit,
       .merchant = *merchant_pub,
       .coin_pub = *coin_pub
     };
 
+    GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
+                                       &dr.h_denom_pub);
     if (GNUNET_OK !=
         TALER_JSON_merchant_wire_signature_hash (receiver_wire_account,
                                                  &dr.h_wire))
@@ -1666,10 +1658,11 @@ deposit_cb (void *cls,
                                      amount_with_fee),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount_with_fee);
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
       return GNUNET_OK;
     }
     TALER_amount_hton (&dr.amount_with_fee,
@@ -1679,7 +1672,7 @@ deposit_cb (void *cls,
        auditor performance! */
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
-                                    &dr.purpose,
+                                    &dr,
                                     &coin_sig->eddsa_signature,
                                     &coin_pub->eddsa_pub))
     {
@@ -1691,10 +1684,11 @@ deposit_cb (void *cls,
                                      amount_with_fee),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount_with_fee);
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
       return GNUNET_OK;
     }
   }
@@ -1718,15 +1712,14 @@ deposit_cb (void *cls,
   {
     struct TALER_Amount tmp;
 
-    if (GNUNET_SYSERR ==
-        TALER_amount_subtract (&tmp,
-                               &ds->denom_balance,
-                               amount_with_fee))
+    if (TALER_ARL_SR_INVALID_NEGATIVE ==
+        TALER_ARL_amount_subtract_neg (&tmp,
+                                       &ds->denom_balance,
+                                       amount_with_fee))
     {
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&ds->denom_loss,
-                                       &ds->denom_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&ds->denom_loss,
+                            &ds->denom_loss,
+                            amount_with_fee);
       ds->report_emergency = GNUNET_YES;
     }
     else
@@ -1752,10 +1745,9 @@ deposit_cb (void *cls,
     }
     else
     {
-      GNUNET_assert (GNUNET_SYSERR !=
-                     TALER_amount_subtract (&total_escrow_balance,
-                                            &total_escrow_balance,
-                                            amount_with_fee));
+      TALER_ARL_amount_subtract (&total_escrow_balance,
+                                 &total_escrow_balance,
+                                 amount_with_fee);
     }
 
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -1770,12 +1762,12 @@ deposit_cb (void *cls,
 
     TALER_amount_ntoh (&dfee,
                        &issue->fee_deposit);
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_deposit_fee_income,
-                                     &total_deposit_fee_income,
-                                     &dfee));
+    TALER_ARL_amount_add (&total_deposit_fee_income,
+                          &total_deposit_fee_income,
+                          &dfee);
   }
-
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1826,7 +1818,9 @@ refund_cb (void *cls,
     report_row_inconsistency ("refunds",
                               rowid,
                               "denomination key not found");
-    return GNUNET_SYSERR;
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
+    return GNUNET_OK;
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
   {
@@ -1843,14 +1837,13 @@ refund_cb (void *cls,
       .coin_pub = *coin_pub,
       .merchant = *merchant_pub,
       .rtransaction_id = GNUNET_htonll (rtransaction_id),
-      .refund_fee = issue->fee_refund
     };
 
     TALER_amount_hton (&rr.refund_amount,
                        amount_with_fee);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-                                    &rr.purpose,
+                                    &rr,
                                     &merchant_sig->eddsa_sig,
                                     &merchant_pub->eddsa_pub))
     {
@@ -1862,26 +1855,29 @@ refund_cb (void *cls,
                                      amount_with_fee),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount_with_fee));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount_with_fee);
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
       return GNUNET_OK;
     }
   }
 
   TALER_amount_ntoh (&refund_fee,
                      &issue->fee_refund);
-  if (GNUNET_OK !=
-      TALER_amount_subtract (&amount_without_fee,
-                             amount_with_fee,
-                             &refund_fee))
+  if (TALER_ARL_SR_INVALID_NEGATIVE ==
+      TALER_ARL_amount_subtract_neg (&amount_without_fee,
+                                     amount_with_fee,
+                                     &refund_fee))
   {
     report_amount_arithmetic_inconsistency ("refund (fee)",
                                             rowid,
                                             &amount_without_fee,
                                             &refund_fee,
                                             -1);
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
 
@@ -1903,32 +1899,29 @@ refund_cb (void *cls,
   }
   else
   {
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&ds->denom_balance,
-                                     &ds->denom_balance,
-                                     &amount_without_fee));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&ds->denom_risk,
-                                     &ds->denom_risk,
-                                     &amount_without_fee));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_escrow_balance,
-                                     &total_escrow_balance,
-                                     &amount_without_fee));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_risk,
-                                     &total_risk,
-                                     &amount_without_fee));
+    TALER_ARL_amount_add (&ds->denom_balance,
+                          &ds->denom_balance,
+                          &amount_without_fee);
+    TALER_ARL_amount_add (&ds->denom_risk,
+                          &ds->denom_risk,
+                          &amount_without_fee);
+    TALER_ARL_amount_add (&total_escrow_balance,
+                          &total_escrow_balance,
+                          &amount_without_fee);
+    TALER_ARL_amount_add (&total_risk,
+                          &total_risk,
+                          &amount_without_fee);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "New balance of denomination `%s' after refund is %s\n",
                 GNUNET_h2s (&issue->denom_hash),
                 TALER_amount2s (&ds->denom_balance));
   }
   /* update total refund fee balance */
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&total_refund_fee_income,
-                                   &total_refund_fee_income,
-                                   &refund_fee));
+  TALER_ARL_amount_add (&total_refund_fee_income,
+                        &total_refund_fee_income,
+                        &refund_fee);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -1972,10 +1965,9 @@ check_recoup (struct CoinContext *cc,
                                  "loss", TALER_JSON_from_amount (amount),
                                  "coin_pub", GNUNET_JSON_from_data_auto (
                                    &coin->denom_pub_hash)));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_bad_sig_loss,
-                                     &total_bad_sig_loss,
-                                     amount));
+    TALER_ARL_amount_add (&total_bad_sig_loss,
+                          &total_bad_sig_loss,
+                          amount);
   }
   qs = TALER_ARL_get_denomination_info_by_hash (&coin->denom_pub_hash,
                                                 &issue);
@@ -1984,6 +1976,8 @@ check_recoup (struct CoinContext *cc,
     report_row_inconsistency (operation,
                               rowid,
                               "denomination key not found");
+    if (TALER_ARL_do_abort ())
+      return GNUNET_SYSERR;
     return GNUNET_OK;
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
@@ -2017,7 +2011,7 @@ check_recoup (struct CoinContext *cc,
 
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_RECOUP,
-                                    &pr.purpose,
+                                    &pr,
                                     &coin_sig->eddsa_signature,
                                     &coin->coin_pub.eddsa_pub))
     {
@@ -2028,10 +2022,11 @@ check_recoup (struct CoinContext *cc,
                                    "loss", TALER_JSON_from_amount (amount),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      &coin->coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount);
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
       return GNUNET_OK;
     }
   }
@@ -2059,20 +2054,19 @@ check_recoup (struct CoinContext *cc,
                                    "loss", TALER_JSON_from_amount (amount),
                                    "coin_pub", GNUNET_JSON_from_data_auto (
                                      &coin->coin_pub)));
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&total_bad_sig_loss,
-                                       &total_bad_sig_loss,
-                                       amount));
+      TALER_ARL_amount_add (&total_bad_sig_loss,
+                            &total_bad_sig_loss,
+                            amount);
     }
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&ds->denom_recoup,
-                                     &ds->denom_recoup,
-                                     amount));
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_add (&total_recoup_loss,
-                                     &total_recoup_loss,
-                                     amount));
+    TALER_ARL_amount_add (&ds->denom_recoup,
+                          &ds->denom_recoup,
+                          amount);
+    TALER_ARL_amount_add (&total_recoup_loss,
+                          &total_recoup_loss,
+                          amount);
   }
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
   return GNUNET_OK;
 }
 
@@ -2189,10 +2183,9 @@ recoup_refresh_cb (void *cls,
     }
     else
     {
-      GNUNET_assert (GNUNET_OK ==
-                     TALER_amount_add (&dso->denom_balance,
-                                       &dso->denom_balance,
-                                       amount));
+      TALER_ARL_amount_add (&dso->denom_balance,
+                            &dso->denom_balance,
+                            amount);
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "New balance of denomination `%s' after refresh-recoup is %s\n",
                   GNUNET_h2s (&issue->denom_hash),
