@@ -96,8 +96,9 @@ struct DepositConfirmationContext
  * @param cls our `struct DepositConfirmationContext`
  * @param serial_id row of the @a dc in the database
  * @param dc the deposit confirmation we know
+ * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop iterating
  */
-static void
+static int
 test_dc (void *cls,
          uint64_t serial_id,
          const struct TALER_AUDITORDB_DepositConfirmation *dc)
@@ -114,30 +115,37 @@ test_dc (void *cls,
       .h_wire = dc->h_wire,
       .refund_deadline = dc->refund_deadline
     };
+    struct GNUNET_TIME_Absolute exchange_timestamp;
+    struct TALER_Amount deposit_fee;
 
     qs = TALER_ARL_edb->have_deposit (TALER_ARL_edb->cls,
                                       TALER_ARL_esession,
                                       &dep,
-                                      GNUNET_NO /* do not check refund deadline */);
+                                      GNUNET_NO /* do not check refund deadline */,
+                                      &deposit_fee,
+                                      &exchange_timestamp);
     if (qs > 0)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Found deposit %s in exchange database\n",
                   GNUNET_h2s (&dc->h_contract_terms));
-      return; /* found, all good */
+      if (TALER_ARL_do_abort ())
+        return GNUNET_SYSERR;
+      return GNUNET_OK; /* found, all good */
     }
     if (qs < 0)
     {
       GNUNET_break (0); /* DB error, complain */
       dcc->qs = qs;
-      return;
+      return GNUNET_SYSERR;
     }
   }
   /* deposit confirmation missing! report! */
   TALER_ARL_report (report_deposit_confirmation_inconsistencies,
                     json_pack ("{s:o, s:o, s:I, s:o}",
                                "timestamp",
-                               TALER_ARL_json_from_time_abs (dc->timestamp),
+                               TALER_ARL_json_from_time_abs (
+                                 dc->exchange_timestamp),
                                "amount",
                                TALER_JSON_from_amount (&dc->amount_without_fee),
                                "rowid",
@@ -147,10 +155,12 @@ test_dc (void *cls,
   dcc->first_missed_coin_serial = GNUNET_MIN (dcc->first_missed_coin_serial,
                                               serial_id);
   dcc->missed_count++;
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_add (&dcc->missed_amount,
-                                   &dcc->missed_amount,
-                                   &dc->amount_without_fee));
+  TALER_ARL_amount_add (&dcc->missed_amount,
+                        &dcc->missed_amount,
+                        &dc->amount_without_fee);
+  if (TALER_ARL_do_abort ())
+    return GNUNET_SYSERR;
+  return GNUNET_OK;
 }
 
 
