@@ -91,8 +91,8 @@ handle_admin_add_incoming_finished (void *cls,
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_uint64 ("row_id",
                                  &row_id),
-        GNUNET_JSON_spec_absolute_time ("timestamp",
-                                        &timestamp),
+        TALER_JSON_spec_absolute_time ("timestamp",
+                                       &timestamp),
         GNUNET_JSON_spec_end ()
       };
 
@@ -141,7 +141,6 @@ handle_admin_add_incoming_finished (void *cls,
                 (unsigned int) response_code);
     GNUNET_break (0);
     ec = TALER_JSON_get_error_code (j);
-    response_code = 0;
     break;
   }
   aai->cb (aai->cb_cls,
