@@ -338,7 +338,7 @@ run (void *cls,
                  TALER_string_to_amount (withdraw_fee_str,
                                          &withdraw_fee));
   for (unsigned int i = 0; i < howmany_coins; i++)
-    GNUNET_assert (GNUNET_OK ==
+    GNUNET_assert (0 <=
                    TALER_amount_add (&total_reserve_amount,
                                      &total_reserve_amount,
                                      &withdraw_fee));
@@ -599,8 +599,7 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
   if ( (MODE_EXCHANGE == mode) || (MODE_BOTH == mode) )
   {
     /* start exchange */
-    exchanged = GNUNET_OS_start_process (GNUNET_NO,
-                                         GNUNET_OS_INHERIT_STD_ALL,
+    exchanged = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                          NULL, NULL, NULL,
                                          "taler-exchange-httpd",
                                          "taler-exchange-httpd",
@@ -626,8 +625,7 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
       return 77;
     }
     /* start auditor */
-    auditord = GNUNET_OS_start_process (GNUNET_NO,
-                                        GNUNET_OS_INHERIT_STD_ALL,
+    auditord = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                         NULL, NULL, NULL,
                                         "taler-auditor-httpd",
                                         "taler-auditor-httpd",
@@ -658,8 +656,7 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
       return 77;
     }
     /* start exchange wirewatch */
-    wirewatch = GNUNET_OS_start_process (GNUNET_NO,
-                                         GNUNET_OS_INHERIT_STD_ALL,
+    wirewatch = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                          NULL, NULL, NULL,
                                          "taler-exchange-wirewatch",
                                          "taler-exchange-wirewatch",
@@ -708,12 +705,9 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
                 remote_cmd);
 
     GNUNET_assert (NULL != (exchange_slave_pipe =
-                              GNUNET_DISK_pipe (GNUNET_YES,
-                                                GNUNET_YES,
-                                                0, 0)));
+                              GNUNET_DISK_pipe (GNUNET_DISK_PF_BLOCKING_RW)));
 
-    exchange_slave = GNUNET_OS_start_process (GNUNET_NO,
-                                              GNUNET_OS_INHERIT_STD_OUT_AND_ERR,
+    exchange_slave = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_OUT_AND_ERR,
                                               exchange_slave_pipe, NULL, NULL,
                                               "ssh",
                                               "ssh",
@@ -993,7 +987,7 @@ main (int argc,
                                    argc,
                                    argv)))
   {
-    GNUNET_free_non_null (cfg_filename);
+    GNUNET_free (cfg_filename);
     return BAD_CLI_ARG;
   }
   GNUNET_log_setup ("taler-exchange-benchmark",
@@ -1010,7 +1004,7 @@ main (int argc,
   else
   {
     TALER_LOG_ERROR ("Unknown mode given: '%s'\n", mode_str);
-    GNUNET_free_non_null (cfg_filename);
+    GNUNET_free (cfg_filename);
     return BAD_CONFIG_FILE;
   }
   if (NULL == cfg_filename)
@@ -1087,8 +1081,7 @@ main (int argc,
     struct GNUNET_OS_Process *compute_wire_response;
 
     compute_wire_response
-      = GNUNET_OS_start_process (GNUNET_NO,
-                                 GNUNET_OS_INHERIT_STD_ALL,
+      = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                  NULL, NULL, NULL,
                                  "taler-exchange-wire",
                                  "taler-exchange-wire",
