@@ -156,7 +156,7 @@ static struct GNUNET_SCHEDULER_Task *task;
 static struct GNUNET_TIME_Relative aggregator_idle_sleep_interval;
 
 /**
- * Value to return from main(). 0 on success, non-zero on erorrs.
+ * Value to return from main(). 0 on success, non-zero on errors.
  */
 static enum
 {
@@ -193,7 +193,7 @@ run_aggregation (void *cls);
 /**
  * Free data stored in @a au, but not @a au itself (stack allocated).
  *
- * @param au aggreation unit to clean up
+ * @param au aggregation unit to clean up
  */
 static void
 cleanup_au (struct AggregationUnit *au)
@@ -269,7 +269,7 @@ parse_wirewatch_config (void)
          (0 != currency_round_unit.value) ) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Invalid value specified in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
+                "Need non-zero value in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
     return GNUNET_SYSERR;
   }
 
@@ -312,7 +312,7 @@ refund_by_coin_cb (void *cls,
               "Aggregator subtracts applicable refund of amount %s\n",
               TALER_amount2s (amount_with_fee));
   aux->have_refund = GNUNET_YES;
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (&aux->total_amount,
                              &aux->total_amount,
                              amount_with_fee))
@@ -330,12 +330,14 @@ refund_by_coin_cb (void *cls,
  *
  * @param cls a `struct AggregationUnit`
  * @param row_id identifies database entry
+ * @param exchange_timestamp when did the deposit happen
+ * @param wallet_timestamp when did the contract happen
  * @param merchant_pub public key of the merchant
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param wire_deadline by which the merchant adviced that he would like the
+ * @param wire_deadline by which the merchant advised that he would like the
  *        wire transfer to be executed
  * @param wire wire details for the merchant
  * @return transaction status code,  #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT to continue to iterate
@@ -343,6 +345,8 @@ refund_by_coin_cb (void *cls,
 static enum GNUNET_DB_QueryStatus
 deposit_cb (void *cls,
             uint64_t row_id,
+            struct GNUNET_TIME_Absolute exchange_timestamp,
+            struct GNUNET_TIME_Absolute wallet_timestamp,
             const struct TALER_MerchantPublicKeyP *merchant_pub,
             const struct TALER_CoinSpendPublicKeyP *coin_pub,
             const struct TALER_Amount *amount_with_fee,
@@ -358,6 +362,8 @@ deposit_cb (void *cls,
   /* NOTE: potential optimization: use custom SQL API to not
      fetch this one: */
   (void) wire_deadline; /* already checked by SQL query */
+  (void) exchange_timestamp;
+  (void) wallet_timestamp;
   au->merchant_pub = *merchant_pub;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Aggregator processing payment %s with amount %s\n",
@@ -385,7 +391,7 @@ deposit_cb (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Non-refunded transaction, subtracting deposit fee %s\n",
                 TALER_amount2s (deposit_fee));
-    if (GNUNET_SYSERR ==
+    if (0 >
         TALER_amount_subtract (&ntotal,
                                amount_with_fee,
                                deposit_fee))
@@ -501,38 +507,24 @@ deposit_cb (void *cls,
  *
  * @param cls a `struct AggregationUnit`
  * @param row_id identifies database entry
- * @param merchant_pub public key of the merchant
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
  * @param h_contract_terms hash of the proposal data known to merchant and customer
- * @param wire_deadline by which the merchant adviced that he would like the
- *        wire transfer to be executed
- * @param wire wire details for the merchant
  * @return transaction status code
  */
 static enum GNUNET_DB_QueryStatus
 aggregate_cb (void *cls,
               uint64_t row_id,
-              const struct TALER_MerchantPublicKeyP *merchant_pub,
               const struct TALER_CoinSpendPublicKeyP *coin_pub,
               const struct TALER_Amount *amount_with_fee,
               const struct TALER_Amount *deposit_fee,
-              const struct GNUNET_HashCode *h_contract_terms,
-              struct GNUNET_TIME_Absolute wire_deadline,
-              const json_t *wire)
+              const struct GNUNET_HashCode *h_contract_terms)
 {
   struct AggregationUnit *au = cls;
   struct TALER_Amount old;
   enum GNUNET_DB_QueryStatus qs;
 
-  /* NOTE: potential optimization: use custom SQL API to not
-     fetch these: */
-  (void) wire_deadline; /* checked by SQL */
-  (void) wire; /* must match */
-  GNUNET_break (0 == GNUNET_memcmp (&au->merchant_pub,
-                                    merchant_pub));
-
   if (au->rows_offset >= TALER_EXCHANGEDB_MATCHING_DEPOSITS_LIMIT)
   {
     /* Bug: we asked for at most #TALER_EXCHANGEDB_MATCHING_DEPOSITS_LIMIT results! */
@@ -571,7 +563,7 @@ aggregate_cb (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Subtracting deposit fee %s for non-refunded coin\n",
                 TALER_amount2s (deposit_fee));
-    if (GNUNET_SYSERR ==
+    if (0 >
         TALER_amount_subtract (&tmp,
                                &au->total_amount,
                                deposit_fee))
@@ -595,7 +587,7 @@ aggregate_cb (void *cls,
   {
     struct TALER_Amount tmp;
 
-    if (GNUNET_OK !=
+    if (0 >
         TALER_amount_add (&tmp,
                           &au->total_amount,
                           &old))
@@ -782,7 +774,7 @@ run_aggregation (void *cls)
   /* Subtract wire transfer fee and round to the unit supported by the
      wire transfer method; Check if after rounding down, we still have
      an amount to transfer, and if not mark as 'tiny'. */
-  if ( (GNUNET_OK !=
+  if ( (0 >=
         TALER_amount_subtract (&au_active.final_amount,
                                &au_active.total_amount,
                                &au_active.wire_fee)) ||
@@ -1039,10 +1031,10 @@ main (int argc,
                           options,
                           &run, NULL))
   {
-    GNUNET_free ((void *) argv);
+    GNUNET_free_nz ((void *) argv);
     return GR_CMD_LINE_OPTIONS_WRONG;
   }
-  GNUNET_free ((void *) argv);
+  GNUNET_free_nz ((void *) argv);
   return global_ret;
 }
 
