@@ -161,7 +161,7 @@ parse_wirewatch_config (void)
          (0 != currency_round_unit.value) ) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Invalid value specified in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
+                "Need non-zero amount in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
     return GNUNET_SYSERR;
   }
 
@@ -560,10 +560,10 @@ main (int argc,
                           options,
                           &run, NULL))
   {
-    GNUNET_free ((void *) argv);
+    GNUNET_free_nz ((void *) argv);
     return GR_CMD_LINE_OPTIONS_WRONG;
   }
-  GNUNET_free ((void *) argv);
+  GNUNET_free_nz ((void *) argv);
   return global_ret;
 }
 
