@@ -24,6 +24,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
+#include <sched.h>
 #include <pthread.h>
 #include <sys/resource.h>
 #include "taler_mhd_lib.h"
@@ -42,7 +43,7 @@
 #include "taler-exchange-httpd_keystate.h"
 #include "taler-exchange-httpd_wire.h"
 #include "taler_exchangedb_plugin.h"
-
+#include <gnunet/gnunet_mhd_compat.h>
 
 /**
  * Backlog for listen operation on unix domain sockets.
@@ -116,6 +117,13 @@ struct TALER_EXCHANGEDB_Plugin *TEH_plugin;
  */
 static unsigned int connection_timeout = 30;
 
+/**
+ * How many threads to use.
+ * The default value (0) sets the actual number of threads
+ * based on the number of available cores.
+ */
+static unsigned int num_threads = 0;
+
 /**
  * The HTTP Daemon.
  */
@@ -158,7 +166,7 @@ static unsigned long long req_max;
  * @param root uploaded JSON data
  * @return MHD result code
  */
-typedef int
+typedef MHD_RESULT
 (*CoinOpHandler)(struct MHD_Connection *connection,
                  const struct TALER_CoinSpendPublicKeyP *coin_pub,
                  const json_t *root);
@@ -175,7 +183,7 @@ typedef int
  *         reserve public key, the second one should be "withdraw")
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_post_coins (const struct TEH_RequestHandler *rh,
                    struct MHD_Connection *connection,
                    const json_t *root,
@@ -228,7 +236,7 @@ handle_post_coins (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_COINS_INVALID_COIN_PUB,
-                                       "coin public key malformed");
+                                       args[0]);
   }
   for (unsigned int i = 0; NULL != h[i].op; i++)
     if (0 == strcmp (h[i].op,
@@ -238,8 +246,8 @@ handle_post_coins (const struct TEH_RequestHandler *rh,
                            root);
   return TALER_MHD_reply_with_error (connection,
                                      MHD_HTTP_NOT_FOUND,
-                                     TALER_EC_OPERATION_INVALID,
-                                     "requested operation on coin unknown");
+                                     TALER_EC_OPERATION_UNKNOWN,
+                                     args[1]);
 }
 
 
@@ -297,7 +305,7 @@ handle_mhd_completion_callback (void *cls,
  * @param[in,out] upload_data_size number of bytes in @a upload_data
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 proceed_with_handler (const struct TEH_RequestHandler *rh,
                       struct MHD_Connection *connection,
                       const char *url,
@@ -308,7 +316,7 @@ proceed_with_handler (const struct TEH_RequestHandler *rh,
   const char *args[rh->nargs + 1];
   size_t ulen = strlen (url) + 1;
   json_t *root = NULL;
-  int ret;
+  MHD_RESULT ret;
 
   /* We do check for "ulen" here, because we'll later stack-allocate a buffer
      of that size and don't want to enable malicious clients to cause us
@@ -323,7 +331,7 @@ proceed_with_handler (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_URI_TOO_LONG,
                                        TALER_EC_URI_TOO_LONG,
-                                       "The URI given is too long");
+                                       url);
   }
 
   /* All POST endpoints come with a body in JSON format. So we parse
@@ -331,7 +339,7 @@ proceed_with_handler (const struct TEH_RequestHandler *rh,
   if (0 == strcasecmp (rh->method,
                        MHD_HTTP_METHOD_POST))
   {
-    int res;
+    enum GNUNET_GenericReturnValue res;
 
     res = TALER_MHD_parse_post_json (connection,
                                      inner_cls,
@@ -420,6 +428,45 @@ proceed_with_handler (const struct TEH_RequestHandler *rh,
 }
 
 
+/**
+ * Handle a "/seed" request.
+ *
+ * @param rh context of the handler
+ * @param connection the MHD connection to handle
+ * @param args array of additional options (must be empty for this function)
+ * @return MHD result code
+ */
+static MHD_RESULT
+handler_seed (const struct TEH_RequestHandler *rh,
+              struct MHD_Connection *connection,
+              const char *const args[])
+{
+#define SEED_SIZE 32
+  char *body;
+  MHD_RESULT ret;
+  struct MHD_Response *resp;
+
+  (void) rh;
+  body = malloc (SEED_SIZE); /* must use malloc(), because MHD will use free() */
+  if (NULL == body)
+    return MHD_NO;
+  GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
+                              body,
+                              SEED_SIZE);
+  resp = MHD_create_response_from_buffer (SEED_SIZE,
+                                          body,
+                                          MHD_RESPMEM_MUST_FREE);
+  TALER_MHD_add_global_headers (resp);
+  ret = MHD_queue_response (connection,
+                            MHD_HTTP_OK,
+                            resp);
+  GNUNET_break (MHD_YES == ret);
+  MHD_destroy_response (resp);
+  return ret;
+#undef SEED_SIZE
+}
+
+
 /**
  * Handle incoming HTTP request.
  *
@@ -433,7 +480,7 @@ proceed_with_handler (const struct TEH_RequestHandler *rh,
  * @param con_cls closure for request (a `struct Buffer *`)
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_mhd_request (void *cls,
                     struct MHD_Connection *connection,
                     const char *url,
@@ -472,6 +519,11 @@ handle_mhd_request (void *cls,
       .method = MHD_HTTP_METHOD_GET,
       .handler.get = &TEH_handler_agpl_redirect
     },
+    {
+      .url = "seed",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler.get = &handler_seed
+    },
     /* Terms of service */
     {
       .url = "terms",
@@ -606,7 +658,7 @@ handle_mhd_request (void *cls,
   /* on repeated requests, check our cache first */
   if (NULL != ecls->rh)
   {
-    int ret;
+    MHD_RESULT ret;
     const char *start;
 
     if ('\0' == url[0])
@@ -654,9 +706,10 @@ handle_mhd_request (void *cls,
     {
       struct TEH_RequestHandler *rh = &handlers[i];
 
-      if (0 != strncmp (tok,
-                        rh->url,
-                        tok_size))
+      if ( (0 != strncmp (tok,
+                          rh->url,
+                          tok_size)) ||
+           (tok_size != strlen (rh->url) ) )
         continue;
       found = GNUNET_YES;
       /* The URL is a match!  What we now do depends on the method. */
@@ -669,7 +722,7 @@ handle_mhd_request (void *cls,
       if (0 == strcasecmp (method,
                            rh->method))
       {
-        int ret;
+        MHD_RESULT ret;
 
         /* cache to avoid the loop next time */
         ecls->rh = rh;
@@ -692,18 +745,18 @@ handle_mhd_request (void *cls,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_METHOD_NOT_ALLOWED,
                                          TALER_EC_METHOD_INVALID,
-                                         "The HTTP method used is invalid for this URL");
+                                         method);
     }
   }
 
   /* No handler matches, generate not found */
   {
-    int ret;
+    MHD_RESULT ret;
 
     ret = TALER_MHD_reply_with_error (connection,
                                       MHD_HTTP_NOT_FOUND,
                                       TALER_EC_ENDPOINT_UNKNOWN,
-                                      "No handler found for the given URL");
+                                      url);
     GNUNET_async_scope_restore (&old_scope);
     return ret;
   }
@@ -716,7 +769,7 @@ handle_mhd_request (void *cls,
  *
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 exchange_serve_process_config (void)
 {
   if (GNUNET_OK !=
@@ -782,8 +835,8 @@ exchange_serve_process_config (void)
                                                     &TEH_master_public_key.
                                                     eddsa_pub))
     {
-      fprintf (stderr,
-               "Invalid master public key given in exchange configuration.");
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Invalid master public key given in exchange configuration.");
       GNUNET_free (master_public_key_str);
       return GNUNET_SYSERR;
     }
@@ -795,14 +848,18 @@ exchange_serve_process_config (void)
 
   if (GNUNET_OK !=
       TEH_WIRE_init (TEH_cfg))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to setup wire subsystem\n");
     return GNUNET_SYSERR;
+  }
 
 
   if (NULL ==
       (TEH_plugin = TALER_EXCHANGEDB_plugin_load (TEH_cfg)))
   {
-    fprintf (stderr,
-             "Failed to initialize DB subsystem\n");
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to initialize DB subsystem\n");
     TEH_WIRE_done ();
     return GNUNET_SYSERR;
   }
@@ -814,6 +871,8 @@ exchange_serve_process_config (void)
                               &serve_unixpath,
                               &unixpath_mode))
   {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to setup HTTPd subsystem\n");
     TEH_WIRE_done ();
     return GNUNET_SYSERR;
   }
@@ -1086,6 +1145,8 @@ run_main_loop (int fh,
 {
   int ret;
 
+  GNUNET_assert (0 < num_threads);
+
   mhd
     = MHD_start_daemon (MHD_USE_SELECT_INTERNALLY | MHD_USE_PIPE_FOR_SHUTDOWN
                         | MHD_USE_DEBUG | MHD_USE_DUAL_STACK
@@ -1094,7 +1155,7 @@ run_main_loop (int fh,
                         (-1 == fh) ? serve_port : 0,
                         NULL, NULL,
                         &handle_mhd_request, NULL,
-                        MHD_OPTION_THREAD_POOL_SIZE, (unsigned int) 32,
+                        MHD_OPTION_THREAD_POOL_SIZE, (unsigned int) num_threads,
                         MHD_OPTION_LISTEN_BACKLOG_SIZE, (unsigned int) 1024,
                         MHD_OPTION_LISTEN_SOCKET, fh,
                         MHD_OPTION_EXTERNAL_LOGGER, &TALER_MHD_handle_logs,
@@ -1216,6 +1277,11 @@ main (int argc,
                                &connection_timeout),
     GNUNET_GETOPT_option_timetravel ('T',
                                      "timetravel"),
+    GNUNET_GETOPT_option_uint ('n',
+                               "num-threads",
+                               "NUM_THREADS",
+                               "size of the thread pool",
+                               &num_threads),
 #if HAVE_DEVELOPER
     GNUNET_GETOPT_option_filename ('f',
                                    "file-input",
@@ -1241,6 +1307,15 @@ main (int argc,
                          options,
                          argc, argv))
     return 1;
+  if (0 == num_threads)
+  {
+    cpu_set_t mask;
+    GNUNET_assert (0 ==
+                   sched_getaffinity (0,
+                                      sizeof (cpu_set_t),
+                                      &mask));
+    num_threads = CPU_COUNT (&mask);
+  }
   go = TALER_MHD_GO_NONE;
   if (connection_close)
     go |= TALER_MHD_GO_FORCE_CONNECTION_CLOSE;
@@ -1249,8 +1324,8 @@ main (int argc,
                  GNUNET_log_setup ("taler-exchange-httpd",
                                    (NULL == loglev) ? "INFO" : loglev,
                                    logfile));
-  GNUNET_free_non_null (loglev);
-  GNUNET_free_non_null (logfile);
+  GNUNET_free (loglev);
+  GNUNET_free (logfile);
   if (NULL == cfgfile)
     cfgfile = GNUNET_strdup (GNUNET_OS_project_data_get ()->user_config_file);
   TEH_cfg = GNUNET_CONFIGURATION_create ();
@@ -1261,10 +1336,10 @@ main (int argc,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Malformed configuration file `%s', exit ...\n",
                 cfgfile);
-    GNUNET_free_non_null (cfgfile);
+    GNUNET_free (cfgfile);
     return 1;
   }
-  GNUNET_free_non_null (cfgfile);
+  GNUNET_free (cfgfile);
   if (GNUNET_OK !=
       exchange_serve_process_config ())
     return 1;
