@@ -41,42 +41,74 @@
 
 
 /**
- * Execute database transaction to ensure coin is known. Run the transaction
- * logic; IF it returns a non-error code, the transaction logic MUST
- * NOT queue a MHD response.  IF it returns an hard error, the
- * transaction logic MUST queue a MHD response and set @a mhd_ret.  IF
- * it returns the soft error code, the function MAY be called again to
- * retry and MUST not queue a MHD response.
+ * Ensure coin is known in the database, and handle conflicts and errors.
  *
- * @param cls a `struct TEH_DB_KnowCoinContext`
- * @param connection MHD request context, must not be NULL
+ * @param coin the coin to make known
+ * @param connection MHD request context
  * @param session database session and transaction to use
- * @param[out] mhd_ret set to MHD status on error, must not be NULL
- * @return transaction status
+ * @param[out] mhd_ret set to MHD status on error
+ * @return transaction status, negative on error (@a mhd_ret will be set in this case)
  */
 enum GNUNET_DB_QueryStatus
-TEH_DB_know_coin_transaction (void *cls,
-                              struct MHD_Connection *connection,
-                              struct TALER_EXCHANGEDB_Session *session,
-                              int *mhd_ret)
+TEH_make_coin_known (const struct TALER_CoinPublicInfo *coin,
+                     struct MHD_Connection *connection,
+                     struct TALER_EXCHANGEDB_Session *session,
+                     MHD_RESULT *mhd_ret)
 {
-  struct TEH_DB_KnowCoinContext *kcc = cls;
-  enum GNUNET_DB_QueryStatus qs;
+  enum TALER_EXCHANGEDB_CoinKnownStatus cks;
 
-  GNUNET_assert (NULL != mhd_ret);
-  qs = TEH_plugin->ensure_coin_known (TEH_plugin->cls,
-                                      session,
-                                      kcc->coin);
-  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+  /* make sure coin is 'known' in database */
+  cks = TEH_plugin->ensure_coin_known (TEH_plugin->cls,
+                                       session,
+                                       coin);
+  switch (cks)
   {
+  case TALER_EXCHANGEDB_CKS_ADDED:
+    return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
+  case TALER_EXCHANGEDB_CKS_PRESENT:
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  case TALER_EXCHANGEDB_CKS_SOFT_FAIL:
+    return GNUNET_DB_STATUS_SOFT_ERROR;
+  case TALER_EXCHANGEDB_CKS_HARD_FAIL:
     *mhd_ret
       = TALER_MHD_reply_with_error (connection,
                                     MHD_HTTP_INTERNAL_SERVER_ERROR,
                                     TALER_EC_DB_COIN_HISTORY_STORE_ERROR,
-                                    "could not persist coin data");
+                                    NULL);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  case TALER_EXCHANGEDB_CKS_CONFLICT:
+    break;
+  }
+
+  {
+    struct TALER_EXCHANGEDB_TransactionList *tl;
+    enum GNUNET_DB_QueryStatus qs;
+
+    qs = TEH_plugin->get_coin_transactions (TEH_plugin->cls,
+                                            session,
+                                            &coin->coin_pub,
+                                            GNUNET_NO,
+                                            &tl);
+    if (0 > qs)
+    {
+      if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+        *mhd_ret = TALER_MHD_reply_with_error (
+          connection,
+          MHD_HTTP_INTERNAL_SERVER_ERROR,
+          TALER_EC_DEPOSIT_HISTORY_DB_ERROR,
+          NULL);
+      return qs;
+    }
+    *mhd_ret
+      = TEH_RESPONSE_reply_coin_insufficient_funds (
+          connection,
+          TALER_EC_COIN_CONFLICTING_DENOMINATION_KEY,
+          &coin->coin_pub,
+          tl);
+    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                            tl);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
-  return qs;
 }
 
 
@@ -96,10 +128,10 @@ TEH_DB_know_coin_transaction (void *cls,
  * @param cb_cls closure for @a cb, must be read-only!
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
  */
-int
+enum GNUNET_GenericReturnValue
 TEH_DB_run_transaction (struct MHD_Connection *connection,
                         const char *name,
-                        int *mhd_ret,
+                        MHD_RESULT *mhd_ret,
                         TEH_DB_TransactionCallback cb,
                         void *cb_cls)
 {
@@ -114,7 +146,7 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_DB_SETUP_FAILED,
-                                             "could not establish database session");
+                                             NULL);
     return GNUNET_SYSERR;
   }
   for (unsigned int retries = 0;
@@ -133,7 +165,7 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_DB_START_FAILED,
-                                               "could not begin transaction");
+                                               NULL);
       return GNUNET_SYSERR;
     }
     qs = cb (cb_cls,
@@ -154,7 +186,7 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_DB_COMMIT_FAILED_HARD,
-                                               "could not commit database transaction");
+                                               NULL);
       return GNUNET_SYSERR;
     }
     /* make sure callback did not violate invariants! */
@@ -170,7 +202,7 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_DB_COMMIT_FAILED_ON_RETRY,
-                                           "repatedly failed to serialize database transaction");
+                                           NULL);
   return GNUNET_SYSERR;
 }
 
