@@ -47,18 +47,18 @@
  * @param coin_pub public key of the coin
  * @param h_wire hash of wire details
  * @param h_contract_terms hash of contract details
- * @param timestamp client's timestamp
+ * @param exchange_timestamp exchange's timestamp
  * @param refund_deadline until when this deposit be refunded
  * @param merchant merchant public key
  * @param amount_without_fee fraction of coin value to deposit, without the fee
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 reply_deposit_success (struct MHD_Connection *connection,
                        const struct TALER_CoinSpendPublicKeyP *coin_pub,
                        const struct GNUNET_HashCode *h_wire,
                        const struct GNUNET_HashCode *h_contract_terms,
-                       struct GNUNET_TIME_Absolute timestamp,
+                       struct GNUNET_TIME_Absolute exchange_timestamp,
                        struct GNUNET_TIME_Absolute refund_deadline,
                        const struct TALER_MerchantPublicKeyP *merchant,
                        const struct TALER_Amount *amount_without_fee)
@@ -70,7 +70,7 @@ reply_deposit_success (struct MHD_Connection *connection,
     .purpose.size = htonl (sizeof (dc)),
     .h_contract_terms = *h_contract_terms,
     .h_wire = *h_wire,
-    .timestamp = GNUNET_TIME_absolute_hton (timestamp),
+    .exchange_timestamp = GNUNET_TIME_absolute_hton (exchange_timestamp),
     .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
     .coin_pub = *coin_pub,
     .merchant = *merchant
@@ -79,7 +79,7 @@ reply_deposit_success (struct MHD_Connection *connection,
   TALER_amount_hton (&dc.amount_without_fee,
                      amount_without_fee);
   if (GNUNET_OK !=
-      TEH_KS_sign (&dc.purpose,
+      TEH_KS_sign (&dc,
                    &pub,
                    &sig))
   {
@@ -88,12 +88,16 @@ reply_deposit_success (struct MHD_Connection *connection,
                                        TALER_EC_EXCHANGE_BAD_CONFIGURATION,
                                        "no keys");
   }
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_OK,
-                                    "{s:s, s:o, s:o}",
-                                    "status", "DEPOSIT_OK",
-                                    "sig", GNUNET_JSON_from_data_auto (&sig),
-                                    "pub", GNUNET_JSON_from_data_auto (&pub));
+  return TALER_MHD_reply_json_pack (
+    connection,
+    MHD_HTTP_OK,
+    "{s:o, s:o, s:o}",
+    "exchange_timestamp",
+    GNUNET_JSON_from_time_abs (exchange_timestamp),
+    "exchange_sig",
+    GNUNET_JSON_from_data_auto (&sig),
+    "exchange_pub",
+    GNUNET_JSON_from_data_auto (&pub));
 }
 
 
@@ -107,6 +111,11 @@ struct DepositContext
    */
   const struct TALER_EXCHANGEDB_Deposit *deposit;
 
+  /**
+   * Our timestamp (when we received the request).
+   */
+  struct GNUNET_TIME_Absolute exchange_timestamp;
+
   /**
    * Value of the coin.
    */
@@ -115,6 +124,74 @@ struct DepositContext
 };
 
 
+/**
+ * Check if /deposit is already in the database.  IF it returns a non-error
+ * code, the transaction logic MUST NOT queue a MHD response.  IF it returns
+ * an hard error, the transaction logic MUST queue a MHD response and set @a
+ * mhd_ret.  We do return a "hard" error also if we found the deposit in the
+ * database and generated a regular response.
+ *
+ * @param cls a `struct DepositContext`
+ * @param connection MHD request context
+ * @param session database session and transaction to use
+ * @param[out] mhd_ret set to MHD status on error
+ * @return transaction status
+ */
+static enum GNUNET_DB_QueryStatus
+deposit_precheck (void *cls,
+                  struct MHD_Connection *connection,
+                  struct TALER_EXCHANGEDB_Session *session,
+                  MHD_RESULT *mhd_ret)
+{
+  struct DepositContext *dc = cls;
+  const struct TALER_EXCHANGEDB_Deposit *deposit = dc->deposit;
+  struct TALER_Amount deposit_fee;
+  enum GNUNET_DB_QueryStatus qs;
+
+  qs = TEH_plugin->have_deposit (TEH_plugin->cls,
+                                 session,
+                                 deposit,
+                                 GNUNET_YES /* check refund deadline */,
+                                 &deposit_fee,
+                                 &dc->exchange_timestamp);
+  if (qs < 0)
+  {
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                             TALER_EC_DEPOSIT_HISTORY_DB_ERROR,
+                                             NULL);
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+    return qs;
+  }
+  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
+  {
+    struct TALER_Amount amount_without_fee;
+
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "/deposit replay, accepting again!\n");
+    GNUNET_assert (0 <=
+                   TALER_amount_subtract (&amount_without_fee,
+                                          &deposit->amount_with_fee,
+                                          &deposit_fee));
+    *mhd_ret = reply_deposit_success (connection,
+                                      &deposit->coin.coin_pub,
+                                      &deposit->h_wire,
+                                      &deposit->h_contract_terms,
+                                      dc->exchange_timestamp,
+                                      deposit->refund_deadline,
+                                      &deposit->merchant_pub,
+                                      &amount_without_fee);
+    /* Treat as 'hard' DB error as we want to rollback and
+       never try again. */
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+  return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+}
+
+
 /**
  * Execute database transaction for /deposit.  Runs the transaction
  * logic; IF it returns a non-error code, the transaction logic MUST
@@ -133,51 +210,30 @@ static enum GNUNET_DB_QueryStatus
 deposit_transaction (void *cls,
                      struct MHD_Connection *connection,
                      struct TALER_EXCHANGEDB_Session *session,
-                     int *mhd_ret)
+                     MHD_RESULT *mhd_ret)
 {
   struct DepositContext *dc = cls;
   const struct TALER_EXCHANGEDB_Deposit *deposit = dc->deposit;
   struct TALER_Amount spent;
   enum GNUNET_DB_QueryStatus qs;
 
-  qs = TEH_plugin->have_deposit (TEH_plugin->cls,
-                                 session,
-                                 deposit,
-                                 GNUNET_YES /* check refund deadline */);
+  /* make sure coin is 'known' in database */
+  qs = TEH_make_coin_known (&deposit->coin,
+                            connection,
+                            session,
+                            mhd_ret);
   if (qs < 0)
-  {
-    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-    {
-      *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                             TALER_EC_DEPOSIT_HISTORY_DB_ERROR,
-                                             "Could not check for existing identical deposit");
-      return GNUNET_DB_STATUS_HARD_ERROR;
-    }
     return qs;
-  }
-  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
-  {
-    struct TALER_Amount amount_without_fee;
 
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "/deposit replay, accepting again!\n");
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_subtract (&amount_without_fee,
-                                          &deposit->amount_with_fee,
-                                          &deposit->deposit_fee));
-    *mhd_ret = reply_deposit_success (connection,
-                                      &deposit->coin.coin_pub,
-                                      &deposit->h_wire,
-                                      &deposit->h_contract_terms,
-                                      deposit->timestamp,
-                                      deposit->refund_deadline,
-                                      &deposit->merchant_pub,
-                                      &amount_without_fee);
-    /* Treat as 'hard' DB error as we want to rollback and
-       never try again. */
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
+  /* Theoretically, someone other threat may have received
+     and committed the deposit in the meantime. Check now
+     that we are in the transaction scope. */
+  qs = deposit_precheck (cls,
+                         connection,
+                         session,
+                         mhd_ret);
+  if (qs < 0)
+    return qs;
 
   /* Start with fee for THIS transaction */
   spent = deposit->amount_with_fee;
@@ -199,7 +255,7 @@ deposit_transaction (void *cls,
           connection,
           MHD_HTTP_INTERNAL_SERVER_ERROR,
           TALER_EC_DEPOSIT_HISTORY_DB_ERROR,
-          "could not fetch coin transaction history");
+          NULL);
       return qs;
     }
     if (GNUNET_OK !=
@@ -213,7 +269,7 @@ deposit_transaction (void *cls,
         connection,
         MHD_HTTP_INTERNAL_SERVER_ERROR,
         TALER_EC_DEPOSIT_HISTORY_DB_ERROR,
-        "could not calculate historic coin amount total");
+        NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
     /* Check that cost of all transactions (including the current one) is
@@ -237,6 +293,7 @@ deposit_transaction (void *cls,
   }
   qs = TEH_plugin->insert_deposit (TEH_plugin->cls,
                                    session,
+                                   dc->exchange_timestamp,
                                    deposit);
   if (GNUNET_DB_STATUS_HARD_ERROR == qs)
   {
@@ -244,51 +301,12 @@ deposit_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_DEPOSIT_STORE_DB_ERROR,
-                                           "Could not persist /deposit data");
+                                           NULL);
   }
   return qs;
 }
 
 
-/**
- * Check that @a ts is reasonably close to our own RTC.
- *
- * @param ts timestamp to check
- * @return #GNUNET_OK if @a ts is reasonable
- */
-static int
-check_timestamp_current (struct GNUNET_TIME_Absolute ts)
-{
-  struct GNUNET_TIME_Relative r;
-  struct GNUNET_TIME_Relative tolerance;
-
-  /* Let's be VERY generous (after all, this is basically about
-     which year the deposit counts for in terms of tax purposes) */
-  tolerance = GNUNET_TIME_UNIT_MONTHS;
-  r = GNUNET_TIME_absolute_get_duration (ts);
-  if (r.rel_value_us > tolerance.rel_value_us)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Deposit timestamp too old: %llu vs %llu > %llu\n",
-                (unsigned long long) ts.abs_value_us,
-                (unsigned long long) GNUNET_TIME_absolute_get ().abs_value_us,
-                (unsigned long long) tolerance.rel_value_us);
-    return GNUNET_SYSERR;
-  }
-  r = GNUNET_TIME_absolute_get_remaining (ts);
-  if (r.rel_value_us > tolerance.rel_value_us)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Deposit timestamp too new: %llu vs %llu < - %llu\n",
-                (unsigned long long) ts.abs_value_us,
-                (unsigned long long) GNUNET_TIME_absolute_get ().abs_value_us,
-                (unsigned long long) tolerance.rel_value_us);
-    return GNUNET_SYSERR;
-  }
-  return GNUNET_OK;
-}
-
-
 /**
  * Handle a "/coins/$COIN_PUB/deposit" request.  Parses the JSON, and, if
  * successful, passes the JSON data to #deposit_transaction() to
@@ -301,7 +319,7 @@ check_timestamp_current (struct GNUNET_TIME_Absolute ts)
  * @param root uploaded JSON data
  * @return MHD result code
   */
-int
+MHD_RESULT
 TEH_handler_deposit (struct MHD_Connection *connection,
                      const struct TALER_CoinSpendPublicKeyP *coin_pub,
                      const json_t *root)
@@ -312,19 +330,26 @@ TEH_handler_deposit (struct MHD_Connection *connection,
   struct GNUNET_HashCode my_h_wire;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_json ("wire", &wire),
-    TALER_JSON_spec_amount ("contribution", &deposit.amount_with_fee),
+    TALER_JSON_spec_amount ("contribution",
+                            &deposit.amount_with_fee),
     GNUNET_JSON_spec_fixed_auto ("denom_pub_hash",
                                  &deposit.coin.denom_pub_hash),
-    TALER_JSON_spec_denomination_signature ("ub_sig", &deposit.coin.denom_sig),
-    GNUNET_JSON_spec_fixed_auto ("merchant_pub", &deposit.merchant_pub),
-    GNUNET_JSON_spec_fixed_auto ("h_contract_terms", &deposit.h_contract_terms),
-    GNUNET_JSON_spec_fixed_auto ("h_wire", &deposit.h_wire),
-    GNUNET_JSON_spec_fixed_auto ("coin_sig",  &deposit.csig),
-    GNUNET_JSON_spec_absolute_time ("timestamp", &deposit.timestamp),
-    GNUNET_JSON_spec_absolute_time ("refund_deadline",
-                                    &deposit.refund_deadline),
-    GNUNET_JSON_spec_absolute_time ("wire_transfer_deadline",
-                                    &deposit.wire_deadline),
+    TALER_JSON_spec_denomination_signature ("ub_sig",
+                                            &deposit.coin.denom_sig),
+    GNUNET_JSON_spec_fixed_auto ("merchant_pub",
+                                 &deposit.merchant_pub),
+    GNUNET_JSON_spec_fixed_auto ("h_contract_terms",
+                                 &deposit.h_contract_terms),
+    GNUNET_JSON_spec_fixed_auto ("h_wire",
+                                 &deposit.h_wire),
+    GNUNET_JSON_spec_fixed_auto ("coin_sig",
+                                 &deposit.csig),
+    TALER_JSON_spec_absolute_time ("timestamp",
+                                   &deposit.timestamp),
+    TALER_JSON_spec_absolute_time ("refund_deadline",
+                                   &deposit.refund_deadline),
+    TALER_JSON_spec_absolute_time ("wire_transfer_deadline",
+                                   &deposit.wire_deadline),
     GNUNET_JSON_spec_end ()
   };
 
@@ -357,18 +382,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSIT_REFUND_DEADLINE_AFTER_WIRE_DEADLINE,
-                                       "refund_deadline");
-  }
-
-  if (GNUNET_OK !=
-      check_timestamp_current (deposit.timestamp))
-  {
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_DEPOSIT_INVALID_TIMESTAMP,
-                                       "timestamp");
+                                       NULL);
   }
   if (GNUNET_OK !=
       TALER_JSON_merchant_wire_signature_hash (wire,
@@ -380,7 +394,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_JSON,
-                                       "wire");
+                                       NULL);
   }
   if (0 != GNUNET_memcmp (&deposit.h_wire,
                           &my_h_wire))
@@ -390,9 +404,29 @@ TEH_handler_deposit (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_CONTRACT_HASH_CONFLICT,
-                                       "h_wire");
+                                       NULL);
   }
 
+  /* Check for idempotency: did we get this request before? */
+  dc.deposit = &deposit;
+  {
+    MHD_RESULT mhd_ret;
+
+    if (GNUNET_OK !=
+        TEH_DB_run_transaction (connection,
+                                "precheck deposit",
+                                &mhd_ret,
+                                &deposit_precheck,
+                                &dc))
+    {
+      GNUNET_JSON_parse_free (spec);
+      return mhd_ret;
+    }
+  }
+
+  /* new deposit */
+  dc.exchange_timestamp = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&dc.exchange_timestamp);
   /* check denomination exists and is valid */
   {
     struct TEH_KS_StateHandle *key_state;
@@ -400,7 +434,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
     enum TALER_ErrorCode ec;
     unsigned int hc;
 
-    key_state = TEH_KS_acquire (GNUNET_TIME_absolute_get ());
+    key_state = TEH_KS_acquire (dc.exchange_timestamp);
     if (NULL == key_state)
     {
       TALER_LOG_ERROR ("Lacking keys to operate\n");
@@ -423,7 +457,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          hc,
                                          ec,
-                                         "Could not find denomination key used in deposit");
+                                         NULL);
     }
     TALER_amount_ntoh (&deposit.deposit_fee,
                        &dki->issue.properties.fee_deposit);
@@ -436,8 +470,8 @@ TEH_handler_deposit (struct MHD_Connection *connection,
       GNUNET_JSON_parse_free (spec);
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_DEPOSIT_CURRENCY_MISSMATCH,
-                                         "contribution");
+                                         TALER_EC_DEPOSIT_CURRENCY_MISMATCH,
+                                         deposit.deposit_fee.currency);
     }
     /* check coin signature */
     if (GNUNET_YES !=
@@ -450,10 +484,8 @@ TEH_handler_deposit (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_UNAUTHORIZED,
                                          TALER_EC_DEPOSIT_DENOMINATION_SIGNATURE_INVALID,
-                                         "ub_sig");
+                                         NULL);
     }
-    TALER_amount_ntoh (&deposit.deposit_fee,
-                       &dki->issue.properties.fee_deposit);
     TALER_amount_ntoh (&dc.value,
                        &dki->issue.properties.value);
     TEH_KS_release (key_state);
@@ -466,27 +498,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSIT_NEGATIVE_VALUE_AFTER_FEE,
-                                       "deposited amount smaller than depositing fee");
-  }
-
-  /* make sure coin is 'known' in database */
-  {
-    int mhd_ret;
-    struct TEH_DB_KnowCoinContext kcc = {
-      .coin = &deposit.coin,
-      .connection = connection
-    };
-
-    if (GNUNET_OK !=
-        TEH_DB_run_transaction (connection,
-                                "know coin for deposit",
-                                &mhd_ret,
-                                &TEH_DB_know_coin_transaction,
-                                &kcc))
-    {
-      GNUNET_JSON_parse_free (spec);
-      return mhd_ret;
-    }
+                                       NULL);
   }
 
   /* check deposit signature */
@@ -496,7 +508,8 @@ TEH_handler_deposit (struct MHD_Connection *connection,
       .purpose.size = htonl (sizeof (dr)),
       .h_contract_terms = deposit.h_contract_terms,
       .h_wire = deposit.h_wire,
-      .timestamp = GNUNET_TIME_absolute_hton (deposit.timestamp),
+      .h_denom_pub = deposit.coin.denom_pub_hash,
+      .wallet_timestamp = GNUNET_TIME_absolute_hton (deposit.timestamp),
       .refund_deadline = GNUNET_TIME_absolute_hton (deposit.refund_deadline),
       .merchant = deposit.merchant_pub,
       .coin_pub = deposit.coin.coin_pub
@@ -508,7 +521,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
                        &deposit.deposit_fee);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
-                                    &dr.purpose,
+                                    &dr,
                                     &deposit.csig.eddsa_signature,
                                     &deposit.coin.coin_pub.eddsa_pub))
     {
@@ -517,14 +530,13 @@ TEH_handler_deposit (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_UNAUTHORIZED,
                                          TALER_EC_DEPOSIT_COIN_SIGNATURE_INVALID,
-                                         "coin_sig");
+                                         NULL);
     }
   }
 
   /* execute transaction */
-  dc.deposit = &deposit;
   {
-    int mhd_ret;
+    MHD_RESULT mhd_ret;
 
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
@@ -541,9 +553,9 @@ TEH_handler_deposit (struct MHD_Connection *connection,
   /* generate regular response */
   {
     struct TALER_Amount amount_without_fee;
-    int res;
+    MHD_RESULT res;
 
-    GNUNET_assert (GNUNET_SYSERR !=
+    GNUNET_assert (0 <=
                    TALER_amount_subtract (&amount_without_fee,
                                           &deposit.amount_with_fee,
                                           &deposit.deposit_fee));
@@ -551,7 +563,7 @@ TEH_handler_deposit (struct MHD_Connection *connection,
                                  &deposit.coin.coin_pub,
                                  &deposit.h_wire,
                                  &deposit.h_contract_terms,
-                                 deposit.timestamp,
+                                 dc.exchange_timestamp,
                                  deposit.refund_deadline,
                                  &deposit.merchant_pub,
                                  &amount_without_fee);
