@@ -45,7 +45,7 @@
  * @param exec_time execution time of the wire transfer
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 reply_deposit_details (struct MHD_Connection *connection,
                        const struct GNUNET_HashCode *h_contract_terms,
                        const struct GNUNET_HashCode *h_wire,
@@ -69,7 +69,7 @@ reply_deposit_details (struct MHD_Connection *connection,
   TALER_amount_hton (&cw.coin_contribution,
                      coin_contribution);
   if (GNUNET_OK !=
-      TEH_KS_sign (&cw.purpose,
+      TEH_KS_sign (&cw,
                    &pub,
                    &sig))
   {
@@ -176,7 +176,7 @@ handle_wtid_data (void *cls,
     ctx->execution_time = execution_time;
     return;
   }
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (&ctx->coin_delta,
                              coin_contribution,
                              coin_fee))
@@ -213,7 +213,7 @@ static enum GNUNET_DB_QueryStatus
 deposits_get_transaction (void *cls,
                           struct MHD_Connection *connection,
                           struct TALER_EXCHANGEDB_Session *session,
-                          int *mhd_ret)
+                          MHD_RESULT *mhd_ret)
 {
   struct DepositWtidContext *ctx = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -234,7 +234,7 @@ deposits_get_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_DEPOSITS_GET_DB_FETCH_FAILED,
-                                             "failed to fetch transaction data");
+                                             NULL);
     }
     return qs;
   }
@@ -243,7 +243,7 @@ deposits_get_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_DEPOSITS_GET_NOT_FOUND,
-                                           "transaction unknown");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   return qs;
@@ -258,13 +258,13 @@ deposits_get_transaction (void *cls,
  * @param merchant_pub public key from the merchant
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_track_transaction_request (
   struct MHD_Connection *connection,
   const struct TALER_DepositTrackPS *tps,
   const struct TALER_MerchantPublicKeyP *merchant_pub)
 {
-  int mhd_ret;
+  MHD_RESULT mhd_ret;
   struct DepositWtidContext ctx = {
     .pending = GNUNET_NO,
     .tps = tps,
@@ -289,7 +289,7 @@ handle_track_transaction_request (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_DEPOSITS_GET_DB_FEE_INCONSISTENT,
-                                       "fees are inconsistent");
+                                       NULL);
   return reply_deposit_details (connection,
                                 &tps->h_contract_terms,
                                 &tps->h_wire,
@@ -310,12 +310,12 @@ handle_track_transaction_request (
  *      h_wire, merchant_pub, h_contract_terms and coin_pub)
  * @return MHD result code
   */
-int
+MHD_RESULT
 TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
                           struct MHD_Connection *connection,
                           const char *const args[4])
 {
-  int res;
+  enum GNUNET_GenericReturnValue res;
   struct TALER_MerchantSignatureP merchant_sig;
   struct TALER_DepositTrackPS tps = {
     .purpose.size = htonl (sizeof (tps)),
@@ -333,7 +333,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSITS_INVALID_H_WIRE,
-                                       "wire hash malformed");
+                                       args[0]);
   }
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (args[1],
@@ -345,7 +345,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSITS_INVALID_MERCHANT_PUB,
-                                       "merchant public key malformed");
+                                       args[1]);
   }
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (args[2],
@@ -357,7 +357,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSITS_INVALID_H_CONTRACT_TERMS,
-                                       "contract terms hash malformed");
+                                       args[2]);
   }
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (args[3],
@@ -369,7 +369,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_DEPOSITS_INVALID_COIN_PUB,
-                                       "coin public key malformed");
+                                       args[3]);
   }
   res = TALER_MHD_parse_request_arg_data (connection,
                                           "merchant_sig",
@@ -381,7 +381,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return MHD_YES; /* parse error */
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_TRACK_TRANSACTION,
-                                  &tps.purpose,
+                                  &tps,
                                   &merchant_sig.eddsa_sig,
                                   &tps.merchant.eddsa_pub))
   {
@@ -389,7 +389,7 @@ TEH_handler_deposits_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_FORBIDDEN,
                                        TALER_EC_DEPOSITS_GET_MERCHANT_SIGNATURE_INVALID,
-                                       "merchant_sig");
+                                       NULL);
   }
 
   return handle_track_transaction_request (connection,
