@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014--2019 Taler Systems SA
+  Copyright (C) 2014--2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -44,7 +44,7 @@
  * #TALER_PROTOCOL_CURRENT and #TALER_PROTOCOL_AGE in
  * exchange_api_handle.c!
  */
-#define EXCHANGE_PROTOCOL_VERSION "7:0:0"
+#define EXCHANGE_PROTOCOL_VERSION "8:0:0"
 
 
 /**
@@ -476,7 +476,7 @@ handle_signal (int signal_number)
                   1);
   /* While one might like to "handle errors" here, even logging via fprintf()
      isn't safe inside of a signal handler. So there is nothing we safely CAN
-     do. OTOH, also very little that can go wrong in pratice. Calling _exit()
+     do. OTOH, also very little that can go wrong in practice. Calling _exit()
      on errors might be a possibility, but that might do more harm than good. *///
 }
 
@@ -489,7 +489,7 @@ handle_signal (int signal_number)
  *
  * @param pk public key of the denomination
  * @param dki the denomination key issue information
- * @return a JSON object describing the denomination key isue (public part)
+ * @return a JSON object describing the denomination key issue (public part)
  */
 static json_t *
 denom_key_issue_to_json (
@@ -560,7 +560,7 @@ store_in_map (struct GNUNET_CONTAINER_MultiHashMap *map,
               const struct TALER_EXCHANGEDB_DenominationKey *dki)
 {
   /* First, we verify that the @a dki is actually well-formed.  While it comes
-     from our own hard disk, there is the possibility of missconfiguration
+     from our own hard disk, there is the possibility of misconfiguration
      (i.e. bogus file in the directory), or that the administrator used the
      wrong master public key, and we should not accept entries that are not
      well-formed. *///
@@ -593,7 +593,7 @@ store_in_map (struct GNUNET_CONTAINER_MultiHashMap *map,
     if (GNUNET_SYSERR ==
         GNUNET_CRYPTO_eddsa_verify (
           TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY,
-          &denom_key_issue.purpose,
+          &denom_key_issue,
           &dkip->signature.eddsa_signature,
           &TEH_master_public_key.eddsa_pub))
     {
@@ -669,7 +669,7 @@ static enum GNUNET_DB_QueryStatus
 add_revocations_transaction (void *cls,
                              struct MHD_Connection *connection,
                              struct TALER_EXCHANGEDB_Session *session,
-                             int *mhd_ret)
+                             MHD_RESULT *mhd_ret)
 {
   struct AddRevocationContext *arc = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -709,7 +709,7 @@ static enum GNUNET_DB_QueryStatus
 add_denomination_transaction (void *cls,
                               struct MHD_Connection *connection,
                               struct TALER_EXCHANGEDB_Session *session,
-                              int *mhd_ret)
+                              MHD_RESULT *mhd_ret)
 {
   const struct TALER_EXCHANGEDB_DenominationKey *dki = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -801,7 +801,7 @@ reload_keys_denom_iter (void *cls,
     handle_signal (SIGTERM);
     return GNUNET_SYSERR;
   }
-
+  GNUNET_assert (NULL != dki->denom_priv.rsa_private_key);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Adding denomination key `%s' (%s) to active set\n",
               alias,
@@ -1489,12 +1489,10 @@ build_keys_response (const struct ResponseFactoryContext *rfc,
   GNUNET_CRYPTO_hash_context_finish (rbc.hash_context,
                                      &ks.hc);
   rbc.hash_context = NULL;
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (
-                   &rfc->key_state->current_sign_key_issue.signkey_priv.
-                   eddsa_priv,
-                   &ks.purpose,
-                   &sig.eddsa_signature));
+  GNUNET_CRYPTO_eddsa_sign (
+    &rfc->key_state->current_sign_key_issue.signkey_priv.eddsa_priv,
+    &ks,
+    &sig.eddsa_signature);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (TEH_cfg,
                                            "exchangedb",
@@ -1662,6 +1660,9 @@ reload_public_denoms_cb (
                 GNUNET_h2s (&issue->properties.denom_hash));
     return;
   }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Loaded denomination key %s from DB, no private key (hopefully revoked!)\n",
+              GNUNET_h2s (&issue->properties.denom_hash));
   /* we can assert here as we checked for duplicates just above */
   GNUNET_assert (GNUNET_OK == ret);
 }
@@ -2016,11 +2017,13 @@ TEH_KS_denomination_key_lookup_by_hash (
   struct GNUNET_TIME_Absolute now;
   const struct GNUNET_CONTAINER_MultiHashMap *map;
 
-  map = (TEH_KS_DKU_RECOUP == use) ? key_state->revoked_map :
-        key_state->denomkey_map;
+  map = (TEH_KS_DKU_RECOUP == use)
+        ? key_state->revoked_map
+        : key_state->denomkey_map;
   dki = GNUNET_CONTAINER_multihashmap_get (map,
                                            denom_pub_hash);
-  if ( (NULL == dki) && (TEH_KS_DKU_ZOMBIE == use))
+  if ( (NULL == dki) &&
+       (TEH_KS_DKU_ZOMBIE == use) )
     dki = GNUNET_CONTAINER_multihashmap_get (key_state->revoked_map,
                                              denom_pub_hash);
   if (NULL == dki)
@@ -2399,16 +2402,20 @@ TEH_KS_free (void)
 /**
  * Sign the message in @a purpose with the exchange's signing key.
  *
+ * The @a purpose data is the beginning of the data of which the signature is
+ * to be created. The `size` field in @a purpose must correctly indicate the
+ * number of bytes of the data structure, including its header.  Use
+ * #TEH_KS_sign() instead of calling this function directly!
+ *
  * @param purpose the message to sign
  * @param[out] pub set to the current public signing key of the exchange
  * @param[out] sig signature over purpose using current signing key
  * @return #GNUNET_OK on success, #GNUNET_SYSERR if we lack key material
  */
 int
-TEH_KS_sign (const struct GNUNET_CRYPTO_EccSignaturePurpose *purpose,
-             struct TALER_ExchangePublicKeyP *pub,
-             struct TALER_ExchangeSignatureP *sig)
-
+TEH_KS_sign_ (const struct GNUNET_CRYPTO_EccSignaturePurpose *purpose,
+              struct TALER_ExchangePublicKeyP *pub,
+              struct TALER_ExchangeSignatureP *sig)
 {
   struct TEH_KS_StateHandle *key_state;
 
@@ -2424,7 +2431,7 @@ TEH_KS_sign (const struct GNUNET_CRYPTO_EccSignaturePurpose *purpose,
   }
   *pub = key_state->current_sign_key_issue.issue.signkey_pub;
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (
+                 GNUNET_CRYPTO_eddsa_sign_ (
                    &key_state->current_sign_key_issue.signkey_priv.eddsa_priv,
                    purpose,
                    &sig->eddsa_signature));
@@ -2465,12 +2472,12 @@ krd_search_comparator (const void *key,
  * @param args array of additional options (must be empty for this function)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_keys (const struct TEH_RequestHandler *rh,
                   struct MHD_Connection *connection,
                   const char *const args[])
 {
-  int ret;
+  MHD_RESULT ret;
   const char *have_cherrypick;
   const char *have_fakenow;
   struct GNUNET_TIME_Absolute last_issue_date;
@@ -2495,7 +2502,7 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
                                          TALER_EC_KEYS_HAVE_NOT_NUMERIC,
-                                         "last_issue_date");
+                                         have_cherrypick);
     }
     /* The following multiplication may overflow; but this should not really
        be a problem, as giving back 'older' data than what the client asks for
@@ -2524,7 +2531,7 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_KEYS_HAVE_NOT_NUMERIC,
-                                         "now");
+                                         have_fakenow);
     }
     if (TEH_allow_keys_timetravel)
     {
@@ -2540,7 +2547,7 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
                                          TALER_EC_KEYS_TIMETRAVEL_FORBIDDEN,
-                                         "timetravel not allowed by this exchange");
+                                         NULL);
     }
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -2554,13 +2561,13 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
     if (NULL == key_state)
     {
       /* Maybe client picked time stamp too far in the future?  In that case,
-         #MHD_HTTP_INTERNAL_SERVER_ERROR might be missleading, could be more like a
+         #MHD_HTTP_INTERNAL_SERVER_ERROR might be misleading, could be more like a
          NOT_FOUND situation. But, OTOH, for 'sane' clients it is more likely
          to be our fault, so let's speculatively assume we are to blame ;-) */
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_EXCHANGE_BAD_CONFIGURATION,
-                                         "no keys for requested time");
+                                         NULL);
     }
     krd = bsearch (&last_issue_date,
                    key_state->krd_array,
@@ -2584,7 +2591,7 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
     if (NULL == krd)
     {
       /* Maybe client picked time stamp too far in the future?  In that case,
-         "INTERNAL_SERVER_ERROR" might be missleading, could be more like a
+         "INTERNAL_SERVER_ERROR" might be misleading, could be more like a
          NOT_FOUND situation. But, OTOH, for 'sane' clients it is more likely
          to be our fault, so let's speculatively assume we are to blame ;-) *///
       GNUNET_break (0);
@@ -2592,7 +2599,7 @@ TEH_handler_keys (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_KEYS_MISSING,
-                                         "no key response found");
+                                         NULL);
     }
     ret = MHD_queue_response (connection,
                               MHD_HTTP_OK,
