@@ -47,7 +47,7 @@
  * @param residual remaining value of the coin (after subtracting @a tl)
  * @return a MHD result code
  */
-static int
+static MHD_RESULT
 reply_melt_insufficient_funds (
   struct MHD_Connection *connection,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -64,25 +64,25 @@ reply_melt_insufficient_funds (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_MELT_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS,
-                                       "Failed to compile transaction history");
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_CONFLICT,
-                                    "{s:s, s:I, s:o, s:o, s:o, s:o, s:o}",
-                                    "hint",
-                                    "insufficient funds",
-                                    "code",
-                                    (json_int_t)
-                                    TALER_EC_MELT_INSUFFICIENT_FUNDS,
-                                    "coin_pub",
-                                    GNUNET_JSON_from_data_auto (coin_pub),
-                                    "original_value",
-                                    TALER_JSON_from_amount (coin_value),
-                                    "residual_value",
-                                    TALER_JSON_from_amount (residual),
-                                    "requested_value",
-                                    TALER_JSON_from_amount (requested),
-                                    "history",
-                                    history);
+                                       NULL);
+  return TALER_MHD_reply_json_pack (
+    connection,
+    MHD_HTTP_CONFLICT,
+    "{s:s, s:I, s:o, s:o, s:o, s:o, s:o}",
+    "hint",
+    TALER_ErrorCode_get_hint (TALER_EC_MELT_INSUFFICIENT_FUNDS),
+    "code",
+    (json_int_t) TALER_EC_MELT_INSUFFICIENT_FUNDS,
+    "coin_pub",
+    GNUNET_JSON_from_data_auto (coin_pub),
+    "original_value",
+    TALER_JSON_from_amount (coin_value),
+    "residual_value",
+    TALER_JSON_from_amount (residual),
+    "requested_value",
+    TALER_JSON_from_amount (requested),
+    "history",
+    history);
 }
 
 
@@ -90,11 +90,11 @@ reply_melt_insufficient_funds (
  * Send a response to a "melt" request.
  *
  * @param connection the connection to send the response to
- * @param rc value the client commited to
+ * @param rc value the client committed to
  * @param noreveal_index which index will the client not have to reveal
  * @return a MHD status code
  */
-static int
+static MHD_RESULT
 reply_melt_success (struct MHD_Connection *connection,
                     const struct TALER_RefreshCommitmentP *rc,
                     uint32_t noreveal_index)
@@ -109,7 +109,7 @@ reply_melt_success (struct MHD_Connection *connection,
   };
 
   if (GNUNET_OK !=
-      TEH_KS_sign (&body.purpose,
+      TEH_KS_sign (&body,
                    &pub,
                    &sig))
   {
@@ -151,11 +151,17 @@ struct MeltContext
   struct TALER_Amount coin_refresh_fee;
 
   /**
-   * Set to #GNUNET_YES if this coin's denomination was revoked and the operation
+   * Set to true if this coin's denomination was revoked and the operation
    * is thus only allowed for zombie coins where the transaction
    * history includes a #TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP.
    */
-  int zombie_required;
+  bool zombie_required;
+
+  /**
+   * We already checked and noticed that the coin is known. Hence we
+   * can skip the "ensure_coin_known" step of the transaction.
+   */
+  bool coin_is_dirty;
 
 };
 
@@ -174,7 +180,7 @@ static enum GNUNET_DB_QueryStatus
 refresh_check_melt (struct MHD_Connection *connection,
                     struct TALER_EXCHANGEDB_Session *session,
                     struct MeltContext *rmc,
-                    int *mhd_ret)
+                    MHD_RESULT *mhd_ret)
 {
   struct TALER_EXCHANGEDB_TransactionList *tl;
   struct TALER_Amount spent;
@@ -196,7 +202,7 @@ refresh_check_melt (struct MHD_Connection *connection,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_MELT_DB_FETCH_ERROR,
-                                             "failed to fetch old coin history");
+                                             NULL);
     return qs;
   }
   if (rmc->zombie_required)
@@ -210,11 +216,11 @@ refresh_check_melt (struct MHD_Connection *connection,
     {
       if (TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP == tp->type)
       {
-        rmc->zombie_required = GNUNET_NO; /* clear flag: was satisfied! */
+        rmc->zombie_required = false; /* clear flag: was satisfied! */
         break;
       }
     }
-    if (GNUNET_YES == rmc->zombie_required)
+    if (rmc->zombie_required)
     {
       /* zombie status not satisfied */
       GNUNET_break_op (0);
@@ -223,7 +229,7 @@ refresh_check_melt (struct MHD_Connection *connection,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_BAD_REQUEST,
                                              TALER_EC_MELT_COIN_EXPIRED_NO_ZOMBIE,
-                                             "denomination expired");
+                                             NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
   }
@@ -238,7 +244,7 @@ refresh_check_melt (struct MHD_Connection *connection,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_MELT_COIN_HISTORY_COMPUTATION_FAILED,
-                                           "failed to compute coin transaction history");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
@@ -248,11 +254,20 @@ refresh_check_melt (struct MHD_Connection *connection,
                             &spent))
   {
     struct TALER_Amount coin_residual;
+    struct TALER_Amount spent_already;
 
-    GNUNET_assert (GNUNET_SYSERR !=
-                   TALER_amount_subtract (&coin_residual,
+    /* First subtract the melt cost from 'spent' to
+       compute the total amount already spent of the coin */
+    GNUNET_assert (0 <=
+                   TALER_amount_subtract (&spent_already,
                                           &spent,
                                           &rmc->refresh_session.amount_with_fee));
+    /* The residual coin value is the original coin value minus
+       what we have spent (before the melt) */
+    GNUNET_assert (0 <=
+                   TALER_amount_subtract (&coin_residual,
+                                          &rmc->coin_value,
+                                          &spent_already));
     *mhd_ret = reply_melt_insufficient_funds (
       connection,
       &rmc->refresh_session.coin.coin_pub,
@@ -296,12 +311,23 @@ static enum GNUNET_DB_QueryStatus
 melt_transaction (void *cls,
                   struct MHD_Connection *connection,
                   struct TALER_EXCHANGEDB_Session *session,
-                  int *mhd_ret)
+                  MHD_RESULT *mhd_ret)
 {
   struct MeltContext *rmc = cls;
   enum GNUNET_DB_QueryStatus qs;
   uint32_t noreveal_index;
 
+  /* First, make sure coin is 'known' in database */
+  if (! rmc->coin_is_dirty)
+  {
+    qs = TEH_make_coin_known (&rmc->refresh_session.coin,
+                              connection,
+                              session,
+                              mhd_ret);
+    if (qs < 0)
+      return qs;
+  }
+
   /* Check if we already created a matching refresh_session */
   qs = TEH_plugin->get_melt_index (TEH_plugin->cls,
                                    session,
@@ -324,7 +350,7 @@ melt_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_MELT_DB_FETCH_ERROR,
-                                             "failed to fetch melt index");
+                                             NULL);
     return qs;
   }
 
@@ -350,7 +376,7 @@ melt_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_MELT_DB_STORE_SESSION_ERROR,
-                                             "failed to persist melt data");
+                                             NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
     return qs;
@@ -370,28 +396,29 @@ melt_transaction (void *cls,
  * @param[in,out] rmc details about the melt request
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 handle_melt (struct MHD_Connection *connection,
              struct MeltContext *rmc)
 {
   /* verify signature of coin for melt operation */
   {
-    struct TALER_RefreshMeltCoinAffirmationPS body;
+    struct TALER_RefreshMeltCoinAffirmationPS body = {
+      .purpose.size = htonl (sizeof (body)),
+      .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT),
+      .rc = rmc->refresh_session.rc,
+      .h_denom_pub = rmc->refresh_session.coin.denom_pub_hash,
+      .coin_pub = rmc->refresh_session.coin.coin_pub
+    };
 
-    body.purpose.size = htonl (sizeof (struct
-                                       TALER_RefreshMeltCoinAffirmationPS));
-    body.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT);
-    body.rc = rmc->refresh_session.rc;
     TALER_amount_hton (&body.amount_with_fee,
                        &rmc->refresh_session.amount_with_fee);
     TALER_amount_hton (&body.melt_fee,
                        &rmc->coin_refresh_fee);
-    body.coin_pub = rmc->refresh_session.coin.coin_pub;
 
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (
           TALER_SIGNATURE_WALLET_COIN_MELT,
-          &body.purpose,
+          &body,
           &rmc->refresh_session.coin_sig.eddsa_signature,
           &rmc->refresh_session.coin.coin_pub.eddsa_pub))
     {
@@ -399,13 +426,13 @@ handle_melt (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_MELT_COIN_SIGNATURE_INVALID,
-                                         "confirm_sig");
+                                         NULL);
     }
   }
 
   /* run database transaction */
   {
-    int mhd_ret;
+    MHD_RESULT mhd_ret;
 
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
@@ -431,12 +458,11 @@ handle_melt (struct MHD_Connection *connection,
  * @param rmc parsed request information
  * @return MHD status code
  */
-static int
+static MHD_RESULT
 check_for_denomination_key (struct MHD_Connection *connection,
                             struct MeltContext *rmc)
 {
   struct TEH_KS_StateHandle *key_state;
-  int coin_is_dirty = GNUNET_NO;
 
   key_state = TEH_KS_acquire (GNUNET_TIME_absolute_get ());
   if (NULL == key_state)
@@ -492,7 +518,7 @@ check_for_denomination_key (struct MHD_Connection *connection,
           return TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_MELT_DB_FETCH_ERROR,
-                                             "failed to find information about old coin");
+                                             NULL);
         }
         /* sanity check */
         GNUNET_break (0 ==
@@ -506,8 +532,8 @@ check_for_denomination_key (struct MHD_Connection *connection,
         else
         {
           /* Minor optimization: no need to run the
-             #TEH_DB_know_coin_transaction below */
-          coin_is_dirty = GNUNET_YES;
+             "ensure_coin_known" part of the transaction */
+          rmc->coin_is_dirty = true;
         }
       }
     }
@@ -524,7 +550,7 @@ check_for_denomination_key (struct MHD_Connection *connection,
                                                     &ec,
                                                     &hc);
       if (NULL != dki)
-        rmc->zombie_required = GNUNET_YES; /* check later that zombie is satisfied */
+        rmc->zombie_required = true; /* check later that zombie is satisfied */
     }
     if (NULL == dki)
     {
@@ -532,7 +558,7 @@ check_for_denomination_key (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          hc,
                                          ec,
-                                         "unknown denomination");
+                                         NULL);
     }
 
     TALER_amount_ntoh (&rmc->coin_refresh_fee,
@@ -548,8 +574,9 @@ check_for_denomination_key (struct MHD_Connection *connection,
       TEH_KS_release (key_state);
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_MELT_CURRENCY_MISSMATCH,
-                                         "value_with_fee");
+                                         TALER_EC_MELT_CURRENCY_MISMATCH,
+                                         rmc->refresh_session.amount_with_fee.
+                                         currency);
 
     }
     /* check coin is actually properly signed */
@@ -562,29 +589,11 @@ check_for_denomination_key (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_MELT_DENOMINATION_SIGNATURE_INVALID,
-                                         "denom_sig");
+                                         NULL);
     }
   }
   TEH_KS_release (key_state);
 
-  /* run actual logic, now that the request was parsed */
-  /* First, make sure coin is 'known' in database */
-  if (GNUNET_NO == coin_is_dirty)
-  {
-    struct TEH_DB_KnowCoinContext kcc;
-    int mhd_ret;
-
-    kcc.coin = &rmc->refresh_session.coin;
-    kcc.connection = connection;
-    if (GNUNET_OK !=
-        TEH_DB_run_transaction (connection,
-                                "know coin for melt",
-                                &mhd_ret,
-                                &TEH_DB_know_coin_transaction,
-                                &kcc))
-      return mhd_ret;
-  }
-
   /* sanity-check that "total melt amount > melt fee" */
   if (0 <
       TALER_amount_cmp (&rmc->coin_refresh_fee,
@@ -594,7 +603,7 @@ check_for_denomination_key (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_MELT_FEES_EXCEED_CONTRIBUTION,
-                                       "melt amount smaller than melting fee");
+                                       NULL);
   }
   return handle_melt (connection,
                       rmc);
@@ -612,13 +621,14 @@ check_for_denomination_key (struct MHD_Connection *connection,
  * @param root uploaded JSON data
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_melt (struct MHD_Connection *connection,
                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
                   const json_t *root)
 {
   struct MeltContext rmc;
-  int res;
+  enum GNUNET_GenericReturnValue ret;
+  MHD_RESULT res;
   struct GNUNET_JSON_Specification spec[] = {
     TALER_JSON_spec_denomination_signature ("denom_sig",
                                             &rmc.refresh_session.coin.denom_sig),
@@ -637,11 +647,11 @@ TEH_handler_melt (struct MHD_Connection *connection,
           0,
           sizeof (rmc));
   rmc.refresh_session.coin.coin_pub = *coin_pub;
-  res = TALER_MHD_parse_json_data (connection,
+  ret = TALER_MHD_parse_json_data (connection,
                                    root,
                                    spec);
-  if (GNUNET_OK != res)
-    return (GNUNET_SYSERR == res) ? MHD_NO : MHD_YES;
+  if (GNUNET_OK != ret)
+    return (GNUNET_SYSERR == ret) ? MHD_NO : MHD_YES;
 
   res = check_for_denomination_key (connection,
                                     &rmc);
