@@ -43,7 +43,7 @@
  * @param args array of additional options (must be empty for this function)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_static_response (const struct TEH_RequestHandler *rh,
                              struct MHD_Connection *connection,
                              const char *const args[])
@@ -71,7 +71,7 @@ TEH_handler_static_response (const struct TEH_RequestHandler *rh,
  * @param args array of additional options (must be empty for this function)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_agpl_redirect (const struct TEH_RequestHandler *rh,
                            struct MHD_Connection *connection,
                            const char *const args[])
