@@ -123,7 +123,7 @@ static enum GNUNET_DB_QueryStatus
 recoup_transaction (void *cls,
                     struct MHD_Connection *connection,
                     struct TALER_EXCHANGEDB_Session *session,
-                    int *mhd_ret)
+                    MHD_RESULT *mhd_ret)
 {
   struct RecoupContext *pc = cls;
   struct TALER_EXCHANGEDB_TransactionList *tl;
@@ -132,6 +132,14 @@ recoup_transaction (void *cls,
   enum GNUNET_DB_QueryStatus qs;
   int existing_recoup_found;
 
+  /* make sure coin is 'known' in database */
+  qs = TEH_make_coin_known (pc->coin,
+                            connection,
+                            session,
+                            mhd_ret);
+  if (qs < 0)
+    return qs;
+
   /* Check whether a recoup is allowed, and if so, to which
      reserve / account the money should go */
   if (pc->refreshed)
@@ -148,7 +156,7 @@ recoup_transaction (void *cls,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_RECOUP_DB_FETCH_FAILED,
-                                               "failed to fetch old coin of blind coin");
+                                               NULL);
       }
       return qs;
     }
@@ -167,7 +175,7 @@ recoup_transaction (void *cls,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_RECOUP_DB_FETCH_FAILED,
-                                               "failed to fetch reserve of blinded coin");
+                                               NULL);
       }
       return qs;
     }
@@ -180,7 +188,7 @@ recoup_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_RECOUP_WITHDRAW_NOT_FOUND,
-                                           "envelope unknown");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
@@ -198,7 +206,7 @@ recoup_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_RECOUP_DB_FETCH_FAILED,
-                                             "failed to fetch coin transaction history");
+                                             NULL);
     }
     return qs;
   }
@@ -233,7 +241,7 @@ recoup_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_RECOUP_HISTORY_DB_ERROR,
-                                           "failed to calculate old coin transaction history");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -242,7 +250,7 @@ recoup_transaction (void *cls,
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Recoup: coin value %s\n",
               TALER_amount2s (&pc->value));
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (&pc->amount,
                              &pc->value,
                              &spent))
@@ -253,14 +261,14 @@ recoup_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_RECOUP_COIN_BALANCE_NEGATIVE,
-                                           "calculated negative coin balance");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   if ( (0 == pc->amount.fraction) &&
        (0 == pc->amount.value) )
   {
     /* Recoup has no effect: coin fully spent! */
-    int ret;
+    enum GNUNET_DB_QueryStatus ret;
 
     TEH_plugin->rollback (TEH_plugin->cls,
                           session);
@@ -320,7 +328,7 @@ recoup_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_RECOUP_DB_PUT_FAILED,
-                                             "failed to persist recoup data");
+                                             NULL);
     }
     return qs;
   }
@@ -342,7 +350,7 @@ recoup_transaction (void *cls,
  * @param refreshed #GNUNET_YES if the coin was refreshed
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 verify_and_execute_recoup (struct MHD_Connection *connection,
                            const struct TALER_CoinPublicInfo *coin,
                            const struct
@@ -384,7 +392,7 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          hc,
                                          ec,
-                                         "denomination not valid for recoup");
+                                         NULL);
     }
     TALER_amount_ntoh (&pc.value,
                        &dki->issue.properties.value);
@@ -399,7 +407,7 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_RECOUP_DENOMINATION_SIGNATURE_INVALID,
-                                         "denom_sig");
+                                         NULL);
     }
 
     /* check recoup request signature */
@@ -414,7 +422,7 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
 
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_RECOUP,
-                                      &pr.purpose,
+                                      &pr,
                                       &coin_sig->eddsa_signature,
                                       &coin->coin_pub.eddsa_pub))
       {
@@ -423,7 +431,7 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_FORBIDDEN,
                                            TALER_EC_RECOUP_SIGNATURE_INVALID,
-                                           "coin_sig");
+                                           NULL);
       }
     }
     GNUNET_CRYPTO_hash (&coin->coin_pub.eddsa_pub,
@@ -441,7 +449,7 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_RECOUP_BLINDING_FAILED,
-                                         "coin_bks");
+                                         NULL);
     }
     TEH_KS_release (key_state);
   }
@@ -450,29 +458,13 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
                       &pc.h_blind);
   GNUNET_free (coin_ev);
 
-  /* make sure coin is 'known' in database */
-  {
-    struct TEH_DB_KnowCoinContext kcc;
-    int mhd_ret;
-
-    kcc.coin = coin;
-    kcc.connection = connection;
-    if (GNUNET_OK !=
-        TEH_DB_run_transaction (connection,
-                                "know coin for recoup",
-                                &mhd_ret,
-                                &TEH_DB_know_coin_transaction,
-                                &kcc))
-      return mhd_ret;
-  }
-
   /* Perform actual recoup transaction */
   pc.coin_sig = coin_sig;
   pc.coin_bks = coin_bks;
   pc.coin = coin;
   pc.refreshed = refreshed;
   {
-    int mhd_ret;
+    MHD_RESULT mhd_ret;
 
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
@@ -512,12 +504,12 @@ verify_and_execute_recoup (struct MHD_Connection *connection,
  * @param root uploaded JSON data
  * @return MHD result code
   */
-int
+MHD_RESULT
 TEH_handler_recoup (struct MHD_Connection *connection,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const json_t *root)
 {
-  int res;
+  enum GNUNET_GenericReturnValue ret;
   struct TALER_CoinPublicInfo coin;
   struct TALER_DenominationBlindingKeyP coin_bks;
   struct TALER_CoinSpendSignatureP coin_sig;
@@ -538,20 +530,24 @@ TEH_handler_recoup (struct MHD_Connection *connection,
   };
 
   coin.coin_pub = *coin_pub;
-  res = TALER_MHD_parse_json_data (connection,
+  ret = TALER_MHD_parse_json_data (connection,
                                    root,
                                    spec);
-  if (GNUNET_SYSERR == res)
+  if (GNUNET_SYSERR == ret)
     return MHD_NO; /* hard failure */
-  if (GNUNET_NO == res)
+  if (GNUNET_NO == ret)
     return MHD_YES; /* failure */
-  res = verify_and_execute_recoup (connection,
-                                   &coin,
-                                   &coin_bks,
-                                   &coin_sig,
-                                   refreshed);
-  GNUNET_JSON_parse_free (spec);
-  return res;
+  {
+    MHD_RESULT res;
+
+    res = verify_and_execute_recoup (connection,
+                                     &coin,
+                                     &coin_bks,
+                                     &coin_sig,
+                                     refreshed);
+    GNUNET_JSON_parse_free (spec);
+    return res;
+  }
 }
 
 
