@@ -52,7 +52,7 @@
  * @param sigs array of @a num_freshcoins signatures revealed
  * @return a MHD result code
  */
-static int
+static MHD_RESULT
 reply_refreshes_reveal_success (struct MHD_Connection *connection,
                                 unsigned int num_freshcoins,
                                 const struct TALER_DenominationSignature *sigs)
@@ -115,7 +115,7 @@ struct RevealContext
 {
 
   /**
-   * Commitment of the refresh operaton.
+   * Commitment of the refresh operation.
    */
   struct TALER_RefreshCommitmentP rc;
 
@@ -235,7 +235,7 @@ static enum GNUNET_DB_QueryStatus
 refreshes_reveal_preflight (void *cls,
                             struct MHD_Connection *connection,
                             struct TALER_EXCHANGEDB_Session *session,
-                            int *mhd_ret)
+                            MHD_RESULT *mhd_ret)
 {
   struct RevealContext *rctx = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -257,7 +257,7 @@ refreshes_reveal_preflight (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_REVEAL_DB_FETCH_REVEAL_ERROR,
-                                           "failed to fetch reveal data");
+                                           NULL);
     rctx->preflight_ok = GNUNET_SYSERR;
     return GNUNET_DB_STATUS_HARD_ERROR;
   case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
@@ -292,7 +292,7 @@ static enum GNUNET_DB_QueryStatus
 refreshes_reveal_transaction (void *cls,
                               struct MHD_Connection *connection,
                               struct TALER_EXCHANGEDB_Session *session,
-                              int *mhd_ret)
+                              MHD_RESULT *mhd_ret)
 {
   struct RevealContext *rctx = cls;
   struct TALER_EXCHANGEDB_Melt melt;
@@ -309,7 +309,7 @@ refreshes_reveal_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_REVEAL_SESSION_UNKNOWN,
-                                           "rc");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
@@ -321,7 +321,7 @@ refreshes_reveal_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_REVEAL_DB_FETCH_SESSION_ERROR,
-                                           "failed to fetch valid challenge from database");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
@@ -365,6 +365,7 @@ refreshes_reveal_transaction (void *cls,
           struct TALER_RefreshCoinData *rcd = &rce->new_coins[j];
           struct TALER_PlanchetSecretsP ps;
           struct TALER_PlanchetDetail pd;
+          struct GNUNET_HashCode c_hash;
 
           rcd->dk = &rctx->dkis[j]->denom_pub;
           TALER_planchet_setup_refresh (&ts,
@@ -373,6 +374,7 @@ refreshes_reveal_transaction (void *cls,
           GNUNET_assert (GNUNET_OK ==
                          TALER_planchet_prepare (rcd->dk,
                                                  &ps,
+                                                 &c_hash,
                                                  &pd));
           rcd->coin_ev = pd.coin_ev;
           rcd->coin_ev_size = pd.coin_ev_size;
@@ -407,16 +409,17 @@ refreshes_reveal_transaction (void *cls,
                             &rc_expected))
     {
       GNUNET_break_op (0);
-      *mhd_ret = TALER_MHD_reply_json_pack (connection,
-                                            MHD_HTTP_CONFLICT,
-                                            "{s:s, s:I, s:o}",
-                                            "hint", "commitment violation",
-                                            "code",
-                                            (json_int_t)
-                                            TALER_EC_REVEAL_COMMITMENT_VIOLATION,
-                                            "rc_expected",
-                                            GNUNET_JSON_from_data_auto (
-                                              &rc_expected));
+      *mhd_ret = TALER_MHD_reply_json_pack (
+        connection,
+        MHD_HTTP_CONFLICT,
+        "{s:s, s:I, s:o}",
+        "hint",
+        TALER_ErrorCode_get_hint (TALER_EC_REVEAL_COMMITMENT_VIOLATION),
+        "code",
+        (json_int_t) TALER_EC_REVEAL_COMMITMENT_VIOLATION,
+        "rc_expected",
+        GNUNET_JSON_from_data_auto (
+          &rc_expected));
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
   } /* end of checking "rc_expected" */
@@ -436,11 +439,11 @@ refreshes_reveal_transaction (void *cls,
                          &rctx->dkis[i]->issue.properties.fee_withdraw);
       TALER_amount_ntoh (&value,
                          &rctx->dkis[i]->issue.properties.value);
-      if ( (GNUNET_OK !=
+      if ( (0 >
             TALER_amount_add (&total,
                               &fee_withdraw,
                               &value)) ||
-           (GNUNET_OK !=
+           (0 >
             TALER_amount_add (&refresh_cost,
                               &refresh_cost,
                               &total)) )
@@ -449,7 +452,7 @@ refreshes_reveal_transaction (void *cls,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_REVEAL_COST_CALCULATION_OVERFLOW,
-                                               "failed to add up refresh costs");
+                                               NULL);
         return GNUNET_DB_STATUS_HARD_ERROR;
       }
     }
@@ -460,7 +463,7 @@ refreshes_reveal_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_REVEAL_AMOUNT_INSUFFICIENT,
-                                             "melted coin value is insufficient to cover cost of operation");
+                                             NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
   }
@@ -482,7 +485,7 @@ static enum GNUNET_DB_QueryStatus
 refreshes_reveal_persist (void *cls,
                           struct MHD_Connection *connection,
                           struct TALER_EXCHANGEDB_Session *session,
-                          int *mhd_ret)
+                          MHD_RESULT *mhd_ret)
 {
   struct RevealContext *rctx = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -515,7 +518,7 @@ refreshes_reveal_persist (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_REVEAL_DB_COMMIT_ERROR,
-                                           "failed to persist reveal data");
+                                           NULL);
   }
   return qs;
 }
@@ -532,7 +535,7 @@ refreshes_reveal_persist (void *cls,
  * @param coin_evs envelopes of gamma-selected coins to be signed
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
                                         struct MHD_Connection *connection,
                                         struct RevealContext *rctx,
@@ -547,7 +550,8 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
   struct TALER_RefreshCoinData rcds[num_fresh_coins];
   struct TALER_CoinSpendSignatureP link_sigs[num_fresh_coins];
   struct TALER_EXCHANGEDB_Melt melt;
-  int res;
+  enum GNUNET_GenericReturnValue res;
+  MHD_RESULT ret;
 
   /* Parse denomination key hashes */
   for (unsigned int i = 0; i<num_fresh_coins; i++)
@@ -579,7 +583,7 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
       return TALER_MHD_reply_with_error (connection,
                                          hc,
                                          ec,
-                                         "failed to find denomination key");
+                                         NULL);
     }
     /* #TEH_KS_DKU_WITHDRAW should warrant that we only get denomination
        keys where we did not yet forget the private key */
@@ -605,7 +609,7 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
     if (GNUNET_OK != res)
     {
       for (unsigned int j = 0; j<i; j++)
-        GNUNET_free_non_null (rcds[j].coin_ev);
+        GNUNET_free (rcds[j].coin_ev);
       return (GNUNET_NO == res) ? MHD_YES : MHD_NO;
     }
     rcd->dk = &dkis[i]->denom_pub;
@@ -624,24 +628,24 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
       switch (qs)
       {
       case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
-        res = TALER_MHD_reply_with_error (connection,
+        ret = TALER_MHD_reply_with_error (connection,
                                           MHD_HTTP_NOT_FOUND,
                                           TALER_EC_REVEAL_SESSION_UNKNOWN,
-                                          "rc");
+                                          NULL);
         break;
       case GNUNET_DB_STATUS_HARD_ERROR:
-        res = TALER_MHD_reply_with_error (connection,
+        ret = TALER_MHD_reply_with_error (connection,
                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
                                           TALER_EC_REVEAL_DB_FETCH_SESSION_ERROR,
-                                          "failed to fetch session data");
+                                          NULL);
         break;
       case GNUNET_DB_STATUS_SOFT_ERROR:
       default:
         GNUNET_break (0);   /* should be impossible */
-        res = TALER_MHD_reply_with_error (connection,
+        ret = TALER_MHD_reply_with_error (connection,
                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
                                           TALER_EC_INTERNAL_INVARIANT_FAILURE,
-                                          "assertion failed (unexpected database serialization error)");
+                                          NULL);
         break;
       }
       goto cleanup;
@@ -676,17 +680,17 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
                           rcds[i].coin_ev_size,
                           &ldp.coin_envelope_hash);
       if (GNUNET_OK !=
-          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_LINK,
-                                      &ldp.purpose,
-                                      &link_sigs[i].eddsa_signature,
-                                      &melt.session.coin.coin_pub.
-                                      eddsa_pub))
+          GNUNET_CRYPTO_eddsa_verify (
+            TALER_SIGNATURE_WALLET_COIN_LINK,
+            &ldp,
+            &link_sigs[i].eddsa_signature,
+            &melt.session.coin.coin_pub.eddsa_pub))
       {
         GNUNET_break_op (0);
-        res = TALER_MHD_reply_with_error (connection,
+        ret = TALER_MHD_reply_with_error (connection,
                                           MHD_HTTP_FORBIDDEN,
                                           TALER_EC_REVEAL_LINK_SIGNATURE_INVALID,
-                                          "link_sig");
+                                          NULL);
         goto cleanup;
       }
     }
@@ -710,10 +714,10 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
     if (NULL == rctx->ev_sigs[i].rsa_signature)
     {
       GNUNET_break (0);
-      res = TALER_MHD_reply_with_error (connection,
+      ret = TALER_MHD_reply_with_error (connection,
                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
                                         TALER_EC_REVEAL_SIGNING_ERROR,
-                                        "internal signing error");
+                                        NULL);
       goto cleanup;
     }
   }
@@ -729,17 +733,17 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
     if ( (GNUNET_OK ==
           TEH_DB_run_transaction (connection,
                                   "reveal pre-check",
-                                  &res,
+                                  &ret,
                                   &refreshes_reveal_preflight,
                                   rctx)) &&
          (GNUNET_YES == rctx->preflight_ok) )
     {
       /* Generate final (positive) response */
       GNUNET_assert (NULL != rctx->ev_sigs);
-      res = reply_refreshes_reveal_success (connection,
+      ret = reply_refreshes_reveal_success (connection,
                                             num_fresh_coins,
                                             rctx->ev_sigs);
-      GNUNET_break (MHD_NO != res);
+      GNUNET_break (MHD_NO != ret);
       goto cleanup;   /* aka 'break' */
     }
     if (GNUNET_SYSERR == rctx->preflight_ok)
@@ -750,7 +754,7 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
                                 "run reveal",
-                                &res,
+                                &ret,
                                 &refreshes_reveal_transaction,
                                 rctx))
     {
@@ -761,13 +765,13 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
     if (GNUNET_OK ==
         TEH_DB_run_transaction (connection,
                                 "persist reveal",
-                                &res,
+                                &ret,
                                 &refreshes_reveal_persist,
                                 rctx))
     {
       /* Generate final (positive) response */
       GNUNET_assert (NULL != rctx->ev_sigs);
-      res = reply_refreshes_reveal_success (connection,
+      ret = reply_refreshes_reveal_success (connection,
                                             num_fresh_coins,
                                             rctx->ev_sigs);
       break;
@@ -778,7 +782,7 @@ resolve_refreshes_reveal_denominations (struct TEH_KS_StateHandle *key_state,
   }   /* end for (retries...) */
 
 cleanup:
-  GNUNET_break (MHD_NO != res);
+  GNUNET_break (MHD_NO != ret);
   /* free resources */
   if (NULL != rctx->ev_sigs)
   {
@@ -789,8 +793,8 @@ cleanup:
     rctx->ev_sigs = NULL; /* just to be safe... */
   }
   for (unsigned int i = 0; i<num_fresh_coins; i++)
-    GNUNET_free_non_null (rcds[i].coin_ev);
-  return res;
+    GNUNET_free (rcds[i].coin_ev);
+  return ret;
 }
 
 
@@ -828,7 +832,7 @@ handle_refreshes_reveal_json (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_REVEAL_NEW_DENOMS_ARRAY_SIZE_EXCESSIVE,
-                                       "new_denoms_h");
+                                       NULL);
 
   }
   if (json_array_size (new_denoms_h_json) !=
@@ -837,7 +841,7 @@ handle_refreshes_reveal_json (struct MHD_Connection *connection,
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_REVEAL_NEW_DENOMS_ARRAY_SIZE_MISSMATCH,
+                                       TALER_EC_REVEAL_NEW_DENOMS_ARRAY_SIZE_MISMATCH,
                                        "new_denoms/coin_evs");
   }
   if (json_array_size (new_denoms_h_json) !=
@@ -846,7 +850,7 @@ handle_refreshes_reveal_json (struct MHD_Connection *connection,
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_REVEAL_NEW_DENOMS_ARRAY_SIZE_MISSMATCH,
+                                       TALER_EC_REVEAL_NEW_DENOMS_ARRAY_SIZE_MISMATCH,
                                        "new_denoms/link_sigs");
   }
 
@@ -908,7 +912,7 @@ handle_refreshes_reveal_json (struct MHD_Connection *connection,
  * @param args array of additional options (length: 2, session hash and the string "reveal")
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_reveal (const struct TEH_RequestHandler *rh,
                     struct MHD_Connection *connection,
                     const json_t *root,
@@ -942,7 +946,7 @@ TEH_handler_reveal (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_REVEAL_INVALID_RCH,
-                                       "refresh commitment hash malformed");
+                                       args[0]);
   }
   if (0 != strcmp (args[1],
                    "reveal"))
@@ -951,7 +955,7 @@ TEH_handler_reveal (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_OPERATION_INVALID,
-                                       "expected 'reveal' operation");
+                                       args[1]);
   }
 
   {
@@ -976,7 +980,7 @@ TEH_handler_reveal (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_REVEAL_CNC_TRANSFER_ARRAY_SIZE_INVALID,
-                                       "transfer_privs");
+                                       NULL);
   }
 
   {
