@@ -43,7 +43,7 @@
  * @param refund details about the successful refund
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 reply_refund_success (struct MHD_Connection *connection,
                       const struct TALER_CoinSpendPublicKeyP *coin_pub,
                       const struct TALER_EXCHANGEDB_RefundListEntry *refund)
@@ -61,10 +61,8 @@ reply_refund_success (struct MHD_Connection *connection,
 
   TALER_amount_hton (&rc.refund_amount,
                      &refund->refund_amount);
-  TALER_amount_hton (&rc.refund_fee,
-                     &refund->refund_fee);
   if (GNUNET_OK !=
-      TEH_KS_sign (&rc.purpose,
+      TEH_KS_sign (&rc,
                    &pub,
                    &sig))
   {
@@ -73,12 +71,12 @@ reply_refund_success (struct MHD_Connection *connection,
                                        TALER_EC_EXCHANGE_BAD_CONFIGURATION,
                                        "no online signing key");
   }
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_OK,
-                                    "{s:s, s:o, s:o}",
-                                    "status", "REFUND_OK",
-                                    "sig", GNUNET_JSON_from_data_auto (&sig),
-                                    "pub", GNUNET_JSON_from_data_auto (&pub));
+  return TALER_MHD_reply_json_pack (
+    connection,
+    MHD_HTTP_OK,
+    "{s:o, s:o}",
+    "exchange_sig", GNUNET_JSON_from_data_auto (&sig),
+    "exchange_pub", GNUNET_JSON_from_data_auto (&pub));
 }
 
 
@@ -104,15 +102,18 @@ static enum GNUNET_DB_QueryStatus
 refund_transaction (void *cls,
                     struct MHD_Connection *connection,
                     struct TALER_EXCHANGEDB_Session *session,
-                    int *mhd_ret)
+                    MHD_RESULT *mhd_ret)
 {
   const struct TALER_EXCHANGEDB_Refund *refund = cls;
-  struct TALER_EXCHANGEDB_TransactionList *tl;
-  const struct TALER_EXCHANGEDB_DepositListEntry *dep;
-  const struct TALER_EXCHANGEDB_RefundListEntry *ref;
+  struct TALER_EXCHANGEDB_TransactionList *tl; /* head of original list */
+  struct TALER_EXCHANGEDB_TransactionList *tlx; /* head of sublist that applies to merchant and contract */
+  struct TALER_EXCHANGEDB_TransactionList *tln; /* next element, during iteration */
+  struct TALER_EXCHANGEDB_TransactionList *tlp; /* previous element in 'tl' list, during iteration */
   enum GNUNET_DB_QueryStatus qs;
-  int deposit_found;
-  int refund_found;
+  bool deposit_found; /* deposit_total initialized? */
+  bool refund_found; /* refund_total initialized? */
+  struct TALER_Amount deposit_total;
+  struct TALER_Amount refund_total;
 
   tl = NULL;
   qs = TEH_plugin->get_coin_transactions (TEH_plugin->cls,
@@ -124,76 +125,166 @@ refund_transaction (void *cls,
   {
     if (GNUNET_DB_STATUS_HARD_ERROR == qs)
       *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_NOT_FOUND,
-                                             TALER_EC_REFUND_COIN_NOT_FOUND,
-                                             "database transaction failure");
+                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                             TALER_EC_REFUND_DATABASE_LOOKUP_ERROR,
+                                             NULL);
     return qs;
   }
-  dep = NULL;
-  ref = NULL;
-  deposit_found = GNUNET_NO;
-  refund_found = GNUNET_NO;
-  for (struct TALER_EXCHANGEDB_TransactionList *tlp = tl;
-       NULL != tlp;
-       tlp = tlp->next)
+  deposit_found = false;
+  refund_found = false;
+  tlx = NULL; /* relevant subset of transactions */
+  tln = NULL;
+  tlp = NULL;
+  for (struct TALER_EXCHANGEDB_TransactionList *tli = tl;
+       NULL != tli;
+       tli = tln)
   {
-    switch (tlp->type)
+    tln = tli->next;
+    switch (tli->type)
     {
     case TALER_EXCHANGEDB_TT_DEPOSIT:
-      if (GNUNET_NO == deposit_found)
       {
-        if ( (0 == memcmp (&tlp->details.deposit->merchant_pub,
-                           &refund->details.merchant_pub,
-                           sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&tlp->details.deposit->h_contract_terms,
-                           &refund->details.h_contract_terms,
-                           sizeof (struct GNUNET_HashCode))) )
+        const struct TALER_EXCHANGEDB_DepositListEntry *dep;
+
+        dep = tli->details.deposit;
+        if ( (0 == GNUNET_memcmp (&dep->merchant_pub,
+                                  &refund->details.merchant_pub)) &&
+             (0 == GNUNET_memcmp (&dep->h_contract_terms,
+                                  &refund->details.h_contract_terms)) )
         {
-          dep = tlp->details.deposit;
-          deposit_found = GNUNET_YES;
+          /* check if we already send the money for this /deposit */
+          if (dep->done)
+          {
+            TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                    tlx);
+            TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                    tln);
+            /* money was already transferred to merchant, can no longer refund */
+            *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                                   MHD_HTTP_GONE,
+                                                   TALER_EC_REFUND_MERCHANT_ALREADY_PAID,
+                                                   NULL);
+            return GNUNET_DB_STATUS_HARD_ERROR;
+          }
+
+          /* deposit applies and was not yet wired; add to total (it is NOT
+             the case that multiple deposits of the same coin for the same
+             contract are really allowed (see UNIQUE constraint on 'deposits'
+             table), but in case this changes we tolerate it with this code
+             anyway). *///
+          if (deposit_found)
+          {
+            GNUNET_assert (0 <=
+                           TALER_amount_add (&deposit_total,
+                                             &deposit_total,
+                                             &dep->amount_with_fee));
+          }
+          else
+          {
+            deposit_total = dep->amount_with_fee;
+            deposit_found = true;
+          }
+          /* move 'tli' from 'tl' to 'tlx' list */
+          if (NULL == tlp)
+            tl = tln;
+          else
+            tlp->next = tln;
+          tli->next = tlx;
+          tlx = tli;
           break;
         }
+        else
+        {
+          tlp = tli;
+        }
+        break;
       }
-      break;
     case TALER_EXCHANGEDB_TT_MELT:
       /* Melts cannot be refunded, ignore here */
       break;
     case TALER_EXCHANGEDB_TT_REFUND:
-      if (GNUNET_NO == refund_found)
       {
-        /* First, check if existing refund request is identical */
-        if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
-                           &refund->details.merchant_pub,
-                           sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&tlp->details.refund->h_contract_terms,
-                           &refund->details.h_contract_terms,
-                           sizeof (struct GNUNET_HashCode))) &&
-             (tlp->details.refund->rtransaction_id ==
-              refund->details.rtransaction_id) )
+        const struct TALER_EXCHANGEDB_RefundListEntry *ref;
+
+        ref = tli->details.refund;
+        if ( (0 != GNUNET_memcmp (&ref->merchant_pub,
+                                  &refund->details.merchant_pub)) ||
+             (0 != GNUNET_memcmp (&ref->h_contract_terms,
+                                  &refund->details.h_contract_terms)) )
         {
-          ref = tlp->details.refund;
-          refund_found = GNUNET_YES;
-          break;
+          tlp = tli;
+          break; /* refund does not apply to our transaction */
         }
-        /* Second, check if existing refund request conflicts */
-        if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
-                           &refund->details.merchant_pub,
-                           sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&tlp->details.refund->h_contract_terms,
-                           &refund->details.h_contract_terms,
-                           sizeof (struct GNUNET_HashCode))) &&
-             (tlp->details.refund->rtransaction_id !=
-              refund->details.rtransaction_id) )
+        /* Check if existing refund request matches in everything but the amount */
+        if ( (ref->rtransaction_id ==
+              refund->details.rtransaction_id) &&
+             (0 != TALER_amount_cmp (&ref->refund_amount,
+                                     &refund->details.refund_amount)) )
         {
-          refund_found = GNUNET_SYSERR;
-          /* NOTE: Alternatively we could total up all existing
-             refunds and check if the sum still permits the
-             refund requested (thus allowing multiple, partial
-             refunds). Fow now, we keep it simple. */
-          break;
+          /* Generate precondition failed response, with ONLY the conflicting entry */
+          TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                  tlx);
+          TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                  tln);
+          tli->next = NULL;
+          *mhd_ret = TALER_MHD_reply_json_pack (
+            connection,
+            MHD_HTTP_PRECONDITION_FAILED,
+            "{s:s, s:s, s:I, s:o}",
+            "detail",
+            "conflicting refund with different amount but same refund transaction ID",
+            "hint", TALER_ErrorCode_get_hint (
+              TALER_EC_REFUND_INCONSISTENT_AMOUNT),
+            "code", (json_int_t) TALER_EC_REFUND_INCONSISTENT_AMOUNT,
+            "history", TEH_RESPONSE_compile_transaction_history (
+              &refund->coin.coin_pub,
+              tli));
+          TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                  tli);
+          return GNUNET_DB_STATUS_HARD_ERROR;
         }
+        /* Check if existing refund request matches in everything including the amount */
+        if ( (ref->rtransaction_id ==
+              refund->details.rtransaction_id) &&
+             (0 == TALER_amount_cmp (&ref->refund_amount,
+                                     &refund->details.refund_amount)) )
+        {
+          /* we can blanketly approve, as this request is identical to one
+             we saw before */
+          *mhd_ret = reply_refund_success (connection,
+                                           &refund->coin.coin_pub,
+                                           ref);
+          TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                  tlx);
+          TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                                  tl);
+          /* we still abort the transaction, as there is nothing to be
+             committed! */
+          return GNUNET_DB_STATUS_HARD_ERROR;
+        }
+
+        /* We have another refund, that relates, add to total */
+        if (refund_found)
+        {
+          GNUNET_assert (0 <=
+                         TALER_amount_add (&refund_total,
+                                           &refund_total,
+                                           &ref->refund_amount));
+        }
+        else
+        {
+          refund_total = ref->refund_amount;
+          refund_found = true;
+        }
+        /* move 'tli' from 'tl' to 'tlx' list */
+        if (NULL == tlp)
+          tl = tln;
+        else
+          tlp->next = tln;
+        tli->next = tlx;
+        tlx = tli;
+        break;
       }
-      break;
     case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
       /* Recoups cannot be refunded, ignore here */
       break;
@@ -205,114 +296,69 @@ refund_transaction (void *cls,
       break;
     }
   }
+  /* no need for 'tl' anymore, everything we may still care about is in tlx now */
+  TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                          tl);
   /* handle if deposit was NOT found */
-  if (GNUNET_NO == deposit_found)
+  if (! deposit_found)
   {
     TALER_LOG_WARNING ("Deposit to /refund was not found\n");
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
+                                            tlx);
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_REFUND_DEPOSIT_NOT_FOUND,
-                                           "deposit unknown");
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-  /* handle if conflicting refund found */
-  if (GNUNET_SYSERR == refund_found)
-  {
-    *mhd_ret = TALER_MHD_reply_json_pack (
-      connection,
-      MHD_HTTP_CONFLICT,
-      "{s:s, s:I, s:o}",
-      "hint", "conflicting refund",
-      "code", (json_int_t) TALER_EC_REFUND_CONFLICT,
-      "history", TEH_RESPONSE_compile_transaction_history (
-        &refund->coin.coin_pub,
-        tl));
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-  /* handle if identical refund found */
-  if (GNUNET_YES == refund_found)
-  {
-    /* /refund already done, simply re-transmit confirmation */
-    *mhd_ret = reply_refund_success (connection,
-                                     &refund->coin.coin_pub,
-                                     ref);
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
   /* check currency is compatible */
-  if ( (GNUNET_YES !=
-        TALER_amount_cmp_currency (&refund->details.refund_amount,
-                                   &dep->amount_with_fee)) ||
-       (GNUNET_YES !=
-        TALER_amount_cmp_currency (&refund->details.refund_fee,
-                                   &dep->deposit_fee)) )
+  if (GNUNET_YES !=
+      TALER_amount_cmp_currency (&refund->details.refund_amount,
+                                 &deposit_total))
   {
     GNUNET_break_op (0); /* currency mismatch */
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
+                                            tlx);
     *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_PRECONDITION_FAILED,
-                                           TALER_EC_REFUND_CURRENCY_MISSMATCH,
-                                           "currencies involved do not match");
+                                           MHD_HTTP_BAD_REQUEST,
+                                           TALER_EC_REFUND_CURRENCY_MISMATCH,
+                                           deposit_total.currency);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
-  /* check if we already send the money for the /deposit */
-  qs = TEH_plugin->test_deposit_done (TEH_plugin->cls,
-                                      session,
-                                      &refund->coin.coin_pub,
-                                      &dep->merchant_pub,
-                                      &dep->h_contract_terms,
-                                      &dep->h_wire);
-  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-  {
-    /* Internal error, we first had the deposit in the history,
-       but now it is gone? */
-    GNUNET_break (0);
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                           TALER_EC_REFUND_DB_INCONSISTENT,
-                                           "database inconsistent (deposit data became inaccessible during transaction)");
-    return qs;
-  }
-  if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
-    return qs; /* go and retry */
+  /* check total refund amount is sufficiently low */
+  if (refund_found)
+    GNUNET_break (0 <=
+                  TALER_amount_add (&refund_total,
+                                    &refund_total,
+                                    &refund->details.refund_amount));
+  else
+    refund_total = refund->details.refund_amount;
 
-  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
+  if (1 == TALER_amount_cmp (&refund_total,
+                             &deposit_total) )
   {
-    /* money was already transferred to merchant, can no longer refund */
+    *mhd_ret = TALER_MHD_reply_json_pack (
+      connection,
+      MHD_HTTP_CONFLICT,
+      "{s:s, s:s, s:I, s:o}",
+      "detail",
+      "total amount refunded exceeds total amount deposited for this coin",
+      "hint",
+      TALER_ErrorCode_get_hint (TALER_EC_REFUND_CONFLICT_DEPOSIT_INSUFFICIENT),
+      "code",
+      (json_int_t) TALER_EC_REFUND_CONFLICT_DEPOSIT_INSUFFICIENT,
+      "history",
+      TEH_RESPONSE_compile_transaction_history (&refund->coin.coin_pub,
+                                                tlx));
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_GONE,
-                                           TALER_EC_REFUND_MERCHANT_ALREADY_PAID,
-                                           "money already sent to merchant");
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-
-  /* check refund amount is sufficiently low */
-  if (1 == TALER_amount_cmp (&refund->details.refund_amount,
-                             &dep->amount_with_fee) )
-  {
-    GNUNET_break_op (0); /* cannot refund more than original value */
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_PRECONDITION_FAILED,
-                                           TALER_EC_REFUND_INSUFFICIENT_FUNDS,
-                                           "refund requested exceeds original value");
+                                            tlx);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                          tl);
+                                          tlx);
+
 
   /* Finally, store new refund data */
   qs = TEH_plugin->insert_refund (TEH_plugin->cls,
@@ -324,7 +370,7 @@ refund_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_REFUND_STORE_DB_ERROR,
-                                           "could not persist store information");
+                                           NULL);
     return qs;
   }
   /* Success or soft failure */
@@ -339,15 +385,14 @@ refund_transaction (void *cls,
  * the fee structure, so this is not done here.
  *
  * @param connection the MHD connection to handle
- * @param refund information about the refund
+ * @param[in,out] refund information about the refund
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 verify_and_execute_refund (struct MHD_Connection *connection,
-                           const struct TALER_EXCHANGEDB_Refund *refund)
+                           struct TALER_EXCHANGEDB_Refund *refund)
 {
   struct GNUNET_HashCode denom_hash;
-  struct TALER_Amount expect_fee;
 
   {
     struct TALER_RefundRequestPS rr = {
@@ -361,11 +406,9 @@ verify_and_execute_refund (struct MHD_Connection *connection,
 
     TALER_amount_hton (&rr.refund_amount,
                        &refund->details.refund_amount);
-    TALER_amount_hton (&rr.refund_fee,
-                       &refund->details.refund_fee);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-                                    &rr.purpose,
+                                    &rr,
                                     &refund->details.merchant_sig.eddsa_sig,
                                     &refund->details.merchant_pub.eddsa_pub))
     {
@@ -373,7 +416,7 @@ verify_and_execute_refund (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_REFUND_MERCHANT_SIGNATURE_INVALID,
-                                         "merchant_sig");
+                                         NULL);
     }
   }
 
@@ -387,11 +430,18 @@ verify_and_execute_refund (struct MHD_Connection *connection,
                                             &denom_hash);
     if (0 > qs)
     {
+      MHD_RESULT res;
+      char *dhs;
+
       GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_NOT_FOUND,
-                                         TALER_EC_REFUND_COIN_NOT_FOUND,
-                                         "denomination of coin to be refunded not found in DB");
+      dhs = GNUNET_STRINGS_data_to_string_alloc (&denom_hash,
+                                                 sizeof (denom_hash));
+      res = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_NOT_FOUND,
+                                        TALER_EC_REFUND_COIN_NOT_FOUND,
+                                        dhs);
+      GNUNET_free (dhs);
+      return res;
     }
   }
 
@@ -427,48 +477,17 @@ verify_and_execute_refund (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            hc,
                                            ec,
-                                           "denomination not found, but coin known");
+                                           NULL);
       }
-      TALER_amount_ntoh (&expect_fee,
+      TALER_amount_ntoh (&refund->details.refund_fee,
                          &dki->issue.properties.fee_refund);
     }
     TEH_KS_release (key_state);
   }
 
-  /* Check refund fee matches fee of denomination key! */
-  if (GNUNET_YES !=
-      TALER_amount_cmp_currency (&expect_fee,
-                                 &refund->details.refund_fee) )
-  {
-    GNUNET_break_op (0);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_REFUND_FEE_CURRENCY_MISSMATCH,
-                                       "refund_fee");
-  }
-  {
-    int fee_cmp;
-
-    fee_cmp = TALER_amount_cmp (&refund->details.refund_fee,
-                                &expect_fee);
-    if (-1 == fee_cmp)
-    {
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_REFUND_FEE_TOO_LOW,
-                                         "refund_fee");
-    }
-    if (1 == fee_cmp)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Refund fee proposed by merchant is higher than necessary.\n");
-    }
-  }
-
-
   /* Finally run the actual transaction logic */
   {
-    int mhd_ret;
+    MHD_RESULT mhd_ret;
 
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
@@ -497,27 +516,31 @@ verify_and_execute_refund (struct MHD_Connection *connection,
  * @param root uploaded JSON data
  * @return MHD result code
   */
-int
+MHD_RESULT
 TEH_handler_refund (struct MHD_Connection *connection,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const json_t *root)
 {
-  struct TALER_EXCHANGEDB_Refund refund;
+  struct TALER_EXCHANGEDB_Refund refund = {
+    .details.refund_fee.currency = {0}                                        /* set to invalid, just to be sure */
+  };
   struct GNUNET_JSON_Specification spec[] = {
-    TALER_JSON_spec_amount ("refund_amount", &refund.details.refund_amount),
-    TALER_JSON_spec_amount ("refund_fee", &refund.details.refund_fee),
+    TALER_JSON_spec_amount ("refund_amount",
+                            &refund.details.refund_amount),
     GNUNET_JSON_spec_fixed_auto ("h_contract_terms",
                                  &refund.details.h_contract_terms),
-    GNUNET_JSON_spec_fixed_auto ("merchant_pub", &refund.details.merchant_pub),
+    GNUNET_JSON_spec_fixed_auto ("merchant_pub",
+                                 &refund.details.merchant_pub),
     GNUNET_JSON_spec_uint64 ("rtransaction_id",
                              &refund.details.rtransaction_id),
-    GNUNET_JSON_spec_fixed_auto ("merchant_sig", &refund.details.merchant_sig),
+    GNUNET_JSON_spec_fixed_auto ("merchant_sig",
+                                 &refund.details.merchant_sig),
     GNUNET_JSON_spec_end ()
   };
 
   refund.coin.coin_pub = *coin_pub;
   {
-    int res;
+    enum GNUNET_GenericReturnValue res;
 
     res = TALER_MHD_parse_json_data (connection,
                                      root,
@@ -527,29 +550,8 @@ TEH_handler_refund (struct MHD_Connection *connection,
     if (GNUNET_NO == res)
       return MHD_YES; /* failure */
   }
-  if (GNUNET_YES !=
-      TALER_amount_cmp_currency (&refund.details.refund_amount,
-                                 &refund.details.refund_fee) )
   {
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_REFUND_FEE_CURRENCY_MISSMATCH,
-                                       "refund_amount or refund_fee");
-  }
-  if (-1 == TALER_amount_cmp (&refund.details.refund_amount,
-                              &refund.details.refund_fee) )
-  {
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_REFUND_FEE_ABOVE_AMOUNT,
-                                       "refund_amount");
-  }
-  {
-    int res;
+    MHD_RESULT res;
 
     res = verify_and_execute_refund (connection,
                                      &refund);
