@@ -15,7 +15,7 @@
 */
 /**
  * @file taler-exchange-httpd_responses.c
- * @brief API for generating genric replies of the exchange; these
+ * @brief API for generating generic replies of the exchange; these
  *        functions are called TEH_RESPONSE_reply_ and they generate
  *        and queue MHD response objects for a given connection.
  * @author Florian Dold
@@ -66,7 +66,8 @@ TEH_RESPONSE_compile_transaction_history (
           .purpose.size = htonl (sizeof (dr)),
           .h_contract_terms = deposit->h_contract_terms,
           .h_wire = deposit->h_wire,
-          .timestamp = GNUNET_TIME_absolute_hton (deposit->timestamp),
+          .h_denom_pub = deposit->h_denom_pub,
+          .wallet_timestamp = GNUNET_TIME_absolute_hton (deposit->timestamp),
           .refund_deadline = GNUNET_TIME_absolute_hton (
             deposit->refund_deadline),
           .merchant = deposit->merchant_pub,
@@ -81,7 +82,7 @@ TEH_RESPONSE_compile_transaction_history (
         /* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
-                                        &dr.purpose,
+                                        &dr,
                                         &deposit->csig.eddsa_signature,
                                         &coin_pub->eddsa_pub))
         {
@@ -94,7 +95,7 @@ TEH_RESPONSE_compile_transaction_history (
             json_array_append_new (
               history,
               json_pack (
-                "{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+                "{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
                 "type",
                 "DEPOSIT",
                 "amount",
@@ -111,6 +112,8 @@ TEH_RESPONSE_compile_transaction_history (
                 GNUNET_JSON_from_data_auto (&deposit->h_contract_terms),
                 "h_wire",
                 GNUNET_JSON_from_data_auto (&deposit->h_wire),
+                "h_denom_pub",
+                GNUNET_JSON_from_data_auto (&deposit->h_denom_pub),
                 "coin_sig",
                 GNUNET_JSON_from_data_auto (&deposit->csig))))
         {
@@ -128,6 +131,7 @@ TEH_RESPONSE_compile_transaction_history (
           .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT),
           .purpose.size = htonl (sizeof (ms)),
           .rc = melt->rc,
+          .h_denom_pub = melt->h_denom_pub,
           .coin_pub = *coin_pub
         };
 
@@ -139,7 +143,7 @@ TEH_RESPONSE_compile_transaction_history (
         /* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_MELT,
-                                        &ms.purpose,
+                                        &ms,
                                         &melt->coin_sig.eddsa_signature,
                                         &coin_pub->eddsa_pub))
         {
@@ -151,7 +155,7 @@ TEH_RESPONSE_compile_transaction_history (
         if (0 !=
             json_array_append_new (
               history,
-              json_pack ("{s:s, s:o, s:o, s:o, s:o}",
+              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
                          "type",
                          "MELT",
                          "amount",
@@ -160,6 +164,8 @@ TEH_RESPONSE_compile_transaction_history (
                          TALER_JSON_from_amount (&melt->melt_fee),
                          "rc",
                          GNUNET_JSON_from_data_auto (&melt->rc),
+                         "h_denom_pub",
+                         GNUNET_JSON_from_data_auto (&melt->h_denom_pub),
                          "coin_sig",
                          GNUNET_JSON_from_data_auto (&melt->coin_sig))))
         {
@@ -185,13 +191,11 @@ TEH_RESPONSE_compile_transaction_history (
 
         TALER_amount_hton (&rr.refund_amount,
                            &refund->refund_amount);
-        TALER_amount_hton (&rr.refund_fee,
-                           &refund->refund_fee);
 #if ENABLE_SANITY_CHECKS
         /* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-                                        &rr.purpose,
+                                        &rr,
                                         &refund->merchant_sig.eddsa_sig,
                                         &refund->merchant_pub.eddsa_pub))
         {
@@ -200,7 +204,7 @@ TEH_RESPONSE_compile_transaction_history (
           return NULL;
         }
 #endif
-        if (GNUNET_OK !=
+        if (0 >
             TALER_amount_subtract (&value,
                                    &refund->refund_amount,
                                    &refund->refund_fee))
@@ -246,14 +250,14 @@ TEH_RESPONSE_compile_transaction_history (
             TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH),
           .purpose.size = htonl (sizeof (pc)),
           .timestamp = GNUNET_TIME_absolute_hton (pr->timestamp),
-          .coin_pub = *coin_pub,
+          .coin_pub = pr->coin.coin_pub,
           .old_coin_pub = pr->old_coin_pub
         };
 
         TALER_amount_hton (&pc.recoup_amount,
                            &pr->value);
         if (GNUNET_OK !=
-            TEH_KS_sign (&pc.purpose,
+            TEH_KS_sign (&pc,
                          &epub,
                          &esig))
         {
@@ -306,7 +310,7 @@ TEH_RESPONSE_compile_transaction_history (
         TALER_amount_hton (&pc.recoup_amount,
                            &recoup->value);
         if (GNUNET_OK !=
-            TEH_KS_sign (&pc.purpose,
+            TEH_KS_sign (&pc,
                          &epub,
                          &esig))
         {
@@ -317,7 +321,7 @@ TEH_RESPONSE_compile_transaction_history (
         if (0 !=
             json_array_append_new (
               history,
-              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
+              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
                          "type",
                          "RECOUP",
                          "amount",
@@ -328,6 +332,14 @@ TEH_RESPONSE_compile_transaction_history (
                          GNUNET_JSON_from_data_auto (&epub),
                          "reserve_pub",
                          GNUNET_JSON_from_data_auto (&recoup->reserve_pub),
+                         "h_denom_pub",
+                         GNUNET_JSON_from_data_auto (&recoup->h_denom_pub),
+                         "coin_sig",
+                         GNUNET_JSON_from_data_auto (&recoup->coin_sig),
+                         "coin_blind",
+                         GNUNET_JSON_from_data_auto (&recoup->coin_blind),
+                         "reserve_pub",
+                         GNUNET_JSON_from_data_auto (&recoup->reserve_pub),
                          "timestamp",
                          GNUNET_JSON_from_time_abs (recoup->timestamp))))
         {
@@ -355,7 +367,7 @@ TEH_RESPONSE_compile_transaction_history (
         TALER_amount_hton (&pc.recoup_amount,
                            &pr->value);
         if (GNUNET_OK !=
-            TEH_KS_sign (&pc.purpose,
+            TEH_KS_sign (&pc,
                          &epub,
                          &esig))
         {
@@ -371,7 +383,7 @@ TEH_RESPONSE_compile_transaction_history (
         if (0 !=
             json_array_append_new (
               history,
-              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
+              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
                          "type",
                          "RECOUP-REFRESH",
                          "amount",
@@ -382,6 +394,12 @@ TEH_RESPONSE_compile_transaction_history (
                          GNUNET_JSON_from_data_auto (&epub),
                          "old_coin_pub",
                          GNUNET_JSON_from_data_auto (&pr->old_coin_pub),
+                         "h_denom_pub",
+                         GNUNET_JSON_from_data_auto (&pr->coin.denom_pub_hash),
+                         "coin_sig",
+                         GNUNET_JSON_from_data_auto (&pr->coin_sig),
+                         "coin_blind",
+                         GNUNET_JSON_from_data_auto (&pr->coin_blind),
                          "timestamp",
                          GNUNET_JSON_from_time_abs (pr->timestamp))))
         {
@@ -411,7 +429,7 @@ TEH_RESPONSE_compile_transaction_history (
  * @param tl transaction list to use to build reply
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_RESPONSE_reply_coin_insufficient_funds (
   struct MHD_Connection *connection,
   enum TALER_ErrorCode ec,
@@ -427,13 +445,13 @@ TEH_RESPONSE_reply_coin_insufficient_funds (
     GNUNET_break (0);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_COIN_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS,
-                                       "failed to convert transaction history to JSON");
+                                       TALER_EC_JSON_ALLOCATION_FAILURE,
+                                       "Failed to generated proof of insufficient funds");
   }
   return TALER_MHD_reply_json_pack (connection,
                                     MHD_HTTP_CONFLICT,
                                     "{s:s, s:I, s:o}",
-                                    "hint", "insufficient funds",
+                                    "hint", TALER_ErrorCode_get_hint (ec),
                                     "code", (json_int_t) ec,
                                     "history", history);
 }
@@ -481,7 +499,7 @@ TEH_RESPONSE_compile_reserve_history (
           credit_total = bank->amount;
           init |= IA_CREDIT;
         }
-        else if (GNUNET_OK !=
+        else if (0 >
                  TALER_amount_add (&credit_total,
                                    &credit_total,
                                    &bank->amount))
@@ -526,7 +544,7 @@ TEH_RESPONSE_compile_reserve_history (
         }
         else
         {
-          if (GNUNET_OK !=
+          if (0 >
               TALER_amount_add (&withdraw_total,
                                 &withdraw_total,
                                 &value))
@@ -572,7 +590,7 @@ TEH_RESPONSE_compile_reserve_history (
           credit_total = recoup->value;
           init |= IA_CREDIT;
         }
-        else if (GNUNET_OK !=
+        else if (0 >
                  TALER_amount_add (&credit_total,
                                    &credit_total,
                                    &recoup->value))
@@ -593,7 +611,7 @@ TEH_RESPONSE_compile_reserve_history (
           TALER_amount_hton (&pc.recoup_amount,
                              &recoup->value);
           if (GNUNET_OK !=
-              TEH_KS_sign (&pc.purpose,
+              TEH_KS_sign (&pc,
                            &pub,
                            &sig))
           {
@@ -642,7 +660,7 @@ TEH_RESPONSE_compile_reserve_history (
         }
         else
         {
-          if (GNUNET_OK !=
+          if (0 >
               TALER_amount_add (&withdraw_total,
                                 &withdraw_total,
                                 &value))
@@ -669,7 +687,7 @@ TEH_RESPONSE_compile_reserve_history (
                               strlen (closing->receiver_account_details) + 1,
                               &rcc.h_wire);
           if (GNUNET_OK !=
-              TEH_KS_sign (&rcc.purpose,
+              TEH_KS_sign (&rcc,
                            &pub,
                            &sig))
           {
@@ -724,7 +742,7 @@ TEH_RESPONSE_compile_reserve_history (
                    TALER_amount_get_zero (credit_total.currency,
                                           &withdraw_total));
   }
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (balance,
                              &credit_total,
                              &withdraw_total))
