@@ -37,7 +37,7 @@
  * @param args array of additional options (must be empty for this function)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_terms (const struct TEH_RequestHandler *rh,
                    struct MHD_Connection *connection,
                    const char *const args[]);
@@ -51,7 +51,7 @@ TEH_handler_terms (const struct TEH_RequestHandler *rh,
  * @param args array of additional options (must be empty for this function)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_privacy (const struct TEH_RequestHandler *rh,
                      struct MHD_Connection *connection,
                      const char *const args[]);
