@@ -83,7 +83,7 @@ struct AggregatedDepositDetail
  * @param wdd_head linked list with details about the combined deposits
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 reply_transfer_details (struct MHD_Connection *connection,
                         const struct TALER_Amount *total,
                         const struct TALER_MerchantPublicKeyP *merchant_pub,
@@ -159,7 +159,7 @@ reply_transfer_details (struct MHD_Connection *connection,
   GNUNET_CRYPTO_hash_context_finish (hash_context,
                                      &wdp.h_details);
   if (GNUNET_OK !=
-      TEH_KS_sign (&wdp.purpose,
+      TEH_KS_sign (&wdp,
                    &pub,
                    &sig))
   {
@@ -314,7 +314,7 @@ handle_deposit_data (void *cls,
     ctx->exec_time = exec_time;
     ctx->wire_method = wire_method; /* captures the reference */
     ctx->is_valid = GNUNET_YES;
-    if (GNUNET_OK !=
+    if (0 >
         TALER_amount_subtract (&ctx->total,
                                deposit_value,
                                deposit_fee))
@@ -343,7 +343,7 @@ handle_deposit_data (void *cls,
       return;
     }
     GNUNET_free (wire_method);
-    if (GNUNET_OK !=
+    if (0 >
         TALER_amount_subtract (&delta,
                                deposit_value,
                                deposit_fee))
@@ -352,7 +352,7 @@ handle_deposit_data (void *cls,
       ctx->is_valid = GNUNET_SYSERR;
       return;
     }
-    if (GNUNET_OK !=
+    if (0 >
         TALER_amount_add (&ctx->total,
                           &ctx->total,
                           &delta))
@@ -395,7 +395,7 @@ free_ctx (struct WtidTransactionContext *ctx)
                                  wdd);
     GNUNET_free (wdd);
   }
-  GNUNET_free_non_null (ctx->wire_method);
+  GNUNET_free (ctx->wire_method);
   ctx->wire_method = NULL;
 }
 
@@ -421,7 +421,7 @@ static enum GNUNET_DB_QueryStatus
 get_transfer_deposits (void *cls,
                        struct MHD_Connection *connection,
                        struct TALER_EXCHANGEDB_Session *session,
-                       int *mhd_ret)
+                       MHD_RESULT *mhd_ret)
 {
   struct WtidTransactionContext *ctx = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -446,7 +446,7 @@ get_transfer_deposits (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_TRANSFERS_GET_DB_FETCH_FAILED,
-                                             "failed to fetch transaction data");
+                                             NULL);
     }
     return qs;
   }
@@ -456,7 +456,7 @@ get_transfer_deposits (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_TRANSFERS_GET_DB_INCONSISTENT,
-                                           "exchange database internally inconsistent");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   if (GNUNET_NO == ctx->is_valid)
@@ -464,7 +464,7 @@ get_transfer_deposits (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_TRANSFERS_GET_WTID_NOT_FOUND,
-                                           "wtid");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   qs = TEH_plugin->get_wire_fee (TEH_plugin->cls,
@@ -485,11 +485,11 @@ get_transfer_deposits (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_TRANSFERS_GET_WIRE_FEE_NOT_FOUND,
-                                             "did not find wire fee");
+                                             NULL);
     }
     return qs;
   }
-  if (GNUNET_OK !=
+  if (0 >
       TALER_amount_subtract (&ctx->total,
                              &ctx->total,
                              &ctx->wire_fee))
@@ -498,7 +498,7 @@ get_transfer_deposits (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_TRANSFERS_GET_WIRE_FEE_INCONSISTENT,
-                                           "could not subtract wire fee");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
@@ -513,13 +513,13 @@ get_transfer_deposits (void *cls,
  * @param args array of additional options (length: 1, just the wtid)
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_transfers_get (const struct TEH_RequestHandler *rh,
                            struct MHD_Connection *connection,
                            const char *const args[1])
 {
   struct WtidTransactionContext ctx;
-  int mhd_ret;
+  MHD_RESULT mhd_ret;
 
   (void) rh;
   memset (&ctx,
@@ -535,7 +535,7 @@ TEH_handler_transfers_get (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_TRANSFERS_INVALID_WTID,
-                                       "wire transfer identifier malformed");
+                                       args[0]);
   }
   if (GNUNET_OK !=
       TEH_DB_run_transaction (connection,
