@@ -51,7 +51,7 @@
  * @param rh reserve history to return
  * @return MHD result code
  */
-static int
+static MHD_RESULT
 reply_withdraw_insufficient_funds (
   struct MHD_Connection *connection,
   const struct TALER_Amount *ebalance,
@@ -66,7 +66,7 @@ reply_withdraw_insufficient_funds (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_WITHDRAW_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS,
-                                       "reserve balance calculation failure");
+                                       NULL);
   if (0 !=
       TALER_amount_cmp (&balance,
                         ebalance))
@@ -76,18 +76,20 @@ reply_withdraw_insufficient_funds (
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_WITHDRAW_RESERVE_BALANCE_CORRUPT,
-                                       "internal balance inconsistency error");
+                                       NULL);
   }
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_CONFLICT,
-                                    "{s:s, s:I, s:o, s:o}",
-                                    "hint", "insufficient funds",
-                                    "code",
-                                    (json_int_t)
-                                    TALER_EC_WITHDRAW_INSUFFICIENT_FUNDS,
-                                    "balance", TALER_JSON_from_amount (
-                                      &balance),
-                                    "history", json_history);
+  return TALER_MHD_reply_json_pack (
+    connection,
+    MHD_HTTP_CONFLICT,
+    "{s:s, s:I, s:o, s:o}",
+    "hint",
+    TALER_ErrorCode_get_hint (TALER_EC_WITHDRAW_INSUFFICIENT_FUNDS),
+    "code",
+    (json_int_t) TALER_EC_WITHDRAW_INSUFFICIENT_FUNDS,
+    "balance",
+    TALER_JSON_from_amount (&balance),
+    "history",
+    json_history);
 }
 
 
@@ -173,7 +175,7 @@ static enum GNUNET_DB_QueryStatus
 withdraw_transaction (void *cls,
                       struct MHD_Connection *connection,
                       struct TALER_EXCHANGEDB_Session *session,
-                      int *mhd_ret)
+                      MHD_RESULT *mhd_ret)
 {
   struct WithdrawContext *wc = cls;
   struct TALER_EXCHANGEDB_Reserve r;
@@ -197,7 +199,7 @@ withdraw_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_WITHDRAW_DB_FETCH_ERROR,
-                                             "failed to fetch withdraw data");
+                                             NULL);
     wc->collectable.sig = denom_sig;
     return qs;
   }
@@ -232,7 +234,7 @@ withdraw_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_WITHDRAW_DB_FETCH_ERROR,
-                                             "failed to fetch reserve data");
+                                             NULL);
     return qs;
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
@@ -240,7 +242,7 @@ withdraw_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_NOT_FOUND,
                                            TALER_EC_WITHDRAW_RESERVE_UNKNOWN,
-                                           "reserve_pub");
+                                           NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   if (0 < TALER_amount_cmp (&wc->amount_required,
@@ -274,7 +276,7 @@ withdraw_transaction (void *cls,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_WITHDRAW_DB_FETCH_ERROR,
-                                               "failed to fetch reserve history");
+                                               NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
     *mhd_ret = reply_withdraw_insufficient_funds (connection,
@@ -299,15 +301,13 @@ withdraw_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_WITHDRAW_SIGNATURE_FAILED,
-                                             "Failed to create blind signature");
+                                             NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
   }
 #endif
   wc->collectable.denom_pub_hash = wc->denom_pub_hash;
   wc->collectable.amount_with_fee = wc->amount_required;
-  TALER_amount_ntoh (&wc->collectable.withdraw_fee,
-                     &wc->dki->issue.properties.fee_withdraw);
   wc->collectable.reserve_pub = wc->wsrd.reserve_pub;
   wc->collectable.h_coin_envelope = wc->wsrd.h_coin_envelope;
   wc->collectable.reserve_sig = wc->signature;
@@ -321,7 +321,7 @@ withdraw_transaction (void *cls,
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_WITHDRAW_DB_STORE_ERROR,
-                                             "failed to persist withdraw data");
+                                             NULL);
     return qs;
   }
   return qs;
@@ -343,7 +343,7 @@ withdraw_transaction (void *cls,
  *         reserve public key, the second one should be "withdraw")
  * @return MHD result code
  */
-int
+MHD_RESULT
 TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
                       struct MHD_Connection *connection,
                       const json_t *root,
@@ -372,11 +372,11 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_RESERVES_INVALID_RESERVE_PUB,
-                                       "reserve public key malformed");
+                                       args[0]);
   }
 
   {
-    int res;
+    enum GNUNET_GenericReturnValue res;
 
     res = TALER_MHD_parse_json_data (connection,
                                      root,
@@ -410,7 +410,7 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          hc,
                                          ec,
-                                         "could not find denomination key");
+                                         NULL);
     }
   }
   GNUNET_assert (NULL != wc.dki->denom_priv.rsa_private_key);
@@ -422,7 +422,7 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
                        &wc.dki->issue.properties.value);
     TALER_amount_ntoh (&fee_withdraw,
                        &wc.dki->issue.properties.fee_withdraw);
-    if (GNUNET_OK !=
+    if (0 >
         TALER_amount_add (&wc.amount_required,
                           &amount,
                           &fee_withdraw))
@@ -432,17 +432,15 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
                                          TALER_EC_WITHDRAW_AMOUNT_FEE_OVERFLOW,
-                                         "amount overflow for value plus withdraw fee");
+                                         NULL);
     }
     TALER_amount_hton (&wc.wsrd.amount_with_fee,
                        &wc.amount_required);
-    TALER_amount_hton (&wc.wsrd.withdraw_fee,
-                       &fee_withdraw);
   }
 
   /* verify signature! */
   wc.wsrd.purpose.size
-    = htonl (sizeof (struct TALER_WithdrawRequestPS));
+    = htonl (sizeof (wc.wsrd));
   wc.wsrd.purpose.purpose
     = htonl (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW);
   wc.wsrd.h_denomination_pub
@@ -452,7 +450,7 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
                       &wc.wsrd.h_coin_envelope);
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW,
-                                  &wc.wsrd.purpose,
+                                  &wc.wsrd,
                                   &wc.signature.eddsa_signature,
                                   &wc.wsrd.reserve_pub.eddsa_pub))
   {
@@ -463,7 +461,7 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_FORBIDDEN,
                                        TALER_EC_WITHDRAW_RESERVE_SIGNATURE_INVALID,
-                                       "reserve_sig");
+                                       NULL);
   }
 
 #if OPTIMISTIC_SIGN
@@ -480,13 +478,13 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_WITHDRAW_SIGNATURE_FAILED,
-                                       "Failed to sign");
+                                       NULL);
   }
 #endif
 
   /* run transaction and sign (if not optimistically signed before) */
   {
-    int mhd_ret;
+    MHD_RESULT mhd_ret;
 
     if (GNUNET_OK !=
         TEH_DB_run_transaction (connection,
@@ -510,7 +508,7 @@ TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
   GNUNET_JSON_parse_free (spec);
 
   {
-    int ret;
+    MHD_RESULT ret;
 
     ret = TALER_MHD_reply_json_pack (
       connection,
