@@ -634,14 +634,14 @@ main (int argc,
       GNUNET_PROGRAM_run (argc, argv,
                           "taler-exchange-wirewatch",
                           gettext_noop (
-                            "background process that watches for incomming wire transfers from customers"),
+                            "background process that watches for incoming wire transfers from customers"),
                           options,
                           &run, NULL))
   {
-    GNUNET_free ((void *) argv);
+    GNUNET_free_nz ((void *) argv);
     return GR_CMD_LINE_OPTIONS_WRONG;
   }
-  GNUNET_free ((void *) argv);
+  GNUNET_free_nz ((void *) argv);
   return global_ret;
 }
 
