@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 Taler Systems SA
+  Copyright (C) 2014, 2015, 2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ extern "C"
  * @brief Number of characters (plus 1 for 0-termination) we use to
  * represent currency names (i.e. EUR, USD, etc.).  We use 8+4 for
  * alignment in the `struct TALER_Amount`.  The amount is typically an
- * ISO 4217 currency code when an alpha-numeric 3-digit code is used.
+ * ISO 4217 currency code when an alphanumeric 3-digit code is used.
  * For regional currencies, the first character should be a "*" followed
  * by a region-specific name (i.e. "*BRETAGNEFR").
  */
@@ -51,6 +51,9 @@ extern "C"
  *
  * Note that we need sub-cent precision here as transaction fees might
  * be that low, and as we want to support microdonations.
+ *
+ * An actual `struct Amount a` thus represents
+ * "a.value + (a.fraction / #TALER_AMOUNT_FRAC_BASE)" units of "a.currency".
  */
 #define TALER_AMOUNT_FRAC_BASE 100000000
 
@@ -76,7 +79,7 @@ struct TALER_AmountNBO
   uint64_t value GNUNET_PACKED;
 
   /**
-   * Additinal fractional value, in NBO.
+   * Fraction (integer multiples of #TALER_AMOUNT_FRAC_BASE), in NBO.
    */
   uint32_t fraction GNUNET_PACKED;
 
@@ -100,7 +103,7 @@ struct TALER_Amount
   uint64_t value;
 
   /**
-   * Fraction (denominator of fraction)
+   * Fraction (integer multiples of #TALER_AMOUNT_FRAC_BASE).
    */
   uint32_t fraction;
 
@@ -120,7 +123,7 @@ struct TALER_Amount
  * @return #GNUNET_OK if the string is a valid monetary amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_string_to_amount (const char *str,
                         struct TALER_Amount *amount);
 
@@ -134,7 +137,7 @@ TALER_string_to_amount (const char *str,
  * @return #GNUNET_OK if the string is a valid amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_string_to_amount_nbo (const char *str,
                             struct TALER_AmountNBO *amount_nbo);
 
@@ -147,7 +150,7 @@ TALER_string_to_amount_nbo (const char *str,
  * @return #GNUNET_OK if @a cur is a valid currency specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_get_zero (const char *cur,
                        struct TALER_Amount *amount);
 
@@ -158,7 +161,7 @@ TALER_amount_get_zero (const char *cur,
  * @param amount amount to check
  * @return #GNUNET_OK if @a amount is valid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_is_valid (const struct TALER_Amount *amount);
 
 
@@ -202,6 +205,24 @@ TALER_amount_cmp (const struct TALER_Amount *a1,
                   const struct TALER_Amount *a2);
 
 
+/**
+ * Compare the value/fraction of two amounts.  Does not compare the currency.
+ * Comparing amounts of different currencies will cause the program to abort().
+ * If unsure, check with #TALER_amount_cmp_currency() first to be sure that
+ * the currencies of the two amounts are identical. NBO variant.
+ *
+ * @param a1 first amount
+ * @param a2 second amount
+ * @return result of the comparison
+ *         -1 if `a1 < a2`
+ *          1 if `a1 > a2`
+ *          0 if `a1 == a2`.
+ */
+int
+TALER_amount_cmp_nbo (const struct TALER_AmountNBO *a1,
+                      const struct TALER_AmountNBO *a2);
+
+
 /**
  * Test if @a a1 and @a a2 are the same currency.
  *
@@ -211,7 +232,7 @@ TALER_amount_cmp (const struct TALER_Amount *a1,
  *         #GNUNET_NO if the currencies are different
  *         #GNUNET_SYSERR if either amount is invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_cmp_currency (const struct TALER_Amount *a1,
                            const struct TALER_Amount *a2);
 
@@ -225,23 +246,60 @@ TALER_amount_cmp_currency (const struct TALER_Amount *a1,
  *         #GNUNET_NO if the currencies are different
  *         #GNUNET_SYSERR if either amount is invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_cmp_currency_nbo (const struct TALER_AmountNBO *a1,
                                const struct TALER_AmountNBO *a2);
 
 
+/**
+ * Possible results from calling #TALER_amount_subtract() and
+ * possibly other arithmetic operations. Negative values
+ * indicate that the operation did not generate a result.
+ */
+enum TALER_AmountArithmeticResult
+{
+
+  /**
+   * Operation succeeded, result is positive.
+   */
+  TALER_AAR_RESULT_POSITIVE = 1,
+
+  /**
+   * Operation succeeded, result is exactly zero.
+   */
+  TALER_AAR_RESULT_ZERO = 0,
+
+  /**
+   * Operation failed, the result would have been negative.
+   */
+  TALER_AAR_INVALID_NEGATIVE_RESULT = -1,
+
+  /**
+   * Operation failed, result outside of the representable range.
+   */
+  TALER_AAR_INVALID_RESULT_OVERFLOW = -2,
+
+  /**
+   * Operation failed, inputs could not be normalized.
+   */
+  TALER_AAR_INVALID_NORMALIZATION_FAILED = -3,
+
+  /**
+   * Operation failed, input currencies were not identical.
+   */
+  TALER_AAR_INVALID_CURRENCIES_INCOMPATIBLE = -4
+
+};
+
 /**
  * Perform saturating subtraction of amounts.
  *
  * @param[out] diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
  * @param a1 amount to subtract from
  * @param a2 amount to subtract
- * @return #GNUNET_OK if the subtraction worked,
- *         #GNUNET_NO if @a a1 = @a a2
- *         #GNUNET_SYSERR if @a a2 > @a a1 or currencies are incompatible;
- *                        @a diff is set to invalid
+ * @return operation status, negative on failures
  */
-int
+enum TALER_AmountArithmeticResult
 TALER_amount_subtract (struct TALER_Amount *diff,
                        const struct TALER_Amount *a1,
                        const struct TALER_Amount *a2);
@@ -253,10 +311,9 @@ TALER_amount_subtract (struct TALER_Amount *diff,
  * @param[out] sum where to store @a a1 + @a a2, set to "invalid" on overflow
  * @param a1 first amount to add
  * @param a2 second amount to add
- * @return #GNUNET_OK if the addition worked,
- *         #GNUNET_SYSERR on overflow
+ * @return operation status, negative on failures
  */
-int
+enum TALER_AmountArithmeticResult
 TALER_amount_add (struct TALER_Amount *sum,
                   const struct TALER_Amount *a1,
                   const struct TALER_Amount *a2);
@@ -322,7 +379,7 @@ TALER_amount2s (const struct TALER_Amount *amount);
  * @return #GNUNET_OK on success, #GNUNET_NO if rounding was unnecessary,
  *         #GNUNET_SYSERR if the amount or currency or @a round_unit was invalid
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_amount_round_down (struct TALER_Amount *amount,
                          const struct TALER_Amount *round_unit);
 
