@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -87,53 +87,58 @@ handle_deposit_confirmation_finished (void *cls,
 {
   const json_t *json = djson;
   struct TALER_AUDITOR_DepositConfirmationHandle *dh = cls;
-  enum TALER_ErrorCode ec;
+  struct TALER_AUDITOR_HttpResponse hr = {
+    .reply = json,
+    .http_status = (unsigned int) response_code
+  };
 
   dh->job = NULL;
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
-    ec = TALER_EC_NONE;
+    hr.ec = TALER_EC_NONE;
     break;
   case MHD_HTTP_BAD_REQUEST:
-    ec = TALER_JSON_get_error_code (json);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
     /* This should never happen, either us or the auditor is buggy
        (or API version conflict); just pass JSON reply to the application */
     break;
   case MHD_HTTP_FORBIDDEN:
-    ec = TALER_JSON_get_error_code (json);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, auditor says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    ec = TALER_JSON_get_error_code (json);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    ec = TALER_JSON_get_error_code (json);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
-    ec = TALER_JSON_get_error_code (json);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                ec);
-    GNUNET_break (0);
-    response_code = 0;
+                hr.ec);
+    GNUNET_break_op (0);
     break;
   }
   dh->cb (dh->cb_cls,
-          response_code,
-          ec,
-          json);
+          &hr);
   TALER_AUDITOR_deposit_confirmation_cancel (dh);
 }
 
@@ -143,7 +148,7 @@ handle_deposit_confirmation_finished (void *cls,
  *
  * @param h_wire hash of merchant wire details
  * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the auditor)
- * @param timestamp timestamp when the contract was finalized, must not be too far in the future
+ * @param exchange_timestamp timestamp when the deposit was received by the wallet
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the auditor (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param amount_without_fee the amount confirmed to be wired by the exchange to the merchant
  * @param coin_pub coin’s public key
@@ -160,7 +165,7 @@ handle_deposit_confirmation_finished (void *cls,
 static int
 verify_signatures (const struct GNUNET_HashCode *h_wire,
                    const struct GNUNET_HashCode *h_contract_terms,
-                   struct GNUNET_TIME_Absolute timestamp,
+                   struct GNUNET_TIME_Absolute exchange_timestamp,
                    struct GNUNET_TIME_Absolute refund_deadline,
                    const struct TALER_Amount *amount_without_fee,
                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -173,49 +178,57 @@ verify_signatures (const struct GNUNET_HashCode *h_wire,
                    struct GNUNET_TIME_Absolute ep_end,
                    const struct TALER_MasterSignatureP *master_sig)
 {
-  struct TALER_DepositConfirmationPS dc;
-  struct TALER_ExchangeSigningKeyValidityPS sv;
+  {
+    struct TALER_DepositConfirmationPS dc = {
+      .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT),
+      .purpose.size = htonl (sizeof (dc)),
+      .h_contract_terms = *h_contract_terms,
+      .h_wire = *h_wire,
+      .exchange_timestamp = GNUNET_TIME_absolute_hton (exchange_timestamp),
+      .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
+      .coin_pub = *coin_pub,
+      .merchant = *merchant_pub
+    };
 
-  dc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT);
-  dc.purpose.size = htonl (sizeof (struct TALER_DepositConfirmationPS));
-  dc.h_contract_terms = *h_contract_terms;
-  dc.h_wire = *h_wire;
-  dc.timestamp = GNUNET_TIME_absolute_hton (timestamp);
-  dc.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
-  TALER_amount_hton (&dc.amount_without_fee,
-                     amount_without_fee);
-  dc.coin_pub = *coin_pub;
-  dc.merchant = *merchant_pub;
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
-                                  &dc.purpose,
-                                  &exchange_sig->eddsa_signature,
-                                  &exchange_pub->eddsa_pub))
-  {
-    GNUNET_break_op (0);
-    TALER_LOG_WARNING ("Invalid signature on /deposit-confirmation request!\n");
+    TALER_amount_hton (&dc.amount_without_fee,
+                       amount_without_fee);
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
+                                    &dc,
+                                    &exchange_sig->eddsa_signature,
+                                    &exchange_pub->eddsa_pub))
     {
-      TALER_LOG_DEBUG ("... amount_without_fee was %s\n",
-                       TALER_amount2s (amount_without_fee));
+      GNUNET_break_op (0);
+      TALER_LOG_WARNING (
+        "Invalid signature on /deposit-confirmation request!\n");
+      {
+        TALER_LOG_DEBUG ("... amount_without_fee was %s\n",
+                         TALER_amount2s (amount_without_fee));
+      }
+      return GNUNET_SYSERR;
     }
-    return GNUNET_SYSERR;
   }
-  sv.purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY);
-  sv.purpose.size = htonl (sizeof (struct TALER_ExchangeSigningKeyValidityPS));
-  sv.master_public_key = *master_pub;
-  sv.start = GNUNET_TIME_absolute_hton (ep_start);
-  sv.expire = GNUNET_TIME_absolute_hton (ep_expire);
-  sv.end = GNUNET_TIME_absolute_hton (ep_end);
-  sv.signkey_pub = *exchange_pub;
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
-                                  &sv.purpose,
-                                  &master_sig->eddsa_signature,
-                                  &master_pub->eddsa_pub))
   {
-    GNUNET_break (0);
-    TALER_LOG_WARNING ("Invalid signature on exchange signing key!\n");
-    return GNUNET_SYSERR;
+    struct TALER_ExchangeSigningKeyValidityPS sv = {
+      .purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY),
+      .purpose.size = htonl (sizeof (sv)),
+      .master_public_key = *master_pub,
+      .start = GNUNET_TIME_absolute_hton (ep_start),
+      .expire = GNUNET_TIME_absolute_hton (ep_expire),
+      .end = GNUNET_TIME_absolute_hton (ep_end),
+      .signkey_pub = *exchange_pub
+    };
+
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
+                                    &sv,
+                                    &master_sig->eddsa_signature,
+                                    &master_pub->eddsa_pub))
+    {
+      GNUNET_break (0);
+      TALER_LOG_WARNING ("Invalid signature on exchange signing key!\n");
+      return GNUNET_SYSERR;
+    }
   }
   if (0 == GNUNET_TIME_absolute_get_remaining (ep_end).rel_value_us)
   {
@@ -243,7 +256,7 @@ verify_signatures (const struct GNUNET_HashCode *h_wire,
  * @param auditor the auditor handle; the auditor must be ready to operate
  * @param h_wire hash of merchant wire details
  * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the auditor)
- * @param timestamp timestamp when the contract was finalized, must not be too far in the future
+ * @param exchange_timestamp timestamp when deposit was received by the exchange
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the auditor (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param amount_without_fee the amount confirmed to be wired by the exchange to the merchant
  * @param coin_pub coin’s public key
@@ -265,7 +278,7 @@ TALER_AUDITOR_deposit_confirmation (
   struct TALER_AUDITOR_Handle *auditor,
   const struct GNUNET_HashCode *h_wire,
   const struct GNUNET_HashCode *h_contract_terms,
-  struct GNUNET_TIME_Absolute timestamp,
+  struct GNUNET_TIME_Absolute exchange_timestamp,
   struct GNUNET_TIME_Absolute refund_deadline,
   const struct TALER_Amount *amount_without_fee,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
@@ -285,7 +298,7 @@ TALER_AUDITOR_deposit_confirmation (
   json_t *deposit_confirmation_obj;
   CURL *eh;
 
-  (void) GNUNET_TIME_round_abs (&timestamp);
+  (void) GNUNET_TIME_round_abs (&exchange_timestamp);
   (void) GNUNET_TIME_round_abs (&refund_deadline);
   (void) GNUNET_TIME_round_abs (&ep_start);
   (void) GNUNET_TIME_round_abs (&ep_expire);
@@ -295,7 +308,7 @@ TALER_AUDITOR_deposit_confirmation (
   if (GNUNET_OK !=
       verify_signatures (h_wire,
                          h_contract_terms,
-                         timestamp,
+                         exchange_timestamp,
                          refund_deadline,
                          amount_without_fee,
                          coin_pub,
@@ -323,7 +336,8 @@ TALER_AUDITOR_deposit_confirmation (
                  "h_wire", GNUNET_JSON_from_data_auto (h_wire),
                  "h_contract_terms", GNUNET_JSON_from_data_auto (
                    h_contract_terms),
-                 "timestamp", GNUNET_JSON_from_time_abs (timestamp),
+                 "exchange_timestamp", GNUNET_JSON_from_time_abs (
+                   exchange_timestamp),
                  "refund_deadline", GNUNET_JSON_from_time_abs (refund_deadline),
                  "amount_without_fee", TALER_JSON_from_amount (
                    amount_without_fee),
