@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -122,6 +122,11 @@ struct TALER_AUDITOR_Handle
    * /version data of the auditor, only valid if
    * @e handshake_complete is past stage #MHS_VERSION.
    */
+  char *version;
+
+  /**
+   * Version information for the callback.
+   */
   struct TALER_AUDITOR_VersionInformation vi;
 
   /**
@@ -177,33 +182,21 @@ free_version_request (struct VersionRequest *vr)
 }
 
 
-/**
- * Free version data object.
- *
- * @param vi data to free (pointer itself excluded)
- */
-static void
-free_version_info (struct TALER_AUDITOR_VersionInformation *vi)
-{
-  GNUNET_free_non_null (vi->version);
-  vi->version = NULL;
-}
-
-
 /**
  * Decode the JSON in @a resp_obj from the /version response and store the data
  * in the @a key_data.
  *
  * @param[in] resp_obj JSON object to parse
- * @param[out] vi where to store the results we decoded
+ * @param[out] auditor where to store the results we decoded
  * @param[out] vc where to store version compatibility data
- * @return #GNUNET_OK on success, #GNUNET_SYSERR on error (malformed JSON)
+ * @return #TALER_EC_NONE on success
  */
-static int
+static enum TALER_ErrorCode
 decode_version_json (const json_t *resp_obj,
-                     struct TALER_AUDITOR_VersionInformation *vi,
+                     struct TALER_AUDITOR_Handle *auditor,
                      enum TALER_AUDITOR_VersionCompatibility *vc)
 {
+  struct TALER_AUDITOR_VersionInformation *vi = &auditor->vi;
   unsigned int age;
   unsigned int revision;
   unsigned int current;
@@ -219,7 +212,7 @@ decode_version_json (const json_t *resp_obj,
   if (JSON_OBJECT != json_typeof (resp_obj))
   {
     GNUNET_break_op (0);
-    return GNUNET_SYSERR;
+    return TALER_EC_JSON_INVALID;
   }
   /* check the version */
   if (GNUNET_OK !=
@@ -228,7 +221,7 @@ decode_version_json (const json_t *resp_obj,
                          NULL, NULL))
   {
     GNUNET_break_op (0);
-    return GNUNET_SYSERR;
+    return TALER_EC_JSON_INVALID;
   }
   if (3 != sscanf (ver,
                    "%u:%u:%u",
@@ -237,9 +230,10 @@ decode_version_json (const json_t *resp_obj,
                    &age))
   {
     GNUNET_break_op (0);
-    return GNUNET_SYSERR;
+    return TALER_EC_VERSION_MALFORMED;
   }
-  vi->version = GNUNET_strdup (ver);
+  auditor->version = GNUNET_strdup (ver);
+  vi->version = auditor->version;
   *vc = TALER_AUDITOR_VC_MATCH;
   if (TALER_PROTOCOL_CURRENT < current)
   {
@@ -253,7 +247,7 @@ decode_version_json (const json_t *resp_obj,
     if (TALER_PROTOCOL_CURRENT - TALER_PROTOCOL_AGE > current)
       *vc |= TALER_AUDITOR_VC_INCOMPATIBLE;
   }
-  return GNUNET_OK;
+  return TALER_EC_NONE;
 }
 
 
@@ -283,6 +277,10 @@ version_completed_cb (void *cls,
   struct VersionRequest *vr = cls;
   struct TALER_AUDITOR_Handle *auditor = vr->auditor;
   enum TALER_AUDITOR_VersionCompatibility vc;
+  struct TALER_AUDITOR_HttpResponse hr = {
+    .reply = resp_obj,
+    .http_status = (unsigned int) response_code
+  };
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Received version from URL `%s' with status %ld.\n",
@@ -293,6 +291,8 @@ version_completed_cb (void *cls,
   {
   case 0:
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
+    /* NOTE: this design is debatable. We MAY want to throw this error at the
+       client. We may then still additionally internally re-try. */
     free_version_request (vr);
     auditor->vr = NULL;
     GNUNET_assert (NULL == auditor->retry_task);
@@ -306,24 +306,28 @@ version_completed_cb (void *cls,
     {
       GNUNET_break_op (0);
       TALER_LOG_WARNING ("NULL body for a 200-OK /version\n");
-      response_code = 0;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_INVALID_RESPONSE;
       break;
     }
-    if (GNUNET_OK !=
-        decode_version_json (resp_obj,
-                             &auditor->vi,
-                             &vc))
+    hr.ec = decode_version_json (resp_obj,
+                                 auditor,
+                                 &vc);
+    if (TALER_EC_NONE != hr.ec)
     {
       GNUNET_break_op (0);
-      response_code = 0;
+      hr.http_status = 0;
       break;
     }
     auditor->retry_delay = GNUNET_TIME_UNIT_ZERO; /* restart quickly */
     break;
   default:
+    hr.ec = TALER_JSON_get_error_code (resp_obj);
+    hr.hint = TALER_JSON_get_error_hint (resp_obj);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u\n",
-                (unsigned int) response_code);
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) hr.ec);
     break;
   }
   if (MHD_HTTP_OK != response_code)
@@ -335,9 +339,9 @@ version_completed_cb (void *cls,
     auditor->vr = NULL;
     free_version_request (vr);
     auditor->state = MHS_FAILED;
-    free_version_info (&auditor->vi);
     /* notify application that we failed */
     auditor->version_cb (auditor->version_cb_cls,
+                         &hr,
                          NULL,
                          vc);
     return;
@@ -352,6 +356,7 @@ version_completed_cb (void *cls,
               auditor);
   /* notify application about the key information */
   auditor->version_cb (auditor->version_cb_cls,
+                       &hr,
                        &auditor->vi,
                        vc);
 }
@@ -495,7 +500,6 @@ request_version (void *cls)
                                   (long) 300));
   vr->job = GNUNET_CURL_job_add (auditor->ctx,
                                  eh,
-                                 GNUNET_NO,
                                  &version_completed_cb,
                                  vr);
   auditor->vr = vr;
@@ -520,7 +524,7 @@ TALER_AUDITOR_disconnect (struct TALER_AUDITOR_Handle *auditor)
     free_version_request (auditor->vr);
     auditor->vr = NULL;
   }
-  free_version_info (&auditor->vi);
+  GNUNET_free (auditor->version);
   if (NULL != auditor->retry_task)
   {
     GNUNET_SCHEDULER_cancel (auditor->retry_task);
