@@ -156,25 +156,24 @@ auditor_cb (void *cls,
   TALER_amount_ntoh (&amount_without_fee,
                      &dh->depconf.amount_without_fee);
   aie = GNUNET_new (struct TEAH_AuditorInteractionEntry);
-  aie->dch = TALER_AUDITOR_deposit_confirmation (ah,
-                                                 &dh->depconf.h_wire,
-                                                 &dh->depconf.h_contract_terms,
-                                                 GNUNET_TIME_absolute_ntoh (
-                                                   dh->depconf.timestamp),
-                                                 GNUNET_TIME_absolute_ntoh (
-                                                   dh->depconf.refund_deadline),
-                                                 &amount_without_fee,
-                                                 &dh->depconf.coin_pub,
-                                                 &dh->depconf.merchant,
-                                                 &dh->exchange_pub,
-                                                 &dh->exchange_sig,
-                                                 &key_state->master_pub,
-                                                 spk->valid_from,
-                                                 spk->valid_until,
-                                                 spk->valid_legal,
-                                                 &spk->master_sig,
-                                                 &TEAH_acc_confirmation_cb,
-                                                 aie);
+  aie->dch = TALER_AUDITOR_deposit_confirmation (
+    ah,
+    &dh->depconf.h_wire,
+    &dh->depconf.h_contract_terms,
+    GNUNET_TIME_absolute_ntoh (dh->depconf.exchange_timestamp),
+    GNUNET_TIME_absolute_ntoh (dh->depconf.refund_deadline),
+    &amount_without_fee,
+    &dh->depconf.coin_pub,
+    &dh->depconf.merchant,
+    &dh->exchange_pub,
+    &dh->exchange_sig,
+    &key_state->master_pub,
+    spk->valid_from,
+    spk->valid_until,
+    spk->valid_legal,
+    &spk->master_sig,
+    &TEAH_acc_confirmation_cb,
+    aie);
   return aie;
 }
 
@@ -197,8 +196,10 @@ verify_deposit_signature_ok (struct TALER_EXCHANGE_DepositHandle *dh,
 {
   const struct TALER_EXCHANGE_Keys *key_state;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_fixed_auto ("sig", exchange_sig),
-    GNUNET_JSON_spec_fixed_auto ("pub", exchange_pub),
+    GNUNET_JSON_spec_fixed_auto ("exchange_sig", exchange_sig),
+    GNUNET_JSON_spec_fixed_auto ("exchange_pub", exchange_pub),
+    TALER_JSON_spec_absolute_time_nbo ("exchange_timestamp",
+                                       &dh->depconf.exchange_timestamp),
     GNUNET_JSON_spec_end ()
   };
 
@@ -220,7 +221,7 @@ verify_deposit_signature_ok (struct TALER_EXCHANGE_DepositHandle *dh,
   }
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
-                                  &dh->depconf.purpose,
+                                  &dh->depconf,
                                   &exchange_sig->eddsa_signature,
                                   &exchange_pub->eddsa_pub))
   {
@@ -245,13 +246,18 @@ verify_deposit_signature_ok (struct TALER_EXCHANGE_DepositHandle *dh,
  * @return #GNUNET_OK if the signature(s) is valid, #GNUNET_SYSERR if not
  */
 static int
-verify_deposit_signature_forbidden (
+verify_deposit_signature_conflict (
   const struct TALER_EXCHANGE_DepositHandle *dh,
   const json_t *json)
 {
   json_t *history;
   struct TALER_Amount total;
+  enum TALER_ErrorCode ec;
+  struct GNUNET_HashCode h_denom_pub;
 
+  memset (&h_denom_pub,
+          0,
+          sizeof (h_denom_pub));
   history = json_object_get (json,
                              "history");
   if (GNUNET_OK !=
@@ -259,30 +265,46 @@ verify_deposit_signature_forbidden (
                                           dh->dki.value.currency,
                                           &dh->depconf.coin_pub,
                                           history,
+                                          &h_denom_pub,
                                           &total))
   {
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  if (GNUNET_OK !=
-      TALER_amount_add (&total,
-                        &total,
-                        &dh->amount_with_fee))
+  ec = TALER_JSON_get_error_code (json);
+  switch (ec)
   {
-    /* clearly not OK if our transaction would have caused
-       the overflow... */
-    return GNUNET_OK;
-  }
+  case TALER_EC_DEPOSIT_INSUFFICIENT_FUNDS:
+    if (0 >
+        TALER_amount_add (&total,
+                          &total,
+                          &dh->amount_with_fee))
+    {
+      /* clearly not OK if our transaction would have caused
+         the overflow... */
+      return GNUNET_OK;
+    }
 
-  if (0 >= TALER_amount_cmp (&total,
-                             &dh->dki.value))
-  {
-    /* transaction should have still fit */
-    GNUNET_break (0);
+    if (0 >= TALER_amount_cmp (&total,
+                               &dh->dki.value))
+    {
+      /* transaction should have still fit */
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
+    /* everything OK, proof of double-spending was provided */
+    return GNUNET_OK;
+  case TALER_EC_COIN_CONFLICTING_DENOMINATION_KEY:
+    if (0 != GNUNET_memcmp (&dh->dki.h_key,
+                            &h_denom_pub))
+      return GNUNET_OK; /* indeed, proof with different denomination key provided */
+    /* invalid proof provided */
+    return GNUNET_SYSERR;
+  default:
+    /* unexpected error code */
+    GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  /* everything OK, proof of double-spending was provided */
-  return GNUNET_OK;
 }
 
 
@@ -305,13 +327,16 @@ handle_deposit_finished (void *cls,
   struct TALER_ExchangeSignatureP *es = NULL;
   struct TALER_ExchangePublicKeyP *ep = NULL;
   const json_t *j = response;
-  enum TALER_ErrorCode ec;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = j,
+    .http_status = (unsigned int) response_code
+  };
 
   dh->job = NULL;
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -321,66 +346,72 @@ handle_deposit_finished (void *cls,
                                      &exchange_pub))
     {
       GNUNET_break_op (0);
-      response_code = 0;
-      ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
     }
     else
     {
       es = &exchange_sig;
       ep = &exchange_pub;
-      ec = TALER_EC_NONE;
     }
     break;
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_CONFLICT:
     /* Double spending; check signatures on transaction history */
-    ec = TALER_JSON_get_error_code (j);
     if (GNUNET_OK !=
-        verify_deposit_signature_forbidden (dh,
-                                            j))
+        verify_deposit_signature_conflict (dh,
+                                           j))
     {
       GNUNET_break_op (0);
-      response_code = 0;
-      ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
+    }
+    else
+    {
+      hr.ec = TALER_JSON_get_error_code (j);
+      hr.hint = TALER_JSON_get_error_hint (j);
     }
     break;
   case MHD_HTTP_FORBIDDEN:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* Nothing really to verify, exchange says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* Nothing really to verify, this should never
-       happen, we should pass the JSON reply to the application */
+     happen, we should pass the JSON reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                ec);
-    GNUNET_break (0);
-    response_code = 0;
+                hr.ec);
+    GNUNET_break_op (0);
     break;
   }
   dh->cb (dh->cb_cls,
-          response_code,
-          ec,
+          &hr,
+          GNUNET_TIME_absolute_ntoh (dh->depconf.exchange_timestamp),
           es,
-          ep,
-          j);
+          ep);
   TALER_EXCHANGE_deposit_cancel (dh);
 }
 
@@ -416,51 +447,60 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
                    struct GNUNET_TIME_Absolute refund_deadline,
                    const struct TALER_CoinSpendSignatureP *coin_sig)
 {
-  struct TALER_DepositRequestPS dr;
-  struct TALER_CoinPublicInfo coin_info;
-
-  dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
-  dr.purpose.size = htonl (sizeof (struct TALER_DepositRequestPS));
-  dr.h_contract_terms = *h_contract_terms;
-  dr.h_wire = *h_wire;
-  dr.timestamp = GNUNET_TIME_absolute_hton (timestamp);
-  dr.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
-  TALER_amount_hton (&dr.amount_with_fee,
-                     amount);
-  TALER_amount_hton (&dr.deposit_fee,
-                     &dki->fee_deposit);
-  dr.merchant = *merchant_pub;
-  dr.coin_pub = *coin_pub;
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
-                                  &dr.purpose,
-                                  &coin_sig->eddsa_signature,
-                                  &coin_pub->eddsa_pub))
   {
-    GNUNET_break_op (0);
-    TALER_LOG_WARNING ("Invalid coin signature on /deposit request!\n");
-    {
-      TALER_LOG_DEBUG ("... amount_with_fee was %s\n",
-                       TALER_amount2s (amount));
-      TALER_LOG_DEBUG ("... deposit_fee was %s\n",
-                       TALER_amount2s (&dki->fee_deposit));
-    }
+    struct TALER_DepositRequestPS dr = {
+      .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT),
+      .purpose.size = htonl (sizeof (dr)),
+      .h_contract_terms = *h_contract_terms,
+      .h_wire = *h_wire,
+      .h_denom_pub = *denom_pub_hash,
+      .wallet_timestamp = GNUNET_TIME_absolute_hton (timestamp),
+      .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
+      .merchant = *merchant_pub,
+      .coin_pub = *coin_pub
+    };
 
-    return GNUNET_SYSERR;
+    TALER_amount_hton (&dr.amount_with_fee,
+                       amount);
+    TALER_amount_hton (&dr.deposit_fee,
+                       &dki->fee_deposit);
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
+                                    &dr,
+                                    &coin_sig->eddsa_signature,
+                                    &coin_pub->eddsa_pub))
+    {
+      GNUNET_break_op (0);
+      TALER_LOG_WARNING ("Invalid coin signature on /deposit request!\n");
+      {
+        TALER_LOG_DEBUG ("... amount_with_fee was %s\n",
+                         TALER_amount2s (amount));
+        TALER_LOG_DEBUG ("... deposit_fee was %s\n",
+                         TALER_amount2s (&dki->fee_deposit));
+      }
+      return GNUNET_SYSERR;
+    }
   }
 
   /* check coin signature */
-  coin_info.coin_pub = *coin_pub;
-  coin_info.denom_pub_hash = *denom_pub_hash;
-  coin_info.denom_sig = *denom_sig;
-  if (GNUNET_YES !=
-      TALER_test_coin_valid (&coin_info,
-                             denom_pub))
   {
-    GNUNET_break_op (0);
-    TALER_LOG_WARNING ("Invalid coin passed for /deposit\n");
-    return GNUNET_SYSERR;
+    struct TALER_CoinPublicInfo coin_info = {
+      .coin_pub = *coin_pub,
+      .denom_pub_hash = *denom_pub_hash,
+      .denom_sig = *denom_sig
+    };
+
+    if (GNUNET_YES !=
+        TALER_test_coin_valid (&coin_info,
+                               denom_pub))
+    {
+      GNUNET_break_op (0);
+      TALER_LOG_WARNING ("Invalid coin passed for /deposit\n");
+      return GNUNET_SYSERR;
+    }
   }
+
+  /* Check coin does make a contribution */
   if (0 < TALER_amount_cmp (&dki->fee_deposit,
                             amount))
   {
@@ -472,6 +512,62 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
 }
 
 
+/**
+ * Sign a deposit permission.  Function for wallets.
+ *
+ * @param amount the amount to be deposited
+ * @param deposit_fee the deposit fee we expect to pay
+ * @param h_wire hash of the merchant’s account details
+ * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
+ * @param h_denom_pub hash of the coin denomination's public key
+ * @param coin_priv coin’s private key
+ * @param wallet_timestamp timestamp when the contract was finalized, must not be too far in the future
+ * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
+ * @param refund_deadline date until which the merchant can issue a refund to the customer via the exchange (can be zero if refunds are not allowed); must not be after the @a wire_deadline
+ * @param[out] coin_sig set to the signature made with purpose #TALER_SIGNATURE_WALLET_COIN_DEPOSIT
+ */
+void
+TALER_EXCHANGE_deposit_permission_sign (
+  const struct TALER_Amount *amount,
+  const struct TALER_Amount *deposit_fee,
+  const struct GNUNET_HashCode *h_wire,
+  const struct GNUNET_HashCode *h_contract_terms,
+  const struct GNUNET_HashCode *h_denom_pub,
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+  struct GNUNET_TIME_Absolute wallet_timestamp,
+  const struct TALER_MerchantPublicKeyP *merchant_pub,
+  struct GNUNET_TIME_Absolute refund_deadline,
+  struct TALER_CoinSpendSignatureP *coin_sig)
+{
+  struct TALER_DepositRequestPS dr = {
+    .purpose.size = htonl
+                      (sizeof (dr)),
+    .purpose.purpose = htonl
+                         (TALER_SIGNATURE_WALLET_COIN_DEPOSIT),
+    .h_contract_terms = *h_contract_terms,
+    .h_wire = *h_wire,
+    .h_denom_pub = *h_denom_pub,
+    .wallet_timestamp = GNUNET_TIME_absolute_hton (wallet_timestamp),
+    .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
+    .merchant = *merchant_pub
+  };
+
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_TIME_round_abs (&wallet_timestamp));
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_TIME_round_abs (&refund_deadline));
+  GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv->eddsa_priv,
+                                      &dr.coin_pub.eddsa_pub);
+  TALER_amount_hton (&dr.amount_with_fee,
+                     amount);
+  TALER_amount_hton (&dr.deposit_fee,
+                     deposit_fee);
+  GNUNET_CRYPTO_eddsa_sign (&coin_priv->eddsa_priv,
+                            &dr,
+                            &coin_sig->eddsa_signature);
+}
+
+
 /**
  * Submit a deposit permission to the exchange and get the exchange's response.
  * Note that while we return the response verbatim to the caller for
@@ -535,11 +631,11 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
     char pub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
     char *end;
 
-    end = GNUNET_STRINGS_data_to_string (coin_pub,
-                                         sizeof (struct
-                                                 TALER_CoinSpendPublicKeyP),
-                                         pub_str,
-                                         sizeof (pub_str));
+    end = GNUNET_STRINGS_data_to_string (
+      coin_pub,
+      sizeof (struct TALER_CoinSpendPublicKeyP),
+      pub_str,
+      sizeof (pub_str));
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
@@ -571,7 +667,7 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
     GNUNET_break (0);
     return NULL;
   }
-  if (GNUNET_SYSERR ==
+  if (0 >
       TALER_amount_subtract (&amount_without_fee,
                              amount,
                              &dki->fee_deposit))
@@ -643,7 +739,7 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
     TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT);
   dh->depconf.h_contract_terms = *h_contract_terms;
   dh->depconf.h_wire = h_wire;
-  dh->depconf.timestamp = GNUNET_TIME_absolute_hton (timestamp);
+  /* dh->depconf.exchange_timestamp; -- initialized later from exchange reply! */
   dh->depconf.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
   TALER_amount_hton (&dh->depconf.amount_without_fee,
                      &amount_without_fee);
