@@ -39,18 +39,18 @@
  * Which version of the Taler protocol is implemented
  * by this library?  Used to determine compatibility.
  */
-#define TALER_PROTOCOL_CURRENT 7
+#define EXCHANGE_PROTOCOL_CURRENT 8
 
 /**
  * How many versions are we backwards compatible with?
  */
-#define TALER_PROTOCOL_AGE 0
+#define EXCHANGE_PROTOCOL_AGE 0
 
 /**
  * Current version for (local) JSON serialization of persisted
  * /keys data.
  */
-#define TALER_SERIALIZATION_FORMAT_VERSION 0
+#define EXCHANGE_SERIALIZATION_FORMAT_VERSION 0
 
 
 /**
@@ -265,27 +265,22 @@ struct KeysRequest
  * auditor's /deposit-confirmation handler.
  *
  * @param cls closure of type `struct TEAH_AuditorInteractionEntry *`
- * @param http_status HTTP status code, 200 on success
- * @param ec taler protocol error status code, 0 on success
- * @param json raw json response
+ * @param hr HTTP response
  */
 void
 TEAH_acc_confirmation_cb (void *cls,
-                          unsigned int http_status,
-                          enum TALER_ErrorCode ec,
-                          const json_t *json)
+                          const struct TALER_AUDITOR_HttpResponse *hr)
 {
   struct TEAH_AuditorInteractionEntry *aie = cls;
   struct TEAH_AuditorListEntry *ale = aie->ale;
 
-  (void) json;
-  if (MHD_HTTP_OK != http_status)
+  if (MHD_HTTP_OK != hr->http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Failed to submit deposit confirmation to auditor `%s' with HTTP status %d (EC: %d). This is acceptable if it does not happen often.\n",
                 ale->auditor_url,
-                http_status,
-                (int) ec);
+                hr->http_status,
+                hr->ec);
   }
   GNUNET_CONTAINER_DLL_remove (ale->ai_head,
                                ale->ai_tail,
@@ -373,19 +368,18 @@ parse_json_signkey (struct TALER_EXCHANGE_SigningPublicKey *sign_key,
                     json_t *sign_key_obj,
                     const struct TALER_MasterPublicKeyP *master_key)
 {
-  struct TALER_ExchangeSigningKeyValidityPS sign_key_issue;
   struct TALER_MasterSignatureP sign_key_issue_sig;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("master_sig",
                                  &sign_key_issue_sig),
     GNUNET_JSON_spec_fixed_auto ("key",
                                  &sign_key->key),
-    GNUNET_JSON_spec_absolute_time ("stamp_start",
-                                    &sign_key->valid_from),
-    GNUNET_JSON_spec_absolute_time ("stamp_expire",
-                                    &sign_key->valid_until),
-    GNUNET_JSON_spec_absolute_time ("stamp_end",
-                                    &sign_key->valid_legal),
+    TALER_JSON_spec_absolute_time ("stamp_start",
+                                   &sign_key->valid_from),
+    TALER_JSON_spec_absolute_time ("stamp_expire",
+                                   &sign_key->valid_until),
+    TALER_JSON_spec_absolute_time ("stamp_end",
+                                   &sign_key->valid_legal),
     GNUNET_JSON_spec_end ()
   };
 
@@ -400,23 +394,26 @@ parse_json_signkey (struct TALER_EXCHANGE_SigningPublicKey *sign_key,
 
   if (! check_sigs)
     return GNUNET_OK;
-  sign_key_issue.signkey_pub = sign_key->key;
-  sign_key_issue.purpose.purpose = htonl (
-    TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY);
-  sign_key_issue.purpose.size = htonl (sizeof (struct
-                                               TALER_ExchangeSigningKeyValidityPS));
-  sign_key_issue.master_public_key = *master_key;
-  sign_key_issue.start = GNUNET_TIME_absolute_hton (sign_key->valid_from);
-  sign_key_issue.expire = GNUNET_TIME_absolute_hton (sign_key->valid_until);
-  sign_key_issue.end = GNUNET_TIME_absolute_hton (sign_key->valid_legal);
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
-                                  &sign_key_issue.purpose,
-                                  &sign_key_issue_sig.eddsa_signature,
-                                  &master_key->eddsa_pub))
   {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
+    struct TALER_ExchangeSigningKeyValidityPS sign_key_issue = {
+      .purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY),
+      .purpose.size = htonl (sizeof (sign_key_issue)),
+      .signkey_pub = sign_key->key,
+      .master_public_key = *master_key,
+      .start = GNUNET_TIME_absolute_hton (sign_key->valid_from),
+      .expire = GNUNET_TIME_absolute_hton (sign_key->valid_until),
+      .end = GNUNET_TIME_absolute_hton (sign_key->valid_legal)
+    };
+
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
+                                    &sign_key_issue,
+                                    &sign_key_issue_sig.eddsa_signature,
+                                    &master_key->eddsa_pub))
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
   }
   sign_key->master_sig = sign_key_issue_sig;
   return GNUNET_OK;
@@ -441,18 +438,17 @@ parse_json_denomkey (struct TALER_EXCHANGE_DenomPublicKey *denom_key,
                      struct TALER_MasterPublicKeyP *master_key,
                      struct GNUNET_HashContext *hash_context)
 {
-  struct TALER_DenominationKeyValidityPS denom_key_issue;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("master_sig",
                                  &denom_key->master_sig),
-    GNUNET_JSON_spec_absolute_time ("stamp_expire_deposit",
-                                    &denom_key->expire_deposit),
-    GNUNET_JSON_spec_absolute_time ("stamp_expire_withdraw",
-                                    &denom_key->withdraw_valid_until),
-    GNUNET_JSON_spec_absolute_time ("stamp_start",
-                                    &denom_key->valid_from),
-    GNUNET_JSON_spec_absolute_time ("stamp_expire_legal",
-                                    &denom_key->expire_legal),
+    TALER_JSON_spec_absolute_time ("stamp_expire_deposit",
+                                   &denom_key->expire_deposit),
+    TALER_JSON_spec_absolute_time ("stamp_expire_withdraw",
+                                   &denom_key->withdraw_valid_until),
+    TALER_JSON_spec_absolute_time ("stamp_start",
+                                   &denom_key->valid_from),
+    TALER_JSON_spec_absolute_time ("stamp_expire_legal",
+                                   &denom_key->expire_legal),
     TALER_JSON_spec_amount ("value",
                             &denom_key->value),
     TALER_JSON_spec_amount ("fee_withdraw",
@@ -479,46 +475,50 @@ parse_json_denomkey (struct TALER_EXCHANGE_DenomPublicKey *denom_key,
 
   GNUNET_CRYPTO_rsa_public_key_hash (denom_key->key.rsa_public_key,
                                      &denom_key->h_key);
+  if (NULL != hash_context)
+    GNUNET_CRYPTO_hash_context_read (hash_context,
+                                     &denom_key->h_key,
+                                     sizeof (struct GNUNET_HashCode));
   if (! check_sigs)
     return GNUNET_OK;
-  memset (&denom_key_issue,
-          0,
-          sizeof (denom_key_issue));
-  denom_key_issue.purpose.purpose
-    = htonl (TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY);
-  denom_key_issue.purpose.size
-    = htonl (sizeof (struct TALER_DenominationKeyValidityPS));
-  denom_key_issue.master = *master_key;
-  denom_key_issue.denom_hash = denom_key->h_key;
-  denom_key_issue.start = GNUNET_TIME_absolute_hton (denom_key->valid_from);
-  denom_key_issue.expire_withdraw = GNUNET_TIME_absolute_hton (
-    denom_key->withdraw_valid_until);
-  denom_key_issue.expire_deposit = GNUNET_TIME_absolute_hton (
-    denom_key->expire_deposit);
-  denom_key_issue.expire_legal = GNUNET_TIME_absolute_hton (
-    denom_key->expire_legal);
-  TALER_amount_hton (&denom_key_issue.value,
-                     &denom_key->value);
-  TALER_amount_hton (&denom_key_issue.fee_withdraw,
-                     &denom_key->fee_withdraw);
-  TALER_amount_hton (&denom_key_issue.fee_deposit,
-                     &denom_key->fee_deposit);
-  TALER_amount_hton (&denom_key_issue.fee_refresh,
-                     &denom_key->fee_refresh);
-  TALER_amount_hton (&denom_key_issue.fee_refund,
-                     &denom_key->fee_refund);
-  EXITIF (GNUNET_SYSERR ==
-          GNUNET_CRYPTO_eddsa_verify (
-            TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY,
-            &denom_key_issue.purpose,
-            &denom_key->master_sig.eddsa_signature,
-            &master_key->eddsa_pub));
-  GNUNET_CRYPTO_hash_context_read (hash_context,
-                                   &denom_key_issue.denom_hash,
-                                   sizeof (struct GNUNET_HashCode));
+  {
+    struct TALER_DenominationKeyValidityPS denom_key_issue =  {
+      .purpose.purpose
+        = htonl (TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY),
+      .purpose.size = htonl (sizeof (denom_key_issue)),
+      .master = *master_key,
+      .denom_hash = denom_key->h_key,
+      .start = GNUNET_TIME_absolute_hton (denom_key->valid_from),
+      .expire_withdraw
+        = GNUNET_TIME_absolute_hton (denom_key->withdraw_valid_until),
+      .expire_deposit = GNUNET_TIME_absolute_hton (denom_key->expire_deposit),
+      .expire_legal = GNUNET_TIME_absolute_hton (denom_key->expire_legal)
+    };
+
+    TALER_amount_hton (&denom_key_issue.value,
+                       &denom_key->value);
+    TALER_amount_hton (&denom_key_issue.fee_withdraw,
+                       &denom_key->fee_withdraw);
+    TALER_amount_hton (&denom_key_issue.fee_deposit,
+                       &denom_key->fee_deposit);
+    TALER_amount_hton (&denom_key_issue.fee_refresh,
+                       &denom_key->fee_refresh);
+    TALER_amount_hton (&denom_key_issue.fee_refund,
+                       &denom_key->fee_refund);
+    EXITIF (GNUNET_SYSERR ==
+            GNUNET_CRYPTO_eddsa_verify (
+              TALER_SIGNATURE_MASTER_DENOMINATION_KEY_VALIDITY,
+              &denom_key_issue,
+              &denom_key->master_sig.eddsa_signature,
+              &master_key->eddsa_pub));
+  }
   return GNUNET_OK;
 
 EXITIF_exit:
+  /* invalidate denom_key, just to be sure */
+  memset (denom_key,
+          0,
+          sizeof (*denom_key));
   GNUNET_JSON_parse_free (spec);
   return GNUNET_SYSERR;
 }
@@ -635,7 +635,7 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
 
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_AUDITOR_EXCHANGE_KEYS,
-                                      &kv.purpose,
+                                      &kv,
                                       &auditor_sig.eddsa_sig,
                                       &auditor->auditor_pub.eddsa_pub))
       {
@@ -659,12 +659,14 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
  * auditor as 'up'.
  *
  * @param cls closure, a `struct TEAH_AuditorListEntry *`
+ * @param hr http response from the auditor
  * @param vi basic information about the auditor
  * @param compat protocol compatibility information
  */
 static void
 auditor_version_cb (
   void *cls,
+  const struct TALER_AUDITOR_HttpResponse *hr,
   const struct TALER_AUDITOR_VersionInformation *vi,
   enum TALER_AUDITOR_VersionCompatibility compat)
 {
@@ -783,7 +785,7 @@ denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
 
   denom1->key.rsa_public_key = NULL;
   denom2->key.rsa_public_key = NULL;
-  /* Then procede with the rest of the object.  */
+  /* Then proceed with the rest of the object.  */
   ret = GNUNET_memcmp (denom1,
                        denom2);
   denom1->revoked = r1;
@@ -799,7 +801,7 @@ denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
  * and store the data in the @a key_data.
  *
  * @param[in] resp_obj JSON object to parse
- * @param check_sig #GNUNET_YES if we should check the signature
+ * @param check_sig true if we should check the signature
  * @param[out] key_data where to store the results we decoded
  * @param[out] vc where to store version compatibility data
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
@@ -807,16 +809,13 @@ denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
  */
 static int
 decode_keys_json (const json_t *resp_obj,
-                  int check_sig,
+                  bool check_sig,
                   struct TALER_EXCHANGE_Keys *key_data,
                   enum TALER_EXCHANGE_VersionCompatibility *vc)
 {
   struct TALER_ExchangeSignatureP sig;
   struct GNUNET_HashContext *hash_context;
   struct TALER_ExchangePublicKeyP pub;
-  unsigned int age;
-  unsigned int revision;
-  unsigned int current;
   struct GNUNET_JSON_Specification mspec[] = {
     GNUNET_JSON_spec_fixed_auto ("eddsa_sig",
                                  &sig),
@@ -826,10 +825,10 @@ decode_keys_json (const json_t *resp_obj,
        check_sig is false! */
     GNUNET_JSON_spec_fixed_auto ("master_public_key",
                                  &key_data->master_pub),
-    GNUNET_JSON_spec_absolute_time ("list_issue_date",
-                                    &key_data->list_issue_date),
-    GNUNET_JSON_spec_relative_time ("reserve_closing_delay",
-                                    &key_data->reserve_closing_delay),
+    TALER_JSON_spec_absolute_time ("list_issue_date",
+                                   &key_data->list_issue_date),
+    TALER_JSON_spec_relative_time ("reserve_closing_delay",
+                                   &key_data->reserve_closing_delay),
     GNUNET_JSON_spec_end ()
   };
 
@@ -841,6 +840,9 @@ decode_keys_json (const json_t *resp_obj,
   /* check the version */
   {
     const char *ver;
+    unsigned int age;
+    unsigned int revision;
+    unsigned int current;
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_string ("version",
                                &ver),
@@ -865,16 +867,16 @@ decode_keys_json (const json_t *resp_obj,
       return GNUNET_SYSERR;
     }
     *vc = TALER_EXCHANGE_VC_MATCH;
-    if (TALER_PROTOCOL_CURRENT < current)
+    if (EXCHANGE_PROTOCOL_CURRENT < current)
     {
       *vc |= TALER_EXCHANGE_VC_NEWER;
-      if (TALER_PROTOCOL_CURRENT < current - age)
+      if (EXCHANGE_PROTOCOL_CURRENT < current - age)
         *vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
     }
-    if (TALER_PROTOCOL_CURRENT > current)
+    if (EXCHANGE_PROTOCOL_CURRENT > current)
     {
       *vc |= TALER_EXCHANGE_VC_OLDER;
-      if (TALER_PROTOCOL_CURRENT - TALER_PROTOCOL_AGE > current)
+      if (EXCHANGE_PROTOCOL_CURRENT - EXCHANGE_PROTOCOL_AGE > current)
         *vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
     }
     key_data->version = GNUNET_strdup (ver);
@@ -885,7 +887,6 @@ decode_keys_json (const json_t *resp_obj,
           GNUNET_JSON_parse (resp_obj,
                              (check_sig) ? mspec : &mspec[2],
                              NULL, NULL));
-
   /* parse the master public key and issue date of the response */
   if (check_sig)
     hash_context = GNUNET_CRYPTO_hash_context_start ();
@@ -1012,13 +1013,10 @@ decode_keys_json (const json_t *resp_obj,
             aix->num_denom_keys,
             ai.num_denom_keys);
 
-          GNUNET_array_grow (aix->denom_keys,
-                             aix->num_denom_keys,
-                             aix->num_denom_keys + ai.num_denom_keys);
-          memcpy (&aix->denom_keys[aix->num_denom_keys - ai.num_denom_keys],
-                  ai.denom_keys,
-                  ai.num_denom_keys * sizeof (struct
-                                              TALER_EXCHANGE_AuditorDenominationInfo));
+          GNUNET_array_concatenate (aix->denom_keys,
+                                    aix->num_denom_keys,
+                                    ai.denom_keys,
+                                    ai.num_denom_keys);
           break;
         }
       }
@@ -1094,7 +1092,7 @@ decode_keys_json (const json_t *resp_obj,
                                              &pub));
     EXITIF (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_KEY_SET,
-                                        &ks.purpose,
+                                        &ks,
                                         &sig.eddsa_signature,
                                         &pub.eddsa_pub));
   }
@@ -1136,7 +1134,7 @@ free_key_data (struct TALER_EXCHANGE_Keys *key_data)
   GNUNET_array_grow (key_data->auditors,
                      key_data->auditors_size,
                      0);
-  GNUNET_free_non_null (key_data->version);
+  GNUNET_free (key_data->version);
   key_data->version = NULL;
 }
 
@@ -1197,27 +1195,27 @@ TALER_EXCHANGE_set_last_denom (struct TALER_EXCHANGE_Handle *exchange,
  * not trigger download.
  *
  * @param exchange exchange to check keys for
- * @param force_download #GNUNET_YES to force download even if /keys is still valid
- * @param pull_all_keys if #GNUNET_YES, then the exchange state is reset to #MHS_INIT,
- *        and all denoms will be redownloaded.
+ * @param flags options controlling when to download what
  * @return until when the response is current, 0 if we are re-downloading
  */
 struct GNUNET_TIME_Absolute
 TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange,
-                                   int force_download,
-                                   int pull_all_keys)
+                                   enum TALER_EXCHANGE_CheckKeysFlags flags)
 {
+  bool force_download = 0 != (flags & TALER_EXCHANGE_CKF_FORCE_DOWNLOAD);
+  bool pull_all_keys = 0 != (flags & TALER_EXCHANGE_CKF_PULL_ALL_KEYS);
+
   if (NULL != exchange->kr)
     return GNUNET_TIME_UNIT_ZERO_ABS;
 
-  if (GNUNET_YES == pull_all_keys)
+  if (pull_all_keys)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Forcing re-download of all exchange keys\n");
     GNUNET_break (GNUNET_YES == force_download);
     exchange->state = MHS_INIT;
   }
-  if ( (GNUNET_NO == force_download) &&
+  if ( (! force_download) &&
        (0 < GNUNET_TIME_absolute_get_remaining (
           exchange->key_data_expiration).rel_value_us) )
     return exchange->key_data_expiration;
@@ -1247,6 +1245,10 @@ keys_completed_cb (void *cls,
   struct TALER_EXCHANGE_Keys kd_old;
   enum TALER_EXCHANGE_VersionCompatibility vc;
   const json_t *j = resp_obj;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = j,
+    .http_status = (unsigned int) response_code
+  };
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Received keys from URL `%s' with status %ld.\n",
@@ -1311,19 +1313,20 @@ keys_completed_cb (void *cls,
                          aold->num_denom_keys);
       memcpy (anew->denom_keys,
               aold->denom_keys,
-              aold->num_denom_keys * sizeof (struct
-                                             TALER_EXCHANGE_AuditorDenominationInfo));
+              aold->num_denom_keys
+              * sizeof (struct TALER_EXCHANGE_AuditorDenominationInfo));
     }
 
     /* Old auditors got just copied into new ones.  */
     if (GNUNET_OK !=
         decode_keys_json (j,
-                          GNUNET_YES,
+                          true,
                           &kd,
                           &vc))
     {
       TALER_LOG_ERROR ("Could not decode /keys response\n");
-      response_code = 0;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_KEYS_INVALID;
       for (unsigned int i = 0; i<kd.num_auditors; i++)
       {
         struct TALER_EXCHANGE_AuditorInformation *anew = &kd.auditors[i];
@@ -1349,9 +1352,12 @@ keys_completed_cb (void *cls,
     exchange->retry_delay = GNUNET_TIME_UNIT_ZERO;
     break;
   default:
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u\n",
-                (unsigned int) response_code);
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) hr.ec);
     break;
   }
   exchange->key_data = kd;
@@ -1375,6 +1381,7 @@ keys_completed_cb (void *cls,
     free_key_data (&kd_old);
     /* notify application that we failed */
     exchange->cert_cb (exchange->cert_cb_cls,
+                       &hr,
                        NULL,
                        vc);
     return;
@@ -1389,6 +1396,7 @@ keys_completed_cb (void *cls,
   update_auditors (exchange);
   /* notify application about the key information */
   exchange->cert_cb (exchange->cert_cb_cls,
+                     &hr,
                      &exchange->key_data,
                      vc);
   free_key_data (&kd_old);
@@ -1491,7 +1499,7 @@ parse_date_string (const char *date,
 /**
  * Function called for each header in the HTTP /keys response.
  * Finds the "Expire:" header and parses it, storing the result
- * in the "expire" field fo the keys request.
+ * in the "expire" field of the keys request.
  *
  * @param buffer header data received
  * @param size size of an item in @a buffer
@@ -1559,11 +1567,16 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
                            &keys),
     GNUNET_JSON_spec_string ("exchange_url",
                              &url),
-    GNUNET_JSON_spec_absolute_time ("expire",
-                                    &expire),
+    TALER_JSON_spec_absolute_time ("expire",
+                                   &expire),
     GNUNET_JSON_spec_end ()
   };
   struct TALER_EXCHANGE_Keys key_data;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .ec = TALER_EC_NONE,
+    .http_status = MHD_HTTP_OK,
+    .reply = data
+  };
 
   if (NULL == data)
     return;
@@ -1592,7 +1605,7 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
           sizeof (struct TALER_EXCHANGE_Keys));
   if (GNUNET_OK !=
       decode_keys_json (keys,
-                        GNUNET_NO,
+                        false,
                         &key_data,
                         &vc))
   {
@@ -1611,6 +1624,7 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
   update_auditors (exchange);
   /* notify application about the key information */
   exchange->cert_cb (exchange->cert_cb_cls,
+                     &hr,
                      &exchange->key_data,
                      vc);
   GNUNET_JSON_parse_free (spec);
@@ -1850,7 +1864,7 @@ TALER_EXCHANGE_serialize_data (struct TALER_EXCHANGE_Handle *exchange)
   }
   return json_pack ("{s:I, s:o, s:s, s:o}",
                     "version",
-                    (json_int_t) TALER_SERIALIZATION_FORMAT_VERSION,
+                    (json_int_t) EXCHANGE_SERIALIZATION_FORMAT_VERSION,
                     "expire",
                     GNUNET_JSON_from_time_abs (exchange->key_data_expiration),
                     "exchange_url",
@@ -2002,11 +2016,10 @@ request_keys (void *cls)
                  curl_easy_setopt (eh,
                                    CURLOPT_HEADERDATA,
                                    kr));
-  kr->job = GNUNET_CURL_job_add (exchange->ctx,
-                                 eh,
-                                 GNUNET_YES,
-                                 &keys_completed_cb,
-                                 kr);
+  kr->job = GNUNET_CURL_job_add_with_ct_json (exchange->ctx,
+                                              eh,
+                                              &keys_completed_cb,
+                                              kr);
   exchange->kr = kr;
 }
 
@@ -2197,8 +2210,7 @@ const struct TALER_EXCHANGE_Keys *
 TALER_EXCHANGE_get_keys (struct TALER_EXCHANGE_Handle *exchange)
 {
   (void) TALER_EXCHANGE_check_keys_current (exchange,
-                                            GNUNET_NO,
-                                            GNUNET_NO);
+                                            TALER_EXCHANGE_CKF_NONE);
   return &exchange->key_data;
 }
 
@@ -2214,8 +2226,7 @@ json_t *
 TALER_EXCHANGE_get_keys_raw (struct TALER_EXCHANGE_Handle *exchange)
 {
   (void) TALER_EXCHANGE_check_keys_current (exchange,
-                                            GNUNET_NO,
-                                            GNUNET_NO);
+                                            TALER_EXCHANGE_CKF_NONE);
   return json_deep_copy (exchange->key_data_raw);
 }
 
