@@ -79,15 +79,11 @@ typedef struct TEAH_AuditorInteractionEntry *
  * auditor's /deposit-confirmation handler.
  *
  * @param cls closure of type `struct TEAH_AuditorInteractionEntry *`
- * @param http_status HTTP status code, 200 on success
- * @param ec taler protocol error status code, 0 on success
- * @param json raw json response
+ * @param hr HTTP response
  */
 void
 TEAH_acc_confirmation_cb (void *cls,
-                          unsigned int http_status,
-                          enum TALER_ErrorCode ec,
-                          const json_t *json);
+                          const struct TALER_AUDITOR_HttpResponse *hr);
 
 
 /**
