@@ -128,18 +128,21 @@ parse_link_coin (const struct TALER_EXCHANGE_LinkHandle *lh,
                                  rpub);
   /* verify link_sig */
   {
-    struct TALER_LinkDataPS ldp;
     struct TALER_PlanchetDetail pd;
+    struct GNUNET_HashCode c_hash;
+    struct TALER_LinkDataPS ldp = {
+      .purpose.size = htonl (sizeof (ldp)),
+      .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_LINK),
+      .transfer_pub = *trans_pub
+    };
 
-    ldp.purpose.size = htonl (sizeof (ldp));
-    ldp.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_LINK);
     GNUNET_CRYPTO_eddsa_key_get_public (&lh->coin_priv.eddsa_priv,
                                         &ldp.old_coin_pub.eddsa_pub);
-    ldp.transfer_pub = *trans_pub;
     pub->rsa_public_key = rpub;
     if (GNUNET_OK !=
         TALER_planchet_prepare (pub,
                                 &fc,
+                                &c_hash,
                                 &pd))
     {
       GNUNET_break (0);
@@ -154,7 +157,7 @@ parse_link_coin (const struct TALER_EXCHANGE_LinkHandle *lh,
 
     if (GNUNET_OK !=
         GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_LINK,
-                                    &ldp.purpose,
+                                    &ldp,
                                     &link_sig.eddsa_signature,
                                     &ldp.old_coin_pub.eddsa_pub))
     {
@@ -186,6 +189,10 @@ parse_link_ok (struct TALER_EXCHANGE_LinkHandle *lh,
   unsigned int session;
   unsigned int num_coins;
   int ret;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = json,
+    .http_status = MHD_HTTP_OK
+  };
 
   if (! json_is_array (json))
   {
@@ -305,13 +312,11 @@ parse_link_ok (struct TALER_EXCHANGE_LinkHandle *lh,
     if (off_coin == num_coins)
     {
       lh->link_cb (lh->link_cb_cls,
-                   MHD_HTTP_OK,
-                   TALER_EC_NONE,
+                   &hr,
                    num_coins,
                    coin_privs,
                    sigs,
-                   pubs,
-                   json);
+                   pubs);
       lh->link_cb = NULL;
       ret = GNUNET_OK;
     }
@@ -350,13 +355,16 @@ handle_link_finished (void *cls,
 {
   struct TALER_EXCHANGE_LinkHandle *lh = cls;
   const json_t *j = response;
-  enum TALER_ErrorCode ec;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = j,
+    .http_status = (unsigned int) response_code
+  };
 
   lh->job = NULL;
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -364,47 +372,49 @@ handle_link_finished (void *cls,
                        j))
     {
       GNUNET_break_op (0);
-      response_code = 0;
-      ec = TALER_EC_LINK_REPLY_MALFORMED;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_LINK_REPLY_MALFORMED;
       break;
     }
     GNUNET_assert (NULL == lh->link_cb);
     TALER_EXCHANGE_link_cancel (lh);
     return;
   case MHD_HTTP_BAD_REQUEST:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* Nothing really to verify, exchange says this coin was not melted; we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
+    GNUNET_break_op (0);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u\n",
-                (unsigned int) response_code);
-    GNUNET_break (0);
-    response_code = 0;
-    ec = TALER_JSON_get_error_code (j);
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) hr.ec);
     break;
   }
   if (NULL != lh->link_cb)
     lh->link_cb (lh->link_cb_cls,
-                 response_code,
-                 ec,
+                 &hr,
                  0,
                  NULL,
                  NULL,
-                 NULL,
-                 j);
+                 NULL);
   TALER_EXCHANGE_link_cancel (lh);
 }
 
@@ -474,11 +484,10 @@ TALER_EXCHANGE_link (struct TALER_EXCHANGE_Handle *exchange,
     return NULL;
   }
   ctx = TEAH_handle_to_context (exchange);
-  lh->job = GNUNET_CURL_job_add (ctx,
-                                 eh,
-                                 GNUNET_YES,
-                                 &handle_link_finished,
-                                 lh);
+  lh->job = GNUNET_CURL_job_add_with_ct_json (ctx,
+                                              eh,
+                                              &handle_link_finished,
+                                              lh);
   return lh;
 }
 
