@@ -75,6 +75,11 @@ struct TALER_EXCHANGE_MeltHandle
    */
   struct MeltData *md;
 
+  /**
+   * Public key of the coin being melted.
+   */
+  struct TALER_CoinSpendPublicKeyP coin_pub;
+
   /**
    * @brief Public information about the coin's denomination key
    */
@@ -142,7 +147,7 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
   confirm.noreveal_index = htonl (*noreveal_index);
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_MELT,
-                                  &confirm.purpose,
+                                  &confirm,
                                   &exchange_sig.eddsa_signature,
                                   &exchange_pub->eddsa_pub))
   {
@@ -153,6 +158,48 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
 }
 
 
+/**
+ * Verify that the signatures on the "409 CONFLICT" response from the
+ * exchange demonstrating customer denomination key differences
+ * resulting from coin private key reuse are valid.
+ *
+ * @param mh melt handle
+ * @param json json reply with the signature(s) and transaction history
+ * @return #GNUNET_OK if the signature(s) is valid, #GNUNET_SYSERR if not
+ */
+static int
+verify_melt_signature_denom_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
+                                      const json_t *json)
+
+{
+  json_t *history;
+  struct TALER_Amount total;
+  struct GNUNET_HashCode h_denom_pub;
+
+  memset (&h_denom_pub,
+          0,
+          sizeof (h_denom_pub));
+  history = json_object_get (json,
+                             "history");
+  if (GNUNET_OK !=
+      TALER_EXCHANGE_verify_coin_history (&mh->dki,
+                                          mh->dki.value.currency,
+                                          &mh->coin_pub,
+                                          history,
+                                          &h_denom_pub,
+                                          &total))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if (0 != GNUNET_memcmp (&mh->dki.h_key,
+                          &h_denom_pub))
+    return GNUNET_OK; /* indeed, proof with different denomination key provided */
+  /* invalid proof provided */
+  return GNUNET_SYSERR;
+}
+
+
 /**
  * Verify that the signatures on the "409 CONFLICT" response from the
  * exchange demonstrating customer double-spending are valid.
@@ -162,8 +209,8 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
  * @return #GNUNET_OK if the signature(s) is valid, #GNUNET_SYSERR if not
  */
 static int
-verify_melt_signature_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
-                                const json_t *json)
+verify_melt_signature_spend_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
+                                      const json_t *json)
 {
   json_t *history;
   struct TALER_Amount original_value;
@@ -178,6 +225,8 @@ verify_melt_signature_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
     GNUNET_JSON_spec_end ()
   };
   const struct MeltedCoin *mc;
+  enum TALER_ErrorCode ec;
+  struct GNUNET_HashCode h_denom_pub;
 
   /* parse JSON reply */
   if (GNUNET_OK !=
@@ -211,6 +260,9 @@ verify_melt_signature_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
   }
 
   /* verify coin history */
+  memset (&h_denom_pub,
+          0,
+          sizeof (h_denom_pub));
   history = json_object_get (json,
                              "history");
   if (GNUNET_OK !=
@@ -218,6 +270,7 @@ verify_melt_signature_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
                                           original_value.currency,
                                           &coin_pub,
                                           history,
+                                          &h_denom_pub,
                                           &total))
   {
     GNUNET_break_op (0);
@@ -226,27 +279,43 @@ verify_melt_signature_conflict (struct TALER_EXCHANGE_MeltHandle *mh,
   }
   json_decref (history);
 
-  /* check if melt operation was really too expensive given history */
-  if (GNUNET_OK !=
-      TALER_amount_add (&total,
-                        &total,
-                        &melt_value_with_fee))
+  ec = TALER_JSON_get_error_code (json);
+  switch (ec)
   {
-    /* clearly not OK if our transaction would have caused
-       the overflow... */
-    return GNUNET_OK;
-  }
+  case TALER_EC_MELT_INSUFFICIENT_FUNDS:
+    /* check if melt operation was really too expensive given history */
+    if (0 >
+        TALER_amount_add (&total,
+                          &total,
+                          &melt_value_with_fee))
+    {
+      /* clearly not OK if our transaction would have caused
+         the overflow... */
+      return GNUNET_OK;
+    }
 
-  if (0 >= TALER_amount_cmp (&total,
-                             &original_value))
-  {
-    /* transaction should have still fit */
-    GNUNET_break (0);
+    if (0 >= TALER_amount_cmp (&total,
+                               &original_value))
+    {
+      /* transaction should have still fit */
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
+
+    /* everything OK, valid proof of double-spending was provided */
+    return GNUNET_OK;
+  case TALER_EC_COIN_CONFLICTING_DENOMINATION_KEY:
+    if (0 != GNUNET_memcmp (&mh->dki.h_key,
+                            &h_denom_pub))
+      return GNUNET_OK; /* indeed, proof with different denomination key provided */
+    /* invalid proof provided */
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  default:
+    /* unexpected error code */
+    GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-
-  /* everything OK, valid proof of double-spending was provided */
-  return GNUNET_OK;
 }
 
 
@@ -267,13 +336,16 @@ handle_melt_finished (void *cls,
   uint32_t noreveal_index = TALER_CNC_KAPPA; /* invalid value */
   struct TALER_ExchangePublicKeyP exchange_pub;
   const json_t *j = response;
-  enum TALER_ErrorCode ec;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = j,
+    .http_status = (unsigned int) response_code
+  };
 
   mh->job = NULL;
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -283,78 +355,96 @@ handle_melt_finished (void *cls,
                                   &noreveal_index))
     {
       GNUNET_break_op (0);
-      response_code = 0;
-      ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
-    }
-    else
-    {
-      ec = TALER_EC_NONE;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
     }
     if (NULL != mh->melt_cb)
     {
       mh->melt_cb (mh->melt_cb_cls,
-                   response_code,
-                   ec,
+                   &hr,
                    noreveal_index,
-                   (0 == response_code)
+                   (0 == hr.http_status)
                    ? NULL
-                   : &exchange_pub,
-                   j);
+                   : &exchange_pub);
       mh->melt_cb = NULL;
     }
     break;
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_CONFLICT:
-    /* Double spending; check signatures on transaction history */
-    if (GNUNET_OK !=
-        verify_melt_signature_conflict (mh,
-                                        j))
+    hr.ec = TALER_JSON_get_error_code (j);
+    switch (hr.ec)
     {
+    case TALER_EC_MELT_INSUFFICIENT_FUNDS:
+      /* Double spending; check signatures on transaction history */
+      if (GNUNET_OK !=
+          verify_melt_signature_spend_conflict (mh,
+                                                j))
+      {
+        GNUNET_break_op (0);
+        hr.http_status = 0;
+        hr.ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
+        hr.hint = TALER_JSON_get_error_hint (j);
+      }
+      break;
+    case TALER_EC_COIN_CONFLICTING_DENOMINATION_KEY:
+      if (GNUNET_OK !=
+          verify_melt_signature_denom_conflict (mh,
+                                                j))
+      {
+        GNUNET_break_op (0);
+        hr.http_status = 0;
+        hr.ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
+        hr.hint = TALER_JSON_get_error_hint (j);
+      }
+      break;
+    default:
       GNUNET_break_op (0);
-      response_code = 0;
-      ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
+      hr.http_status = 0;
+      hr.ec = TALER_EC_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
+      hr.hint = TALER_JSON_get_error_hint (j);
+      break;
     }
-    else
-      ec = TALER_EC_NONE;
     break;
   case MHD_HTTP_FORBIDDEN:
     /* Nothing really to verify, exchange says one of the signatures is
        invalid; assuming we checked them, this should never happen, we
        should pass the JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_NOT_FOUND:
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   default:
     /* unexpected response code */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                ec);
-    GNUNET_break (0);
-    response_code = 0;
+                hr.ec);
+    GNUNET_break_op (0);
     break;
   }
   if (NULL != mh->melt_cb)
     mh->melt_cb (mh->melt_cb_cls,
-                 response_code,
-                 ec,
+                 &hr,
                  UINT32_MAX,
-                 NULL,
-                 j);
+                 NULL);
   TALER_EXCHANGE_melt_cancel (mh);
 }
 
@@ -394,9 +484,11 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
   struct GNUNET_CURL_Context *ctx;
   struct MeltData *md;
   struct TALER_CoinSpendSignatureP confirm_sig;
-  struct TALER_RefreshMeltCoinAffirmationPS melt;
-  struct GNUNET_HashCode h_denom_pub;
   char arg_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2 + 32];
+  struct TALER_RefreshMeltCoinAffirmationPS melt = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT),
+    .purpose.size = htonl (sizeof (melt)),
+  };
 
   GNUNET_assert (GNUNET_YES ==
                  TEAH_handle_is_ready (exchange));
@@ -407,9 +499,6 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
     GNUNET_break (0);
     return NULL;
   }
-  melt.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT);
-  melt.purpose.size = htonl (sizeof (struct
-                                     TALER_RefreshMeltCoinAffirmationPS));
   melt.rc = md->rc;
   TALER_amount_hton (&melt.amount_with_fee,
                      &md->melted_coin.melt_amount_with_fee);
@@ -417,16 +506,16 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
                      &md->melted_coin.fee_melt);
   GNUNET_CRYPTO_eddsa_key_get_public (&md->melted_coin.coin_priv.eddsa_priv,
                                       &melt.coin_pub.eddsa_pub);
-  GNUNET_CRYPTO_eddsa_sign (&md->melted_coin.coin_priv.eddsa_priv,
-                            &melt.purpose,
-                            &confirm_sig.eddsa_signature);
   GNUNET_CRYPTO_rsa_public_key_hash (md->melted_coin.pub_key.rsa_public_key,
-                                     &h_denom_pub);
+                                     &melt.h_denom_pub);
+  GNUNET_CRYPTO_eddsa_sign (&md->melted_coin.coin_priv.eddsa_priv,
+                            &melt,
+                            &confirm_sig.eddsa_signature);
   melt_obj = json_pack ("{s:o, s:o, s:o, s:o, s:o, s:o}",
                         "coin_pub",
                         GNUNET_JSON_from_data_auto (&melt.coin_pub),
                         "denom_pub_hash",
-                        GNUNET_JSON_from_data_auto (&h_denom_pub),
+                        GNUNET_JSON_from_data_auto (&melt.h_denom_pub),
                         "denom_sig",
                         GNUNET_JSON_from_rsa_signature (
                           md->melted_coin.sig.rsa_signature),
@@ -447,11 +536,11 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
     char pub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
     char *end;
 
-    end = GNUNET_STRINGS_data_to_string (&melt.coin_pub,
-                                         sizeof (struct
-                                                 TALER_CoinSpendPublicKeyP),
-                                         pub_str,
-                                         sizeof (pub_str));
+    end = GNUNET_STRINGS_data_to_string (
+      &melt.coin_pub,
+      sizeof (struct TALER_CoinSpendPublicKeyP),
+      pub_str,
+      sizeof (pub_str));
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
@@ -466,6 +555,7 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
   /* and now we can at last begin the actual request handling */
   mh = GNUNET_new (struct TALER_EXCHANGE_MeltHandle);
   mh->exchange = exchange;
+  mh->coin_pub = melt.coin_pub;
   mh->dki = *dki;
   mh->dki.key.rsa_public_key = NULL; /* lifetime not warranted, so better
                                          not copy the pointer */
