@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -148,10 +148,10 @@ parse_fees (json_t *fees)
                                 &wa->wire_fee),
         TALER_JSON_spec_amount ("closing_fee",
                                 &wa->closing_fee),
-        GNUNET_JSON_spec_absolute_time ("start_date",
-                                        &wa->start_date),
-        GNUNET_JSON_spec_absolute_time ("end_date",
-                                        &wa->end_date),
+        TALER_JSON_spec_absolute_time ("start_date",
+                                       &wa->start_date),
+        TALER_JSON_spec_absolute_time ("end_date",
+                                       &wa->end_date),
         GNUNET_JSON_spec_end ()
       };
 
@@ -208,15 +208,18 @@ handle_wire_finished (void *cls,
                       const void *response)
 {
   struct TALER_EXCHANGE_WireHandle *wh = cls;
-  enum TALER_ErrorCode ec;
   const json_t *j = response;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .reply = j,
+    .http_status = (unsigned int) response_code
+  };
 
   TALER_LOG_DEBUG ("Checking raw /wire response\n");
   wh->job = NULL;
   switch (response_code)
   {
   case 0:
-    ec = TALER_EC_INVALID_RESPONSE;
+    hr.ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
@@ -238,8 +241,8 @@ handle_wire_finished (void *cls,
       {
         /* bogus reply */
         GNUNET_break_op (0);
-        response_code = 0;
-        ec = TALER_EC_SERVER_JSON_INVALID;
+        hr.http_status = 0;
+        hr.ec = TALER_EC_SERVER_JSON_INVALID;
         break;
       }
       if (0 == (num_accounts = json_array_size (accounts)))
@@ -247,8 +250,8 @@ handle_wire_finished (void *cls,
         /* bogus reply */
         GNUNET_break_op (0);
         GNUNET_JSON_parse_free (spec);
-        response_code = 0;
-        ec = TALER_EC_SERVER_JSON_INVALID;
+        hr.http_status = 0;
+        hr.ec = TALER_EC_SERVER_JSON_INVALID;
         break;
       }
       if (NULL == (fm = parse_fees (fees)))
@@ -256,12 +259,11 @@ handle_wire_finished (void *cls,
         /* bogus reply */
         GNUNET_break_op (0);
         GNUNET_JSON_parse_free (spec);
-        response_code = 0;
-        ec = TALER_EC_SERVER_JSON_INVALID;
+        hr.http_status = 0;
+        hr.ec = TALER_EC_SERVER_JSON_INVALID;
         break;
       }
 
-      ec = TALER_EC_NONE;
       key_state = TALER_EXCHANGE_get_keys (wh->exchange);
       /* parse accounts */
       {
@@ -286,8 +288,8 @@ handle_wire_finished (void *cls,
           {
             /* bogus reply */
             GNUNET_break_op (0);
-            response_code = 0;
-            ec = TALER_EC_SERVER_SIGNATURE_INVALID;
+            hr.http_status = 0;
+            hr.ec = TALER_EC_SERVER_SIGNATURE_INVALID;
             break;
           }
           if (GNUNET_OK !=
@@ -297,16 +299,16 @@ handle_wire_finished (void *cls,
           {
             /* bogus reply */
             GNUNET_break_op (0);
-            response_code = 0;
-            ec = TALER_EC_SERVER_JSON_INVALID;
+            hr.http_status = 0;
+            hr.ec = TALER_EC_SERVER_JSON_INVALID;
             break;
           }
           if (NULL == (method = TALER_payto_get_method (wa->payto_uri)))
           {
             /* bogus reply */
             GNUNET_break_op (0);
-            response_code = 0;
-            ec = TALER_EC_SERVER_JSON_INVALID;
+            hr.http_status = 0;
+            hr.ec = TALER_EC_SERVER_JSON_INVALID;
             break;
           }
           if (NULL == (wa->fees = lookup_fee (fm,
@@ -314,8 +316,8 @@ handle_wire_finished (void *cls,
           {
             /* bogus reply */
             GNUNET_break_op (0);
-            response_code = 0;
-            ec = TALER_EC_SERVER_JSON_INVALID;
+            hr.http_status = 0;
+            hr.ec = TALER_EC_SERVER_JSON_INVALID;
             GNUNET_free (method);
             break;
           }
@@ -325,8 +327,7 @@ handle_wire_finished (void *cls,
              (NULL != wh->cb) )
         {
           wh->cb (wh->cb_cls,
-                  response_code,
-                  ec,
+                  &hr,
                   num_accounts,
                   was);
           wh->cb = NULL;
@@ -339,33 +340,35 @@ handle_wire_finished (void *cls,
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_NOT_FOUND:
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    ec = TALER_JSON_get_error_code (j);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     break;
   default:
     /* unexpected response code */
-    ec = TALER_JSON_get_error_code (j);
+    GNUNET_break_op (0);
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) ec);
-    GNUNET_break (0);
-    response_code = 0;
+                (int) hr.ec);
     break;
   }
   if (NULL != wh->cb)
     wh->cb (wh->cb_cls,
-            response_code,
-            ec,
+            &hr,
             0,
             NULL);
   TALER_EXCHANGE_wire_cancel (wh);
@@ -422,11 +425,10 @@ TALER_EXCHANGE_wire (struct TALER_EXCHANGE_Handle *exchange,
     return NULL;
   }
   ctx = TEAH_handle_to_context (exchange);
-  wh->job = GNUNET_CURL_job_add (ctx,
-                                 eh,
-                                 GNUNET_YES,
-                                 &handle_wire_finished,
-                                 wh);
+  wh->job = GNUNET_CURL_job_add_with_ct_json (ctx,
+                                              eh,
+                                              &handle_wire_finished,
+                                              wh);
   return wh;
 }
 
