@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2019 Taler Systems SA
+  Copyright (C) 2019, 2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -24,6 +24,7 @@
 #include <gnunet/gnunet_json_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
+#include "taler_util.h"
 #include "taler_mhd_lib.h"
 
 
@@ -38,7 +39,8 @@ struct Terms
   const char *mime_type;
 
   /**
-   * The terms (NOT 0-terminated!).
+   * The terms (NOT 0-terminated!), mmap()'ed. Do not free,
+   * use munmap() instead.
    */
   void *terms;
 
@@ -47,10 +49,23 @@ struct Terms
    */
   char *language;
 
+  /**
+   * deflated @e terms, to return if client supports deflate compression.
+   * malloc()'ed.  NULL if @e terms does not compress.
+   */
+  void *compressed_terms;
+
   /**
    * Number of bytes in @e terms.
    */
   size_t terms_size;
+
+  /**
+   * Number of bytes in @e compressed_terms.
+   */
+  size_t compressed_terms_size;
+
+
 };
 
 
@@ -80,7 +95,8 @@ struct TALER_MHD_Legal
 /**
  * Check if @a mime matches the @a accept_pattern.
  *
- * @param accept_pattern a mime pattern like text/plain or image/STAR
+ * @param accept_pattern a mime pattern like "text/plain"
+ *        or "image/STAR"
  * @param mime the mime type to match
  * @return true if @a mime matches the @a accept_pattern
  */
@@ -108,42 +124,34 @@ mime_matches (const char *accept_pattern,
 
 
 /**
- * Check if @a lang matches the @a language_pattern, and if so with
- * which preference.
+ * Check if @a mime matches the @a accept_pattern.  For this function, the @a
+ * accept_pattern may include multiple values separated by ";".
  *
- * @param language_pattern a language preferences string
- *        like "fr-CH, fr;q=0.9, en;q=0.8, *;q=0.1"
- * @param lang the 2-digit language to match
- * @return q-weight given for @a lang in @a language_pattern, 1.0 if no weights are given;
- *         0 if @a lang is not in @a language_pattern
+ * @param accept_pattern a mime pattern like "text/plain"
+ *        or "image/STAR" or "text/plain; text/xml"
+ * @param mime the mime type to match
+ * @return true if @a mime matches the @a accept_pattern
  */
-static double
-language_matches (const char *language_pattern,
-                  const char *lang)
+static bool
+xmime_matches (const char *accept_pattern,
+               const char *mime)
 {
-  char *p = GNUNET_strdup (language_pattern);
+  char *ap = GNUNET_strdup (accept_pattern);
   char *sptr;
-  double r = 0.0;
 
-  for (char *tok = strtok_r (p, ", ", &sptr);
+  for (const char *tok = strtok_r (ap, ";", &sptr);
        NULL != tok;
-       tok = strtok_r (NULL, ", ", &sptr))
+       tok = strtok_r (NULL, ";", &sptr))
   {
-    char *sptr2;
-    char *lp = strtok_r (tok, ";", &sptr2);
-    char *qp = strtok_r (NULL, ";", &sptr2);
-    double q = 1.0;
-
-    GNUNET_break_op ( (NULL == qp) ||
-                      (1 == sscanf (qp,
-                                    "q=%lf",
-                                    &q)) );
-    if (0 == strcasecmp (lang,
-                         lp))
-      r = GNUNET_MAX (r, q);
+    if (mime_matches (tok,
+                      mime))
+    {
+      GNUNET_free (ap);
+      return true;
+    }
   }
-  GNUNET_free (p);
-  return r;
+  GNUNET_free (ap);
+  return false;
 }
 
 
@@ -155,7 +163,7 @@ language_matches (const char *language_pattern,
  * @param legal legal document to serve
  * @return MHD result code
  */
-int
+MHD_RESULT
 TALER_MHD_reply_legal (struct MHD_Connection *conn,
                        struct TALER_MHD_Legal *legal)
 {
@@ -174,11 +182,12 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
          (0 == strcasecmp (etag,
                            legal->terms_etag)) )
     {
-      int ret;
+      MHD_RESULT ret;
 
       resp = MHD_create_response_from_buffer (0,
                                               NULL,
                                               MHD_RESPMEM_PERSISTENT);
+      TALER_MHD_add_global_headers (resp);
       ret = MHD_queue_response (conn,
                                 MHD_HTTP_NOT_MODIFIED,
                                 resp);
@@ -211,16 +220,16 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
       struct Terms *p = &legal->terms[i];
 
       if ( (NULL == t) ||
-           (mime_matches (mime,
-                          p->mime_type)) )
+           (xmime_matches (mime,
+                           p->mime_type)) )
       {
         if ( (NULL == t) ||
-             (! mime_matches (mime,
-                              t->mime_type)) ||
-             (language_matches (lang,
-                                p->language) >
-              language_matches (lang,
-                                t->language) ) )
+             (! xmime_matches (mime,
+                               t->mime_type)) ||
+             (TALER_language_matches (lang,
+                                      p->language) >
+              TALER_language_matches (lang,
+                                      t->language) ) )
           t = p;
       }
     }
@@ -250,29 +259,17 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
   if (MHD_YES ==
       TALER_MHD_can_compress (conn))
   {
-    void *buf = GNUNET_memdup (t->terms,
-                               t->terms_size);
-    size_t buf_size = t->terms_size;
-
-    if (TALER_MHD_body_compress (&buf,
-                                 &buf_size))
+    resp = MHD_create_response_from_buffer (t->compressed_terms_size,
+                                            t->compressed_terms,
+                                            MHD_RESPMEM_PERSISTENT);
+    if (MHD_NO ==
+        MHD_add_response_header (resp,
+                                 MHD_HTTP_HEADER_CONTENT_ENCODING,
+                                 "deflate"))
     {
-      resp = MHD_create_response_from_buffer (buf_size,
-                                              buf,
-                                              MHD_RESPMEM_MUST_FREE);
-      if (MHD_NO ==
-          MHD_add_response_header (resp,
-                                   MHD_HTTP_HEADER_CONTENT_ENCODING,
-                                   "deflate"))
-      {
-        GNUNET_break (0);
-        MHD_destroy_response (resp);
-        resp = NULL;
-      }
-    }
-    else
-    {
-      GNUNET_free (buf);
+      GNUNET_break (0);
+      MHD_destroy_response (resp);
+      resp = NULL;
     }
   }
   if (NULL == resp)
@@ -282,6 +279,7 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
                                             (void *) t->terms,
                                             MHD_RESPMEM_PERSISTENT);
   }
+  TALER_MHD_add_global_headers (resp);
   if (NULL != legal)
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (resp,
@@ -292,7 +290,7 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
                                          MHD_HTTP_HEADER_CONTENT_TYPE,
                                          t->mime_type));
   {
-    int ret;
+    MHD_RESULT ret;
 
     ret = MHD_queue_response (conn,
                               MHD_HTTP_OK,
@@ -417,31 +415,22 @@ load_terms (struct TALER_MHD_Legal *legal,
       return;
     }
     {
-      char *buf;
+      void *buf;
       size_t bsize;
-      ssize_t ret;
 
       bsize = (size_t) st.st_size;
-      buf = GNUNET_malloc_large (bsize);
-      if (NULL == buf)
-      {
-        GNUNET_log_strerror (GNUNET_ERROR_TYPE_WARNING,
-                             "malloc");
-        GNUNET_break (0 == close (fd));
-        GNUNET_free (fn);
-        return;
-      }
-      ret = read (fd,
-                  buf,
-                  bsize);
-      if ( (ret < 0) ||
-           (bsize != ((size_t) ret)) )
+      buf = mmap (NULL,
+                  bsize,
+                  PROT_READ,
+                  MAP_SHARED,
+                  fd,
+                  0);
+      if (MAP_FAILED == buf)
       {
         GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_WARNING,
-                                  "read",
+                                  "mmap",
                                   fn);
         GNUNET_break (0 == close (fd));
-        GNUNET_free (buf);
         GNUNET_free (fn);
         return;
       }
@@ -457,6 +446,18 @@ load_terms (struct TALER_MHD_Legal *legal,
           .terms_size = bsize
         };
 
+        buf = GNUNET_memdup (t.terms,
+                             t.terms_size);
+        if (TALER_MHD_body_compress (&buf,
+                                     &bsize))
+        {
+          t.compressed_terms = buf;
+          t.compressed_terms_size = bsize;
+        }
+        else
+        {
+          GNUNET_free (buf);
+        }
         GNUNET_array_append (legal->terms,
                              legal->terms_len,
                              t);
@@ -513,8 +514,7 @@ load_language (struct TALER_MHD_Legal *legal,
  *
  * @param cfg configuration to use
  * @param section section to load values from
- * @param diroption name of the option with the
- *        path to the legal documents
+ * @param diroption name of the option with the path to the legal documents
  * @param tagoption name of the files to use
  *        for the legal documents and the Etag
  * @return NULL on error
@@ -598,7 +598,10 @@ TALER_MHD_legal_free (struct TALER_MHD_Legal *legal)
     struct Terms *t = &legal->terms[i];
 
     GNUNET_free (t->language);
-    GNUNET_free (t->terms);
+    GNUNET_free (t->compressed_terms);
+    if (0 != munmap (t->terms, t->terms_size))
+      GNUNET_log_strerror (GNUNET_ERROR_TYPE_WARNING,
+                           "munmap");
   }
   GNUNET_array_grow (legal->terms,
                      legal->terms_len,
