@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014--2019 Taler Systems SA
+  Copyright (C) 2014--2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -56,7 +56,7 @@
  *               (we could not even queue an error message,
  *                close HTTP session with MHD_NO)
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_MHD_parse_post_json (struct MHD_Connection *connection,
                            void **con_cls,
                            const char *upload_data,
@@ -76,11 +76,11 @@ TALER_MHD_parse_post_json (struct MHD_Connection *connection,
   case GNUNET_JSON_PR_OUT_OF_MEMORY:
     GNUNET_break (NULL == *json);
     return (MHD_NO ==
-            TALER_MHD_reply_with_error
-              (connection,
+            TALER_MHD_reply_with_error (
+              connection,
               MHD_HTTP_INTERNAL_SERVER_ERROR,
               TALER_EC_PARSER_OUT_OF_MEMORY,
-              "out of memory")) ? GNUNET_SYSERR : GNUNET_NO;
+              NULL)) ? GNUNET_SYSERR : GNUNET_NO;
 
   case GNUNET_JSON_PR_CONTINUE:
     GNUNET_break (NULL == *json);
@@ -96,7 +96,7 @@ TALER_MHD_parse_post_json (struct MHD_Connection *connection,
             TALER_MHD_reply_with_error (connection,
                                         MHD_HTTP_BAD_REQUEST,
                                         TALER_EC_JSON_INVALID,
-                                        "invalid JSON uploaded"))
+                                        NULL))
            ? GNUNET_NO : GNUNET_SYSERR;
   case GNUNET_JSON_PR_SUCCESS:
     GNUNET_break (NULL != *json);
@@ -186,12 +186,12 @@ TALER_MHD_parse_request_arg_data (struct MHD_Connection *connection,
  *    #GNUNET_NO if json is malformed, error response was generated
  *    #GNUNET_SYSERR on internal error
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_MHD_parse_json_data (struct MHD_Connection *connection,
                            const json_t *root,
                            struct GNUNET_JSON_Specification *spec)
 {
-  int ret;
+  enum GNUNET_GenericReturnValue ret;
   const char *error_json_name;
   unsigned int error_line;
 
@@ -204,15 +204,67 @@ TALER_MHD_parse_json_data (struct MHD_Connection *connection,
     if (NULL == error_json_name)
       error_json_name = "<no field>";
     ret = (MHD_YES ==
-           TALER_MHD_reply_json_pack (connection,
-                                      MHD_HTTP_BAD_REQUEST,
-                                      "{s:s, s:I, s:s, s:I}",
-                                      "hint", "JSON parse error",
-                                      "code",
-                                      (json_int_t)
-                                      TALER_EC_JSON_INVALID_WITH_DETAILS,
-                                      "field", error_json_name,
-                                      "line", (json_int_t) error_line))
+           TALER_MHD_reply_json_pack (
+             connection,
+             MHD_HTTP_BAD_REQUEST,
+             "{s:s, s:I, s:s, s:I}",
+             "hint", TALER_ErrorCode_get_hint (
+               TALER_EC_JSON_INVALID_WITH_DETAILS),
+             "code", (json_int_t) TALER_EC_JSON_INVALID_WITH_DETAILS,
+             "field", error_json_name,
+             "line", (json_int_t) error_line))
+          ? GNUNET_NO : GNUNET_SYSERR;
+    return ret;
+  }
+  return GNUNET_YES;
+}
+
+
+/**
+ * Parse JSON object that we (the server!) generated into components based on
+ * the given field specification.  The difference to
+ * #TALER_MHD_parse_json_data() is that this function will fail
+ * with an HTTP failure of 500 (internal server error) in case
+ * parsing fails, instead of blaming it on the client with a
+ * 400 (#MHD_HTTP_BAD_REQUEST).
+ *
+ * @param connection the connection to send an error response to
+ * @param root the JSON node to start the navigation at.
+ * @param spec field specification for the parser
+ * @return
+ *    #GNUNET_YES if navigation was successful (caller is responsible
+ *                for freeing allocated variable-size data using
+ *                GNUNET_JSON_parse_free() when done)
+ *    #GNUNET_NO if json is malformed, error response was generated
+ *    #GNUNET_SYSERR on internal error
+ */
+enum GNUNET_GenericReturnValue
+TALER_MHD_parse_internal_json_data (struct MHD_Connection *connection,
+                                    const json_t *root,
+                                    struct GNUNET_JSON_Specification *spec)
+{
+  enum GNUNET_GenericReturnValue ret;
+  const char *error_json_name;
+  unsigned int error_line;
+
+  ret = GNUNET_JSON_parse (root,
+                           spec,
+                           &error_json_name,
+                           &error_line);
+  if (GNUNET_SYSERR == ret)
+  {
+    if (NULL == error_json_name)
+      error_json_name = "<no field>";
+    ret = (MHD_YES ==
+           TALER_MHD_reply_json_pack (
+             connection,
+             MHD_HTTP_INTERNAL_SERVER_ERROR,
+             "{s:s, s:I, s:s, s:I}",
+             "hint", TALER_ErrorCode_get_hint (
+               TALER_EC_INTERNAL_INVARIANT_FAILURE),
+             "code", (json_int_t) TALER_EC_INTERNAL_INVARIANT_FAILURE,
+             "field", error_json_name,
+             "line", (json_int_t) error_line))
           ? GNUNET_NO : GNUNET_SYSERR;
     return ret;
   }
@@ -235,13 +287,13 @@ TALER_MHD_parse_json_data (struct MHD_Connection *connection,
  *    #GNUNET_NO if json is malformed, error response was generated
  *    #GNUNET_SYSERR on internal error
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_MHD_parse_json_array (struct MHD_Connection *connection,
                             const json_t *root,
                             struct GNUNET_JSON_Specification *spec,
                             ...)
 {
-  int ret;
+  enum GNUNET_GenericReturnValue ret;
   const char *error_json_name;
   unsigned int error_line;
   va_list ap;
@@ -259,14 +311,15 @@ TALER_MHD_parse_json_array (struct MHD_Connection *connection,
   if (NULL == root)
   {
     ret = (MHD_YES ==
-           TALER_MHD_reply_json_pack (connection,
-                                      MHD_HTTP_BAD_REQUEST,
-                                      "{s:s, s:I, s:I}",
-                                      "hint", "expected array",
-                                      "code",
-                                      (json_int_t)
-                                      TALER_EC_JSON_INVALID_WITH_DETAILS,
-                                      "dimension", dim))
+           TALER_MHD_reply_json_pack (
+             connection,
+             MHD_HTTP_BAD_REQUEST,
+             "{s:s, s:I, s:s, s:I}",
+             "hint", TALER_ErrorCode_get_hint (
+               TALER_EC_JSON_INVALID_WITH_DETAILS),
+             "code", (json_int_t) TALER_EC_JSON_INVALID_WITH_DETAILS,
+             "detail", "expected array",
+             "dimension", dim))
           ? GNUNET_NO : GNUNET_SYSERR;
     return ret;
   }
@@ -279,14 +332,15 @@ TALER_MHD_parse_json_array (struct MHD_Connection *connection,
     if (NULL == error_json_name)
       error_json_name = "<no field>";
     ret = (MHD_YES ==
-           TALER_MHD_reply_json_pack (connection,
-                                      MHD_HTTP_BAD_REQUEST,
-                                      "{s:s, s:I, s:I}",
-                                      "hint", error_json_name,
-                                      "code",
-                                      (json_int_t)
-                                      TALER_EC_JSON_INVALID_WITH_DETAILS,
-                                      "line", (json_int_t) error_line))
+           TALER_MHD_reply_json_pack (
+             connection,
+             MHD_HTTP_BAD_REQUEST,
+             "{s:s, s:s, s:I, s:I}",
+             "detail", error_json_name,
+             "hint", TALER_ErrorCode_get_hint (
+               TALER_EC_JSON_INVALID_WITH_DETAILS),
+             "code", (json_int_t) TALER_EC_JSON_INVALID_WITH_DETAILS,
+             "line", (json_int_t) error_line))
           ? GNUNET_NO : GNUNET_SYSERR;
     return ret;
   }
