@@ -353,7 +353,6 @@ run (void *cls,
     TALER_TESTING_cmd_refund ("refund-ok",
                               MHD_HTTP_OK,
                               "EUR:5",
-                              "EUR:0.01",
                               "deposit-refund-1"),
     /**
      * Spend 4.99 EUR of the refunded 4.99 EUR coin (1ct gone
@@ -411,7 +410,7 @@ run (void *cls,
                                        MHD_HTTP_OK),
     /**
      * These commands should close the reserve because the aggregator
-     * is given a config file that ovverrides the reserve expiration
+     * is given a config file that overrides the reserve expiration
      * time (making it now-ish)
      */CMD_TRANSFER_TO_EXCHANGE ("short-lived-reserve",
                               "EUR:5.01"),
