@@ -136,12 +136,12 @@ run (void *cls,
      * Do another transfer to the same reserve
      */
     TALER_TESTING_cmd_admin_add_incoming_with_ref ("create-reserve-1.2",
-                                                   "EUR:1",
+                                                   "EUR:2.01",
                                                    &bc.exchange_auth,
                                                    bc.user42_payto,
                                                    "create-reserve-1"),
     TALER_TESTING_cmd_check_bank_admin_transfer ("check-create-reserve-1.2",
-                                                 "EUR:1",
+                                                 "EUR:2.01",
                                                  bc.user42_payto,
                                                  bc.exchange_payto,
                                                  "create-reserve-1.2"),
@@ -153,6 +153,15 @@ run (void *cls,
                                        "create-reserve-1",
                                        "EUR:5",
                                        MHD_HTTP_OK),
+    /**
+     * Withdraw EUR:1 using the SAME private coin key as for the previous coin
+     * (in violation of the specification, to be detected on spending!).
+     */
+    TALER_TESTING_cmd_withdraw_amount_reuse_key ("withdraw-coin-1x",
+                                                 "create-reserve-1",
+                                                 "EUR:1",
+                                                 "withdraw-coin-1",
+                                                 MHD_HTTP_OK),
     /**
      * Check the reserve is depleted.
      */
@@ -160,6 +169,13 @@ run (void *cls,
                               "create-reserve-1",
                               "EUR:0",
                               MHD_HTTP_OK),
+    /*
+     * Try to overdraw.
+     */
+    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2",
+                                       "create-reserve-1",
+                                       "EUR:5",
+                                       MHD_HTTP_CONFLICT),
     TALER_TESTING_cmd_end ()
   };
 
@@ -178,13 +194,14 @@ run (void *cls,
     TALER_TESTING_cmd_deposit_replay ("deposit-simple-replay",
                                       "deposit-simple",
                                       MHD_HTTP_OK),
-    /*
-     * Try to overdraw.
-     */
-    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2",
-                                       "create-reserve-1",
-                                       "EUR:5",
-                                       MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_deposit ("deposit-reused-coin-key-failure",
+                               "withdraw-coin-1x",
+                               0,
+                               bc.user42_payto,
+                               "{\"items\":[{\"name\":\"ice cream\",\"value\":1}]}",
+                               GNUNET_TIME_UNIT_ZERO,
+                               "EUR:1",
+                               MHD_HTTP_CONFLICT),
     /**
      * Try to double spend using different wire details.
      */
@@ -201,7 +218,7 @@ run (void *cls,
      * is currently the case because of the "timestamp" field,
      * which is set automatically by #TALER_TESTING_cmd_deposit().
      * This could theoretically fail if at some point a deposit
-     * command executs in less than 1 ms. *///
+     * command executes in less than 1 ms. *///
     TALER_TESTING_cmd_deposit ("deposit-double-1",
                                "withdraw-coin-1",
                                0,
@@ -225,6 +242,14 @@ run (void *cls,
   };
 
   struct TALER_TESTING_Command refresh[] = {
+    /**
+     * Try to melt the coin that shared the private key with another
+     * coin (should fail). */
+    TALER_TESTING_cmd_melt ("refresh-melt-reused-coin-key-failure",
+                            "withdraw-coin-1x",
+                            MHD_HTTP_CONFLICT,
+                            NULL),
+
     /* Fill reserve with EUR:5, 1ct is for fees. */
     CMD_TRANSFER_TO_EXCHANGE ("refresh-create-reserve-1",
                               "EUR:5.01"),
@@ -346,6 +371,8 @@ run (void *cls,
                                             NULL,
                                             0,
                                             MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_sleep ("sleep-before-aggregator",
+                             1),
     /* Run transfers. Note that _actual_ aggregation will NOT
      * happen here, as each deposit operation is run with a
      * fresh merchant public key, so the aggregator will treat
@@ -525,23 +552,35 @@ run (void *cls,
      * fakebank and the second to actually check there are not
      * other transfers around. *///
     TALER_TESTING_cmd_check_bank_empty ("check_bank_transfer-pre-refund"),
-    TALER_TESTING_cmd_refund ("refund-ok",
-                              MHD_HTTP_OK,
-                              "EUR:5",
-                              "EUR:0.01",
-                              "deposit-refund-1"),
-    TALER_TESTING_cmd_refund ("refund-ok-double",
-                              MHD_HTTP_OK,
-                              "EUR:5",
-                              "EUR:0.01",
-                              "deposit-refund-1"),
+    TALER_TESTING_cmd_refund_with_id ("refund-ok",
+                                      MHD_HTTP_OK,
+                                      "EUR:3",
+                                      "deposit-refund-1",
+                                      3),
+    TALER_TESTING_cmd_refund_with_id ("refund-ok-double",
+                                      MHD_HTTP_OK,
+                                      "EUR:3",
+                                      "deposit-refund-1",
+                                      3),
     /* Previous /refund(s) had id == 0.  */
     TALER_TESTING_cmd_refund_with_id ("refund-conflicting",
                                       MHD_HTTP_CONFLICT,
                                       "EUR:5",
-                                      "EUR:0.01",
                                       "deposit-refund-1",
                                       1),
+    TALER_TESTING_cmd_deposit ("deposit-refund-insufficient-refund",
+                               "withdraw-coin-r1",
+                               0,
+                               bc.user42_payto,
+                               "{\"items\":[{\"name\":\"ice cream\",\"value\":\"EUR:4\"}]}",
+                               GNUNET_TIME_UNIT_MINUTES,
+                               "EUR:4",
+                               MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_refund_with_id ("refund-ok-increase",
+                                      MHD_HTTP_OK,
+                                      "EUR:2",
+                                      "deposit-refund-1",
+                                      2),
     /**
      * Spend 4.99 EUR of the refunded 4.99 EUR coin (1ct gone
      * due to refund) (merchant would receive EUR:4.98 due to
@@ -573,7 +612,6 @@ run (void *cls,
     TALER_TESTING_cmd_refund ("refund-fail",
                               MHD_HTTP_GONE,
                               "EUR:4.99",
-                              "EUR:0.01",
                               "deposit-refund-2"),
     TALER_TESTING_cmd_check_bank_empty ("check-empty-after-refund"),
     /**
@@ -607,7 +645,6 @@ run (void *cls,
     TALER_TESTING_cmd_refund ("refund-ok-fast",
                               MHD_HTTP_OK,
                               "EUR:5",
-                              "EUR:0.01",
                               "deposit-refund-1b"),
     /**
      * Run transfers. This will do the transfer as refund deadline
@@ -626,10 +663,10 @@ run (void *cls,
      * config.
      */
     CMD_TRANSFER_TO_EXCHANGE ("recoup-create-reserve-1",
-                              "EUR:5.01"),
+                              "EUR:15.02"),
     TALER_TESTING_cmd_check_bank_admin_transfer (
       "recoup-create-reserve-1-check",
-      "EUR:5.01",
+      "EUR:15.02",
       bc.user42_payto,
       bc.exchange_payto,
       "recoup-create-reserve-1"),
@@ -642,12 +679,26 @@ run (void *cls,
                                        "recoup-create-reserve-1",
                                        "EUR:5",
                                        MHD_HTTP_OK),
-    /* Make coin invalid */
+    /* Withdraw a 10 EUR coin, at fee of 1 ct */
+    TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-1b",
+                                       "recoup-create-reserve-1",
+                                       "EUR:10",
+                                       MHD_HTTP_OK),
+    /* melt 10 EUR coin to get 5 EUR refreshed coin */
+    TALER_TESTING_cmd_melt ("recoup-melt-coin-1b",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:5",
+                            NULL),
+    TALER_TESTING_cmd_refresh_reveal ("recoup-reveal-coin-1b",
+                                      "recoup-melt-coin-1b",
+                                      MHD_HTTP_OK),
+    /* Revoke both 5 EUR coins */
     TALER_TESTING_cmd_revoke ("revoke-0-EUR:5",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-1",
                               CONFIG_FILE),
-    /* Refund coin to bank account */
+    /* Recoup coin to reserve */
     TALER_TESTING_cmd_recoup ("recoup-1",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-1",
@@ -658,6 +709,67 @@ run (void *cls,
                               "recoup-create-reserve-1",
                               "EUR:5.0",
                               MHD_HTTP_OK),
+    /* Recoup-refresh coin to 10 EUR coin */
+    TALER_TESTING_cmd_recoup ("recoup-1b",
+                              MHD_HTTP_OK,
+                              "recoup-reveal-coin-1b",
+                              "recoup-melt-coin-1b",
+                              "EUR:5"),
+    /* melt 10 EUR coin *again* to get 1 EUR refreshed coin */
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1a",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_refresh_reveal ("recoup-reveal-coin-1a",
+                                      "recoup-remelt-coin-1a",
+                                      MHD_HTTP_OK),
+    /* Try melting for more than the residual value to provoke an error */
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1b",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1c",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1d",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1e",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1f",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1g",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1h",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1i",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_OK,
+                            "EUR:1",
+                            NULL),
+    TALER_TESTING_cmd_melt ("recoup-remelt-coin-1b-failing",
+                            "recoup-withdraw-coin-1b",
+                            MHD_HTTP_CONFLICT,
+                            "EUR:1",
+                            NULL),
     /* Re-withdraw from this reserve */
     TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-2",
                                        "recoup-create-reserve-1",
@@ -676,7 +788,7 @@ run (void *cls,
                               "EUR:3.99",
                               MHD_HTTP_OK),
     /* These commands should close the reserve because
-     * the aggregator is given a config file that ovverrides
+     * the aggregator is given a config file that overrides
      * the reserve expiration time (making it now-ish) */
     CMD_TRANSFER_TO_EXCHANGE ("short-lived-reserve",
                               "EUR:5.01"),
@@ -744,6 +856,12 @@ run (void *cls,
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-2a",
                               CONFIG_FILE),
+    /* Check recoup is failing for the coin with the reused coin key */
+    TALER_TESTING_cmd_recoup ("recoup-2x",
+                              MHD_HTTP_CONFLICT,
+                              "withdraw-coin-1x",
+                              NULL,
+                              "EUR:1"),
     TALER_TESTING_cmd_recoup ("recoup-2",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-2a",
