@@ -81,6 +81,11 @@ struct AdminAddIncomingState
    */
   struct TALER_ReservePrivateKeyP reserve_priv;
 
+  /**
+   * Whether we know the private key or not.
+   */
+  bool reserve_priv_known;
+
   /**
    * Reserve public key matching @e reserve_priv.
    */
@@ -271,6 +276,7 @@ admin_add_incoming_run (void *cls,
                         struct TALER_TESTING_Interpreter *is)
 {
   struct AdminAddIncomingState *fts = cls;
+  bool have_public = false;
 
   (void) cmd;
   /* Use reserve public key as subject */
@@ -278,6 +284,7 @@ admin_add_incoming_run (void *cls,
   {
     const struct TALER_TESTING_Command *ref;
     const struct TALER_ReservePrivateKeyP *reserve_priv;
+    const struct TALER_ReservePublicKeyP *reserve_pub;
 
     ref = TALER_TESTING_interpreter_lookup_command
             (is, fts->reserve_reference);
@@ -292,80 +299,34 @@ admin_add_incoming_run (void *cls,
                                               0,
                                               &reserve_priv))
     {
-      GNUNET_break (0);
-      TALER_TESTING_interpreter_fail (is);
-      return;
-    }
-    fts->reserve_priv.eddsa_priv = reserve_priv->eddsa_priv;
-  }
-  else
-  {
-    if (NULL != fts->instance)
-    {
-      char *section;
-      char *keys;
-      struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
-      struct GNUNET_CONFIGURATION_Handle *cfg;
-
-      GNUNET_assert (NULL != fts->config_filename);
-      cfg = GNUNET_CONFIGURATION_create ();
-      if (GNUNET_OK !=
-          GNUNET_CONFIGURATION_load (cfg,
-                                     fts->config_filename))
+      if (GNUNET_OK != TALER_TESTING_get_trait_reserve_pub (ref,
+                                                            0,
+                                                            &reserve_pub))
       {
         GNUNET_break (0);
         TALER_TESTING_interpreter_fail (is);
         return;
       }
-
-      GNUNET_asprintf (&section,
-                       "instance-%s",
-                       fts->instance);
-      if (GNUNET_OK !=
-          GNUNET_CONFIGURATION_get_value_filename
-            (cfg,
-            section,
-            "TIP_RESERVE_PRIV_FILENAME",
-            &keys))
-      {
-        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Configuration fails to specify reserve"
-                    " private key filename in section %s\n",
-                    section);
-        GNUNET_free (section);
-        TALER_TESTING_interpreter_fail (is);
-        return;
-      }
-      priv = GNUNET_CRYPTO_eddsa_key_create_from_file (keys);
-      GNUNET_free (keys);
-      if (NULL == priv)
-      {
-        GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
-                                   section,
-                                   "TIP_RESERVE_PRIV_FILENAME",
-                                   "Failed to read private key");
-        GNUNET_free (section);
-        TALER_TESTING_interpreter_fail (is);
-        return;
-      }
-      fts->reserve_priv.eddsa_priv = *priv;
-      GNUNET_free (section);
-      GNUNET_free (priv);
-      GNUNET_CONFIGURATION_destroy (cfg);
+      have_public = true;
+      fts->reserve_pub.eddsa_pub = reserve_pub->eddsa_pub;
+      fts->reserve_priv_known = false;
     }
     else
     {
-      /* No referenced reserve, no instance to take priv
-       * from, no explicit subject given: create new key! */
-      struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
-
-      priv = GNUNET_CRYPTO_eddsa_key_create ();
-      fts->reserve_priv.eddsa_priv = *priv;
-      GNUNET_free (priv);
+      fts->reserve_priv.eddsa_priv = reserve_priv->eddsa_priv;
+      fts->reserve_priv_known = true;
     }
   }
-  GNUNET_CRYPTO_eddsa_key_get_public (&fts->reserve_priv.eddsa_priv,
-                                      &fts->reserve_pub.eddsa_pub);
+  else
+  {
+    /* No referenced reserve, no instance to take priv
+     * from, no explicit subject given: create new key! */
+    GNUNET_CRYPTO_eddsa_key_create (&fts->reserve_priv.eddsa_priv);
+    fts->reserve_priv_known = true;
+  }
+  if (! have_public)
+    GNUNET_CRYPTO_eddsa_key_get_public (&fts->reserve_priv.eddsa_priv,
+                                        &fts->reserve_pub.eddsa_pub);
   fts->reserve_history.type = TALER_EXCHANGE_RTT_CREDIT;
   fts->reserve_history.amount = fts->amount;
   fts->reserve_history.details.in_details.sender_url
@@ -436,30 +397,58 @@ admin_add_incoming_traits (void *cls,
                            unsigned int index)
 {
   struct AdminAddIncomingState *fts = cls;
-  struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_bank_row (&fts->serial_id),
-    TALER_TESTING_make_trait_payto (TALER_TESTING_PT_DEBIT,
-                                    fts->payto_debit_account),
-    /* Used as a marker, content does not matter */
-    TALER_TESTING_make_trait_payto (TALER_TESTING_PT_CREDIT,
-                                    "payto://void/the-exchange"),
-    TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BANK_ACCOUNT_URL,
-                                  fts->exchange_credit_url),
-    TALER_TESTING_make_trait_amount_obj (0, &fts->amount),
-    TALER_TESTING_make_trait_absolute_time (0, &fts->timestamp),
-    TALER_TESTING_make_trait_reserve_priv (0,
-                                           &fts->reserve_priv),
-    TALER_TESTING_make_trait_reserve_pub (0,
-                                          &fts->reserve_pub),
-    TALER_TESTING_make_trait_reserve_history (0,
-                                              &fts->reserve_history),
-    TALER_TESTING_trait_end ()
-  };
+  if (fts->reserve_priv_known)
+  {
+    struct TALER_TESTING_Trait traits[] = {
+      TALER_TESTING_make_trait_bank_row (&fts->serial_id),
+      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_DEBIT,
+                                      fts->payto_debit_account),
+      /* Used as a marker, content does not matter */
+      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_CREDIT,
+                                      "payto://void/the-exchange"),
+      TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BANK_ACCOUNT_URL,
+                                    fts->exchange_credit_url),
+      TALER_TESTING_make_trait_amount_obj (0, &fts->amount),
+      TALER_TESTING_make_trait_absolute_time (0, &fts->timestamp),
+      TALER_TESTING_make_trait_reserve_priv (0,
+                                             &fts->reserve_priv),
+      TALER_TESTING_make_trait_reserve_pub (0,
+                                            &fts->reserve_pub),
+      TALER_TESTING_make_trait_reserve_history (0,
+                                                &fts->reserve_history),
+      TALER_TESTING_trait_end ()
+    };
 
-  return TALER_TESTING_get_trait (traits,
-                                  ret,
-                                  trait,
-                                  index);
+    return TALER_TESTING_get_trait (traits,
+                                    ret,
+                                    trait,
+                                    index);
+  }
+  else
+  {
+    struct TALER_TESTING_Trait traits[] = {
+      TALER_TESTING_make_trait_bank_row (&fts->serial_id),
+      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_DEBIT,
+                                      fts->payto_debit_account),
+      /* Used as a marker, content does not matter */
+      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_CREDIT,
+                                      "payto://void/the-exchange"),
+      TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BANK_ACCOUNT_URL,
+                                    fts->exchange_credit_url),
+      TALER_TESTING_make_trait_amount_obj (0, &fts->amount),
+      TALER_TESTING_make_trait_absolute_time (0, &fts->timestamp),
+      TALER_TESTING_make_trait_reserve_pub (0,
+                                            &fts->reserve_pub),
+      TALER_TESTING_make_trait_reserve_history (0,
+                                                &fts->reserve_history),
+      TALER_TESTING_trait_end ()
+    };
+
+    return TALER_TESTING_get_trait (traits,
+                                    ret,
+                                    trait,
+                                    index);
+  }
 }
 
 
@@ -552,7 +541,7 @@ TALER_TESTING_cmd_admin_add_incoming (const char *label,
  * @param payto_debit_account which account sends money
  * @param auth authentication data
  * @param ref reference to a command that can offer a reserve
- *        private key.
+ *        private key or public key.
  * @return the command.
  */
 struct TALER_TESTING_Command
@@ -574,47 +563,6 @@ TALER_TESTING_cmd_admin_add_incoming_with_ref
 }
 
 
-/**
- * Create "/admin/add-incoming" CMD, letting the caller specifying
- * the merchant instance.  This version is useful when a tip
- * reserve should be topped up, in fact the interpreter will need
- * the "tipping instance" in order to get the instance public key
- * and make a wire transfer subject out of it.
- *
- * @param label command label.
- * @param amount amount to transfer.
- * @param payto_debit_account which account (expressed as a number)
- *        gives money
- * @param auth authentication data
- * @param instance the instance that runs the tipping.  Under this
- *        instance, the configuration file will provide the private
- *        key of the tipping reserve.  This data will then used to
- *        construct the wire transfer subject line.
- * @param config_filename configuration file to use.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_admin_add_incoming_with_instance
-  (const char *label,
-  const char *amount,
-  const struct TALER_BANK_AuthenticationData *auth,
-  const char *payto_debit_account,
-  const char *instance,
-  const char *config_filename)
-{
-  struct AdminAddIncomingState *fts;
-
-  fts = make_fts (amount,
-                  auth,
-                  payto_debit_account);
-  fts->instance = instance;
-  fts->config_filename = config_filename;
-
-  return make_command (label,
-                       fts);
-}
-
-
 /**
  * Modify a fakebank transfer command to enable retries when the
  * reserve is not yet full or we get other transient errors from the
