@@ -52,6 +52,11 @@ struct DepositState
    */
   struct TALER_Amount amount;
 
+  /**
+   * Deposit fee.
+   */
+  struct TALER_Amount deposit_fee;
+
   /**
    * Reference to any command that is able to provide a coin.
    */
@@ -91,9 +96,9 @@ struct DepositState
   struct TALER_EXCHANGE_DepositHandle *dh;
 
   /**
-   * Timestamp of the /deposit operation.
+   * Timestamp of the /deposit operation in the wallet (contract signing time).
    */
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Absolute wallet_timestamp;
 
   /**
    * Interpreter state.
@@ -126,6 +131,11 @@ struct DepositState
    */
   int deposit_succeeded;
 
+  /**
+   * When did the exchange receive the deposit?
+   */
+  struct GNUNET_TIME_Absolute exchange_timestamp;
+
   /**
    * Signing key used by the exchange to sign the
    * deposit confirmation.
@@ -197,40 +207,38 @@ do_retry (void *cls)
  * check if the response code is acceptable.
  *
  * @param cls closure.
- * @param http_status HTTP response code.
- * @param ec taler-specific error code.
+ * @param hr HTTP response details
+ * @param exchange_timestamp when did the exchange receive the deposit permission
  * @param exchange_sig signature provided by the exchange
  *        (NULL on errors)
  * @param exchange_pub public key of the exchange,
  *        used for signing the response.
- * @param obj raw response from the exchange.
  */
 static void
 deposit_cb (void *cls,
-            unsigned int http_status,
-            enum TALER_ErrorCode ec,
+            const struct TALER_EXCHANGE_HttpResponse *hr,
+            const struct GNUNET_TIME_Absolute exchange_timestamp,
             const struct TALER_ExchangeSignatureP *exchange_sig,
-            const struct TALER_ExchangePublicKeyP *exchange_pub,
-            const json_t *obj)
+            const struct TALER_ExchangePublicKeyP *exchange_pub)
 {
   struct DepositState *ds = cls;
 
   ds->dh = NULL;
-  if (ds->expected_response_code != http_status)
+  if (ds->expected_response_code != hr->http_status)
   {
     if (0 != ds->do_retry)
     {
       ds->do_retry--;
-      if ( (0 == http_status) ||
-           (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == ec) ||
-           (MHD_HTTP_INTERNAL_SERVER_ERROR == http_status) )
+      if ( (0 == hr->http_status) ||
+           (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == hr->ec) ||
+           (MHD_HTTP_INTERNAL_SERVER_ERROR == hr->http_status) )
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Retrying deposit failed with %u/%d\n",
-                    http_status,
-                    (int) ec);
+                    hr->http_status,
+                    (int) hr->ec);
         /* on DB conflicts, do not use backoff */
-        if (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == ec)
+        if (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == hr->ec)
           ds->backoff = GNUNET_TIME_UNIT_ZERO;
         else
           ds->backoff = GNUNET_TIME_randomized_backoff (ds->backoff,
@@ -245,17 +253,20 @@ deposit_cb (void *cls,
     }
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u to command %s in %s:%u\n",
-                http_status,
+                hr->http_status,
                 ds->is->commands[ds->is->ip].label,
                 __FILE__,
                 __LINE__);
-    json_dumpf (obj, stderr, 0);
+    json_dumpf (hr->reply,
+                stderr,
+                0);
     TALER_TESTING_interpreter_fail (ds->is);
     return;
   }
-  if (MHD_HTTP_OK == http_status)
+  if (MHD_HTTP_OK == hr->http_status)
   {
     ds->deposit_succeeded = GNUNET_YES;
+    ds->exchange_timestamp = exchange_timestamp;
     ds->exchange_pub = *exchange_pub;
     ds->exchange_sig = *exchange_sig;
   }
@@ -307,7 +318,7 @@ deposit_run (void *cls,
     ds->coin_index = ods->coin_index;
     ds->wire_details = json_incref (ods->wire_details);
     ds->contract_terms = json_incref (ods->contract_terms);
-    ds->timestamp = ods->timestamp;
+    ds->wallet_timestamp = ods->wallet_timestamp;
     ds->refund_deadline = ods->refund_deadline;
     ds->amount = ods->amount;
     ds->merchant_priv = ods->merchant_priv;
@@ -361,13 +372,14 @@ deposit_run (void *cls,
                                            ds->coin_index,
                                            &denom_pub_sig)) ||
        (GNUNET_OK !=
-        TALER_JSON_hash (ds->contract_terms,
-                         &h_contract_terms)) )
+        TALER_JSON_contract_hash (ds->contract_terms,
+                                  &h_contract_terms)) )
   {
     GNUNET_break (0);
     TALER_TESTING_interpreter_fail (is);
     return;
   }
+  ds->deposit_fee = denom_pub->fee_deposit;
   GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv->eddsa_priv,
                                       &coin_pub.eddsa_pub);
 
@@ -381,40 +393,28 @@ deposit_run (void *cls,
   }
   else
   {
-    ds->refund_deadline = ds->timestamp;
-    wire_deadline = GNUNET_TIME_relative_to_absolute
-                      (GNUNET_TIME_UNIT_ZERO);
+    ds->refund_deadline = ds->wallet_timestamp;
+    wire_deadline = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_ZERO);
   }
   GNUNET_CRYPTO_eddsa_key_get_public (&ds->merchant_priv.eddsa_priv,
                                       &merchant_pub.eddsa_pub);
-
   (void) GNUNET_TIME_round_abs (&wire_deadline);
-
   {
-    struct TALER_DepositRequestPS dr;
+    struct GNUNET_HashCode h_wire;
 
-    memset (&dr, 0, sizeof (dr));
-    dr.purpose.size = htonl
-                        (sizeof (struct TALER_DepositRequestPS));
-    dr.purpose.purpose = htonl
-                           (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
-    dr.h_contract_terms = h_contract_terms;
     GNUNET_assert (GNUNET_OK ==
                    TALER_JSON_merchant_wire_signature_hash (ds->wire_details,
-                                                            &dr.h_wire));
-    dr.timestamp = GNUNET_TIME_absolute_hton (ds->timestamp);
-    dr.refund_deadline = GNUNET_TIME_absolute_hton
-                           (ds->refund_deadline);
-    TALER_amount_hton (&dr.amount_with_fee,
-                       &ds->amount);
-    TALER_amount_hton (&dr.deposit_fee,
-                       &denom_pub->fee_deposit);
-    dr.merchant = merchant_pub;
-    dr.coin_pub = coin_pub;
-    GNUNET_assert (GNUNET_OK ==
-                   GNUNET_CRYPTO_eddsa_sign (&coin_priv->eddsa_priv,
-                                             &dr.purpose,
-                                             &coin_sig.eddsa_signature));
+                                                            &h_wire));
+    TALER_EXCHANGE_deposit_permission_sign (&ds->amount,
+                                            &denom_pub->fee_deposit,
+                                            &h_wire,
+                                            &h_contract_terms,
+                                            &denom_pub->h_key,
+                                            coin_priv,
+                                            ds->wallet_timestamp,
+                                            &merchant_pub,
+                                            ds->refund_deadline,
+                                            &coin_sig);
   }
   ds->dh = TALER_EXCHANGE_deposit (is->exchange,
                                    &ds->amount,
@@ -424,7 +424,7 @@ deposit_run (void *cls,
                                    &coin_pub,
                                    denom_pub_sig,
                                    &denom_pub->key,
-                                   ds->timestamp,
+                                   ds->wallet_timestamp,
                                    &merchant_pub,
                                    ds->refund_deadline,
                                    &coin_sig,
@@ -535,8 +535,14 @@ deposit_traits (void *cls,
                                                ds->contract_terms),
       TALER_TESTING_make_trait_merchant_priv (0,
                                               &ds->merchant_priv),
-      TALER_TESTING_make_trait_amount_obj (0,
-                                           &ds->amount),
+      TALER_TESTING_make_trait_amount_obj (
+        TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_VALUE,
+        &ds->amount),
+      TALER_TESTING_make_trait_amount_obj (
+        TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_FEE,
+        &ds->deposit_fee),
+      TALER_TESTING_make_trait_absolute_time (0,
+                                              &ds->exchange_timestamp),
       TALER_TESTING_trait_end ()
     };
 
@@ -584,7 +590,6 @@ TALER_TESTING_cmd_deposit (const char *label,
 {
   struct DepositState *ds;
   json_t *wire_details;
-  struct GNUNET_CRYPTO_EddsaPrivateKey *merchant_priv;
 
   wire_details = TALER_TESTING_make_wire_details (target_account_payto);
   ds = GNUNET_new (struct DepositState);
@@ -594,9 +599,7 @@ TALER_TESTING_cmd_deposit (const char *label,
   ds->contract_terms = json_loads (contract_terms,
                                    JSON_REJECT_DUPLICATES,
                                    NULL);
-  merchant_priv = GNUNET_CRYPTO_eddsa_key_create ();
-  ds->merchant_priv.eddsa_priv = *merchant_priv;
-  GNUNET_free (merchant_priv);
+  GNUNET_CRYPTO_eddsa_key_create (&ds->merchant_priv.eddsa_priv);
   if (NULL == ds->contract_terms)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -605,12 +608,12 @@ TALER_TESTING_cmd_deposit (const char *label,
                 label);
     GNUNET_assert (0);
   }
-  ds->timestamp = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&ds->timestamp);
+  ds->wallet_timestamp = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&ds->wallet_timestamp);
 
   json_object_set_new (ds->contract_terms,
                        "timestamp",
-                       GNUNET_JSON_from_time_abs (ds->timestamp));
+                       GNUNET_JSON_from_time_abs (ds->wallet_timestamp));
   if (0 != refund_deadline.rel_value_us)
   {
     ds->refund_deadline = GNUNET_TIME_relative_to_absolute (refund_deadline);
@@ -693,12 +696,12 @@ TALER_TESTING_cmd_deposit_with_ref (const char *label,
                 label);
     GNUNET_assert (0);
   }
-  ds->timestamp = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&ds->timestamp);
+  ds->wallet_timestamp = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&ds->wallet_timestamp);
 
   json_object_set_new (ds->contract_terms,
                        "timestamp",
-                       GNUNET_JSON_from_time_abs (ds->timestamp));
+                       GNUNET_JSON_from_time_abs (ds->wallet_timestamp));
   if (0 != refund_deadline.rel_value_us)
   {
     ds->refund_deadline = GNUNET_TIME_relative_to_absolute (refund_deadline);
