@@ -61,8 +61,7 @@ transfer_run (void *cls,
   struct TransferState *as = cls;
 
   as->transfer_proc
-    = GNUNET_OS_start_process (GNUNET_NO,
-                               GNUNET_OS_INHERIT_STD_ALL,
+    = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                NULL, NULL, NULL,
                                "taler-exchange-transfer",
                                "taler-exchange-transfer",
