@@ -232,15 +232,14 @@ hack_response_code_run (void *cls,
   GNUNET_asprintf (&http_status, "%u",
                    hrcs->http_status);
 
-  hrcs->proc = GNUNET_OS_start_process
-                 (GNUNET_NO,
-                 GNUNET_OS_INHERIT_STD_ALL,
-                 NULL, NULL, NULL,
-                 "taler-twister",
-                 "taler-twister",
-                 "-c", hrcs->config_filename,
-                 "--responsecode", http_status,
-                 NULL);
+  hrcs->proc = GNUNET_OS_start_process (
+    GNUNET_OS_INHERIT_STD_ALL,
+    NULL, NULL, NULL,
+    "taler-twister",
+    "taler-twister",
+    "-c", hrcs->config_filename,
+    "--responsecode", http_status,
+    NULL);
   if (NULL == hrcs->proc)
   {
     GNUNET_break (0);
@@ -355,8 +354,7 @@ delete_object_run (void *cls,
 {
   struct DeleteObjectState *dos = cls;
 
-  dos->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  dos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -443,8 +441,7 @@ modify_object_dl_run (void *cls,
 {
   struct ModifyObjectState *mos = cls;
 
-  mos->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  mos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -476,8 +473,7 @@ modify_object_ul_run (void *cls,
 {
   struct ModifyObjectState *mos = cls;
 
-  mos->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  mos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -495,6 +491,38 @@ modify_object_ul_run (void *cls,
 }
 
 
+/**
+ * Run a "modify header" CMD
+ *
+ * @param cls closure.
+ * @param cmd the command being run.
+ * @param is the interpreter state.
+ */
+static void
+modify_header_dl_run (void *cls,
+                      const struct TALER_TESTING_Command *cmd,
+                      struct TALER_TESTING_Interpreter *is)
+{
+  struct ModifyObjectState *mos = cls;
+
+  mos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
+                                       NULL, NULL, NULL,
+                                       "taler-twister",
+                                       "taler-twister",
+                                       "-H", mos->path,
+                                       "--value", mos->value,
+                                       "-c", mos->config_filename,
+                                       NULL);
+  if (NULL == mos->proc)
+  {
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (is);
+    return;
+  }
+  TALER_TESTING_wait_for_sigchld (is);
+}
+
+
 /**
  * Create a "delete object" CMD.  This command deletes
  * the JSON object pointed by @a path.
@@ -598,8 +626,7 @@ flip_upload_run (void *cls,
 {
   struct FlipObjectState *fos = cls;
 
-  fos->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  fos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -630,8 +657,7 @@ flip_download_run (void *cls,
 {
   struct FlipObjectState *fos = cls;
 
-  fos->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  fos->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -780,8 +806,7 @@ malform_request_run (void *cls,
 {
   struct MalformRequestState *mrs = cls;
 
-  mrs->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  mrs->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -867,8 +892,7 @@ malform_response_run (void *cls,
 {
   struct MalformResponseState *mrs = cls;
 
-  mrs->proc = GNUNET_OS_start_process (GNUNET_NO,
-                                       GNUNET_OS_INHERIT_STD_ALL,
+  mrs->proc = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ALL,
                                        NULL, NULL, NULL,
                                        "taler-twister",
                                        "taler-twister",
@@ -1020,4 +1044,39 @@ TALER_TESTING_cmd_modify_object_ul (const char *label,
 }
 
 
+/**
+ * Create a "modify header" CMD.  This command instructs
+ * the twister to modify a header in the next HTTP response.
+ *
+ * @param label command label
+ * @param config_filename configuration filename.
+ * @param path identifies the location to modify
+ * @param value value to set the header to.
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_modify_header_dl (const char *label,
+                                    const char *config_filename,
+                                    const char *path,
+                                    const char *value)
+{
+  struct ModifyObjectState *mos;
+
+  mos = GNUNET_new (struct ModifyObjectState);
+  mos->path = path;
+  mos->value = value;
+  mos->config_filename = config_filename;
+
+  struct TALER_TESTING_Command cmd = {
+    .label = label,
+    .run = &modify_header_dl_run,
+    .cleanup = &modify_object_cleanup,
+    .traits = &modify_object_traits,
+    .cls = mos
+  };
+
+  return cmd;
+}
+
+
 /* end of testing_api_cmd_exec_client.c */
