@@ -59,6 +59,12 @@ struct WithdrawState
    */
   const char *reserve_reference;
 
+  /**
+   * Reference to a withdraw or reveal operation from which we should
+   * re-use the private coin key, or NULL for regular withdrawal.
+   */
+  const char *reuse_coin_key_ref;
+
   /**
    * String describing the denomination value we should withdraw.
    * A corresponding denomination key must exist in the exchange's
@@ -172,42 +178,38 @@ do_retry (void *cls)
  * in the state.
  *
  * @param cls closure.
- * @param http_status HTTP response code.
- * @param ec taler-specific error code.
+ * @param hr HTTP response details
  * @param sig signature over the coin, NULL on error.
- * @param full_response raw response.
  */
 static void
 reserve_withdraw_cb (void *cls,
-                     unsigned int http_status,
-                     enum TALER_ErrorCode ec,
-                     const struct TALER_DenominationSignature *sig,
-                     const json_t *full_response)
+                     const struct TALER_EXCHANGE_HttpResponse *hr,
+                     const struct TALER_DenominationSignature *sig)
 {
   struct WithdrawState *ws = cls;
   struct TALER_TESTING_Interpreter *is = ws->is;
 
   ws->wsh = NULL;
-  if (ws->expected_response_code != http_status)
+  if (ws->expected_response_code != hr->http_status)
   {
     if (0 != ws->do_retry)
     {
-      if (TALER_EC_WITHDRAW_RESERVE_UNKNOWN != ec)
+      if (TALER_EC_WITHDRAW_RESERVE_UNKNOWN != hr->ec)
         ws->do_retry--; /* we don't count reserve unknown as failures here */
-      if ( (0 == http_status) ||
-           (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == ec) ||
-           (TALER_EC_WITHDRAW_INSUFFICIENT_FUNDS == ec) ||
-           (TALER_EC_WITHDRAW_RESERVE_UNKNOWN == ec) ||
-           (MHD_HTTP_INTERNAL_SERVER_ERROR == http_status) )
+      if ( (0 == hr->http_status) ||
+           (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == hr->ec) ||
+           (TALER_EC_WITHDRAW_INSUFFICIENT_FUNDS == hr->ec) ||
+           (TALER_EC_WITHDRAW_RESERVE_UNKNOWN == hr->ec) ||
+           (MHD_HTTP_INTERNAL_SERVER_ERROR == hr->http_status) )
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Retrying withdraw failed with %u/%d\n",
-                    http_status,
-                    (int) ec);
+                    hr->http_status,
+                    (int) hr->ec);
         /* on DB conflicts, do not use backoff */
-        if (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == ec)
+        if (TALER_EC_DB_COMMIT_FAILED_ON_RETRY == hr->ec)
           ws->backoff = GNUNET_TIME_UNIT_ZERO;
-        else if (TALER_EC_WITHDRAW_RESERVE_UNKNOWN != ec)
+        else if (TALER_EC_WITHDRAW_RESERVE_UNKNOWN != hr->ec)
           ws->backoff = EXCHANGE_LIB_BACKOFF (ws->backoff);
         else
           ws->backoff = GNUNET_TIME_relative_max (UNKNOWN_MIN_BACKOFF,
@@ -225,19 +227,19 @@ reserve_withdraw_cb (void *cls,
     }
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d to command %s in %s:%u\n",
-                http_status,
-                (int) ec,
+                hr->http_status,
+                (int) hr->ec,
                 TALER_TESTING_interpreter_get_current_label (is),
                 __FILE__,
                 __LINE__);
-    json_dumpf (full_response,
+    json_dumpf (hr->reply,
                 stderr,
                 0);
     GNUNET_break (0);
     TALER_TESTING_interpreter_fail (is);
     return;
   }
-  switch (http_status)
+  switch (hr->http_status)
   {
   case MHD_HTTP_OK:
     if (NULL == sig)
@@ -246,8 +248,8 @@ reserve_withdraw_cb (void *cls,
       TALER_TESTING_interpreter_fail (is);
       return;
     }
-    ws->sig.rsa_signature
-      = GNUNET_CRYPTO_rsa_signature_dup (sig->rsa_signature);
+    ws->sig.rsa_signature = GNUNET_CRYPTO_rsa_signature_dup (
+      sig->rsa_signature);
     if (0 != ws->total_backoff.rel_value_us)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -270,7 +272,7 @@ reserve_withdraw_cb (void *cls,
     /* Unsupported status code (by test harness) */
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Withdraw test command does not support status code %u\n",
-                http_status);
+                hr->http_status);
     GNUNET_break (0);
     break;
   }
@@ -278,6 +280,50 @@ reserve_withdraw_cb (void *cls,
 }
 
 
+/**
+ * Parser reference to a coin.
+ *
+ * @param coin_reference of format $LABEL['#' $INDEX]?
+ * @param[out] cref where we return a copy of $LABEL
+ * @param[out] idx where we set $INDEX
+ * @return #GNUNET_SYSERR if $INDEX is present but not numeric
+ */
+static int
+parse_coin_reference (const char *coin_reference,
+                      char **cref,
+                      unsigned int *idx)
+{
+  const char *index;
+
+  /* We allow command references of the form "$LABEL#$INDEX" or
+     just "$LABEL", which implies the index is 0. Figure out
+     which one it is. */
+  index = strchr (coin_reference, '#');
+  if (NULL == index)
+  {
+    *idx = 0;
+    *cref = GNUNET_strdup (coin_reference);
+    return GNUNET_OK;
+  }
+  *cref = GNUNET_strndup (coin_reference,
+                          index - coin_reference);
+  if (1 != sscanf (index + 1,
+                   "%u",
+                   idx))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Numeric index (not `%s') required after `#' in command reference of command in %s:%u\n",
+                index,
+                __FILE__,
+                __LINE__);
+    GNUNET_free (*cref);
+    *cref = NULL;
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
+
 /**
  * Run the command.
  */
@@ -311,7 +357,32 @@ withdraw_run (void *cls,
     TALER_TESTING_interpreter_fail (is);
     return;
   }
-  TALER_planchet_setup_random (&ws->ps);
+  if (NULL == ws->reuse_coin_key_ref)
+  {
+    TALER_planchet_setup_random (&ws->ps);
+  }
+  else
+  {
+    const struct TALER_CoinSpendPrivateKeyP *coin_priv;
+    const struct TALER_TESTING_Command *cref;
+    char *cstr;
+    unsigned int index;
+
+    GNUNET_assert (GNUNET_OK ==
+                   parse_coin_reference (ws->reuse_coin_key_ref,
+                                         &cstr,
+                                         &index));
+    cref = TALER_TESTING_interpreter_lookup_command (is,
+                                                     cstr);
+    GNUNET_assert (NULL != cref);
+    GNUNET_free (cstr);
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_TESTING_get_trait_coin_priv (cref,
+                                                      index,
+                                                      &coin_priv));
+    TALER_planchet_setup_random (&ws->ps);
+    ws->ps.coin_priv = *coin_priv;
+  }
   ws->is = is;
   if (NULL == ws->pk)
   {
@@ -335,7 +406,7 @@ withdraw_run (void *cls,
     ws->amount = ws->pk->value;
   }
   ws->reserve_history.type = TALER_EXCHANGE_RTT_WITHDRAWAL;
-  GNUNET_assert (GNUNET_OK ==
+  GNUNET_assert (0 <=
                  TALER_amount_add (&ws->reserve_history.amount,
                                    &ws->amount,
                                    &ws->pk->fee_withdraw));
@@ -391,7 +462,7 @@ withdraw_cleanup (void *cls,
     TALER_EXCHANGE_destroy_denomination_key (ws->pk);
     ws->pk = NULL;
   }
-  GNUNET_free_non_null (ws->exchange_url);
+  GNUNET_free (ws->exchange_url);
   GNUNET_free (ws);
 }
 
@@ -530,6 +601,44 @@ TALER_TESTING_cmd_withdraw_amount (const char *label,
 }
 
 
+/**
+ * Create a withdraw command, letting the caller specify
+ * the desired amount as string and also re-using an existing
+ * coin private key in the process (violating the specification,
+ * which will result in an error when spending the coin!).
+ *
+ * @param label command label.
+ * @param reserve_reference command providing us with a reserve to withdraw from
+ * @param amount how much we withdraw.
+ * @param coin_ref reference to (withdraw/reveal) command of a coin
+ *        from which we should re-use the private key
+ * @param expected_response_code which HTTP response code
+ *        we expect from the exchange.
+ * @return the withdraw command to be executed by the interpreter.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_withdraw_amount_reuse_key (
+  const char *label,
+  const char *reserve_reference,
+  const char *amount,
+  const char *coin_ref,
+  unsigned int expected_response_code)
+{
+  struct TALER_TESTING_Command cmd;
+
+  cmd = TALER_TESTING_cmd_withdraw_amount (label,
+                                           reserve_reference,
+                                           amount,
+                                           expected_response_code);
+  {
+    struct WithdrawState *ws = cmd.cls;
+
+    ws->reuse_coin_key_ref = coin_ref;
+  }
+  return cmd;
+}
+
+
 /**
  * Create withdraw command, letting the caller specify the
  * amount by a denomination key.
