@@ -100,6 +100,128 @@ TALER_TESTING_has_in_name (const char *prog,
 }
 
 
+/**
+ * Start the (nexus) bank process.  Assume the port
+ * is available and the database is clean.  Use the "prepare
+ * bank" function to do such tasks.  This function is also
+ * responsible to create the exchange user at Nexus.
+ *
+ * @param bc bank configuration of the bank
+ * @return the pair of both service handles.  In case of
+ *         errors, each element of the pair will be set to NULL.
+ */
+struct TALER_TESTING_LibeufinServices
+TALER_TESTING_run_libeufin (const struct TALER_TESTING_BankConfiguration *bc)
+{
+  struct GNUNET_OS_Process *nexus_proc;
+  struct GNUNET_OS_Process *sandbox_proc;
+  struct TALER_TESTING_LibeufinServices ret = { 0 };
+  unsigned int iter;
+  char *curl_check_cmd;
+
+  nexus_proc = GNUNET_OS_start_process (
+    GNUNET_OS_INHERIT_STD_NONE,
+    NULL, NULL, NULL,
+    "libeufin-nexus",
+    "libeufin-nexus",
+    "serve",
+    "--db-name", "/tmp/nexus-exchange-test.sqlite3",
+    NULL);
+  if (NULL == nexus_proc)
+  {
+    GNUNET_break (0);
+    return ret;
+  }
+  GNUNET_asprintf (&curl_check_cmd,
+                   "curl -s %s",
+                   bc->exchange_auth.wire_gateway_url);
+  /* give child time to start and bind against the socket */
+  fprintf (stderr,
+           "Waiting for `nexus' to be ready (via %s)\n", curl_check_cmd);
+  iter = 0;
+  do
+  {
+    if (10 == iter)
+    {
+      fprintf (
+        stderr,
+        "Failed to launch `nexus'\n");
+      GNUNET_OS_process_kill (nexus_proc,
+                              SIGTERM);
+      GNUNET_OS_process_wait (nexus_proc);
+      GNUNET_OS_process_destroy (nexus_proc);
+      GNUNET_free (curl_check_cmd);
+      GNUNET_break (0);
+      return ret;
+    }
+    fprintf (stderr, ".");
+    sleep (1);
+    iter++;
+  }
+  while (0 != system (curl_check_cmd));
+
+  // start sandbox.
+  GNUNET_free (curl_check_cmd);
+  fprintf (stderr, "\n");
+
+  sandbox_proc = GNUNET_OS_start_process (
+    GNUNET_OS_INHERIT_STD_NONE,
+    NULL, NULL, NULL,
+    "libeufin-sandbox",
+    "libeufin-sandbox",
+    "serve",
+    "--db-name", "/tmp/sandbox-exchange-test.sqlite3",
+    NULL);
+  if (NULL == sandbox_proc)
+  {
+    GNUNET_break (0);
+    return ret;
+  }
+
+  /* give child time to start and bind against the socket */
+  fprintf (stderr,
+           "Waiting for `sandbox' to be ready.\n");
+  iter = 0;
+  do
+  {
+    if (10 == iter)
+    {
+      fprintf (
+        stderr,
+        "Failed to launch `sandbox'\n");
+      GNUNET_OS_process_kill (sandbox_proc,
+                              SIGTERM);
+      GNUNET_OS_process_wait (sandbox_proc);
+      GNUNET_OS_process_destroy (sandbox_proc);
+      GNUNET_break (0);
+      return ret;
+    }
+    fprintf (stderr, ".");
+    sleep (1);
+    iter++;
+  }
+  while (0 != system ("curl -s http://localhost:5000/"));
+  fprintf (stderr, "\n");
+
+  // Creates nexus user + bank loopback connection + Taler facade.
+  if (0 != system ("taler-nexus-prepare"))
+  {
+    GNUNET_OS_process_kill (nexus_proc, SIGTERM);
+    GNUNET_OS_process_wait (nexus_proc);
+    GNUNET_OS_process_destroy (nexus_proc);
+    GNUNET_OS_process_kill (sandbox_proc, SIGTERM);
+    GNUNET_OS_process_wait (sandbox_proc);
+    GNUNET_OS_process_destroy (sandbox_proc);
+    TALER_LOG_ERROR ("Could not prepare nexus\n");
+    GNUNET_break (0);
+    return ret;
+  }
+  ret.nexus = nexus_proc;
+  ret.sandbox = sandbox_proc;
+  return ret;
+}
+
+
 /**
  * Start the (Python) bank process.  Assume the port
  * is available and the database is clean.  Use the "prepare
@@ -119,8 +241,6 @@ TALER_TESTING_run_bank (const char *config_filename,
   unsigned int iter;
   char *wget_cmd;
   char *database;
-  char *serve_cfg;
-  char *serve_arg;
   struct GNUNET_CONFIGURATION_Handle *cfg;
 
   cfg = GNUNET_CONFIGURATION_create ();
@@ -146,36 +266,15 @@ TALER_TESTING_run_bank (const char *config_filename,
     GNUNET_CONFIGURATION_destroy (cfg);
     exit (77);
   }
-
-  if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_string (cfg,
-                                             "bank",
-                                             "serve",
-                                             &serve_cfg))
-  {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_WARNING,
-                               "bank",
-                               "serve");
-    GNUNET_break (0);
-    GNUNET_CONFIGURATION_destroy (cfg);
-    GNUNET_free (database);
-    exit (77);
-  }
   GNUNET_CONFIGURATION_destroy (cfg);
-
-  serve_arg = "serve-http";
-  if (0 != strcmp ("http", serve_cfg))
-    serve_arg = "serve-uwsgi";
-  GNUNET_free (serve_cfg);
-  bank_proc = GNUNET_OS_start_process
-                (GNUNET_NO,
-                GNUNET_OS_INHERIT_STD_NONE,
-                NULL, NULL, NULL,
-                "taler-bank-manage-testing",
-                "taler-bank-manage-testing",
-                config_filename,
-                database,
-                serve_arg, NULL);
+  bank_proc = GNUNET_OS_start_process (
+    GNUNET_OS_INHERIT_STD_NONE,
+    NULL, NULL, NULL,
+    "taler-bank-manage-testing",
+    "taler-bank-manage-testing",
+    config_filename,
+    database,
+    "serve", NULL);
   GNUNET_free (database);
   if (NULL == bank_proc)
   {
@@ -217,6 +316,122 @@ TALER_TESTING_run_bank (const char *config_filename,
 }
 
 
+/**
+ * Prepare the Nexus execution.  Check if the port is available
+ * and delete old database.
+ *
+ * @param config_filename configuration file name.
+ * @param reset_db should we reset the bank's database
+ * @param config_section section of the configuration with the exchange's account
+ * @param[out] bc set to the bank's configuration data
+ * @return the base url, or NULL upon errors.  Must be freed
+ *         by the caller.
+ */
+int
+TALER_TESTING_prepare_nexus (const char *config_filename,
+                             int reset_db,
+                             const char *config_section,
+                             struct TALER_TESTING_BankConfiguration *bc)
+{
+  struct GNUNET_CONFIGURATION_Handle *cfg;
+  unsigned long long port;
+  char *database = NULL; // silence compiler
+  char *exchange_payto_uri;
+
+  cfg = GNUNET_CONFIGURATION_create ();
+
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_load (cfg, config_filename))
+  {
+    GNUNET_CONFIGURATION_destroy (cfg);
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (cfg,
+                                             config_section,
+                                             "PAYTO_URI",
+                                             &exchange_payto_uri))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_WARNING,
+                               config_section,
+                               "PAYTO_URI");
+    GNUNET_CONFIGURATION_destroy (cfg);
+    return GNUNET_SYSERR;
+  }
+
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_number (cfg,
+                                             "bank",
+                                             "HTTP_PORT",
+                                             &port))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "bank",
+                               "HTTP_PORT");
+    GNUNET_CONFIGURATION_destroy (cfg);
+    GNUNET_free (database);
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+
+  if (GNUNET_OK !=
+      GNUNET_NETWORK_test_port_free (IPPROTO_TCP,
+                                     (uint16_t) port))
+  {
+    fprintf (stderr,
+             "Required port %llu not available, skipping.\n",
+             port);
+    GNUNET_break (0);
+    GNUNET_free (database);
+    GNUNET_CONFIGURATION_destroy (cfg);
+    return GNUNET_SYSERR;
+  }
+
+  /* DB preparation */
+  if (GNUNET_YES == reset_db)
+  {
+    if (0 != system (
+          "rm -f /tmp/nexus-exchange-test.sqlite3 && rm -f /tmp/sandbox-exchange-test.sqlite3"))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Failed to invoke db-removal command.\n");
+      GNUNET_free (database);
+      GNUNET_CONFIGURATION_destroy (cfg);
+      return GNUNET_SYSERR;
+    }
+  }
+
+  if (GNUNET_OK !=
+      TALER_BANK_auth_parse_cfg (cfg,
+                                 config_section,
+                                 &bc->exchange_auth))
+  {
+    GNUNET_break (0);
+    GNUNET_CONFIGURATION_destroy (cfg);
+    return GNUNET_SYSERR;
+  }
+  GNUNET_CONFIGURATION_destroy (cfg);
+  bc->exchange_payto = exchange_payto_uri;
+  bc->user42_payto =
+    "payto://iban/BIC/FR7630006000011234567890189?receiver-name=User42";
+  bc->user43_payto =
+    "payto://iban/BIC/GB33BUKB20201555555555?receiver-name=User43";
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Relying on nexus %s on port %u\n",
+              bc->exchange_auth.wire_gateway_url,
+              (unsigned int) port);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "exchange payto: %s\n",
+              bc->exchange_payto);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
+              bc->user42_payto);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
+              bc->user43_payto);
+  return GNUNET_OK;
+}
+
+
 /**
  * Prepare the bank execution.  Check if the port is available
  * and reset database.
@@ -311,7 +526,6 @@ TALER_TESTING_prepare_bank (const char *config_filename,
   {
     if (NULL ==
         (dbreset_proc = GNUNET_OS_start_process (
-           GNUNET_NO,
            GNUNET_OS_INHERIT_STD_NONE,
            NULL, NULL, NULL,
            "taler-bank-manage",
@@ -476,7 +690,7 @@ TALER_TESTING_prepare_fakebank (const char *config_filename,
               bc->exchange_payto);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
               bc->user42_payto);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user43_payto: %s\n",
               bc->user43_payto);
   return GNUNET_OK;
 }
