@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2015 Taler Systems SA
+  (C) 2015, 2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -32,10 +32,8 @@
 static int
 test_high_level (void)
 {
-  struct GNUNET_CRYPTO_EddsaPrivateKey *pk;
   struct TALER_CoinSpendPrivateKeyP coin_priv;
   struct TALER_CoinSpendPublicKeyP coin_pub;
-  struct GNUNET_CRYPTO_EcdhePrivateKey *pk2;
   struct TALER_TransferPrivateKeyP trans_priv;
   struct TALER_TransferPublicKeyP trans_pub;
   struct TALER_TransferSecretP secret;
@@ -43,14 +41,10 @@ test_high_level (void)
   struct TALER_PlanchetSecretsP fc1;
   struct TALER_PlanchetSecretsP fc2;
 
-  pk = GNUNET_CRYPTO_eddsa_key_create ();
-  coin_priv.eddsa_priv = *pk;
-  GNUNET_free (pk);
+  GNUNET_CRYPTO_eddsa_key_create (&coin_priv.eddsa_priv);
   GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv.eddsa_priv,
                                       &coin_pub.eddsa_pub);
-  pk2 = GNUNET_CRYPTO_ecdhe_key_create ();
-  trans_priv.ecdhe_priv = *pk2;
-  GNUNET_free (pk2);
+  GNUNET_CRYPTO_ecdhe_key_create (&trans_priv.ecdhe_priv);
   GNUNET_CRYPTO_ecdhe_key_get_public (&trans_priv.ecdhe_priv,
                                       &trans_pub.ecdhe_pub);
   TALER_link_derive_transfer_secret (&coin_priv,
@@ -96,6 +90,7 @@ test_planchets (void)
   struct TALER_PlanchetDetail pd;
   struct GNUNET_CRYPTO_RsaSignature *blind_sig;
   struct TALER_FreshCoin coin;
+  struct GNUNET_HashCode c_hash;
 
   dk_priv.rsa_private_key = GNUNET_CRYPTO_rsa_private_key_create (1024);
   dk_pub.rsa_public_key = GNUNET_CRYPTO_rsa_private_key_get_public (
@@ -104,6 +99,7 @@ test_planchets (void)
   GNUNET_assert (GNUNET_OK ==
                  TALER_planchet_prepare (&dk_pub,
                                          &ps,
+                                         &c_hash,
                                          &pd));
   blind_sig = GNUNET_CRYPTO_rsa_sign_blinded (dk_priv.rsa_private_key,
                                               pd.coin_ev,
@@ -113,7 +109,7 @@ test_planchets (void)
                  TALER_planchet_to_coin (&dk_pub,
                                          blind_sig,
                                          &ps,
-                                         &pd.c_hash,
+                                         &c_hash,
                                          &coin));
   GNUNET_CRYPTO_rsa_signature_free (blind_sig);
   GNUNET_CRYPTO_rsa_signature_free (coin.sig.rsa_signature);
@@ -123,6 +119,89 @@ test_planchets (void)
 }
 
 
+static int
+test_exchange_sigs ()
+{
+  const char *pt = "payto://x-taler-bank/localhost/Account";
+  struct TALER_MasterPrivateKeyP priv;
+  struct TALER_MasterPublicKeyP pub;
+  struct TALER_MasterSignatureP sig;
+
+  GNUNET_CRYPTO_eddsa_key_create (&priv.eddsa_priv);
+  TALER_exchange_wire_signature_make (pt,
+                                      &priv,
+                                      &sig);
+  GNUNET_CRYPTO_eddsa_key_get_public (&priv.eddsa_priv,
+                                      &pub.eddsa_pub);
+  if (GNUNET_OK !=
+      TALER_exchange_wire_signature_check (pt,
+                                           &pub,
+                                           &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  if (GNUNET_OK ==
+      TALER_exchange_wire_signature_check (
+        "payto://x-taler-bank/localhost/Other",
+        &pub,
+        &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  return 0;
+}
+
+
+static int
+test_merchant_sigs ()
+{
+  const char *pt = "payto://x-taler-bank/localhost/Account";
+  const char *salt = "my test salt";
+  struct TALER_MerchantPrivateKeyP priv;
+  struct TALER_MerchantPublicKeyP pub;
+  struct TALER_MerchantSignatureP sig;
+
+  GNUNET_CRYPTO_eddsa_key_create (&priv.eddsa_priv);
+  TALER_merchant_wire_signature_make (pt,
+                                      salt,
+                                      &priv,
+                                      &sig);
+  GNUNET_CRYPTO_eddsa_key_get_public (&priv.eddsa_priv,
+                                      &pub.eddsa_pub);
+  if (GNUNET_OK !=
+      TALER_merchant_wire_signature_check (pt,
+                                           salt,
+                                           &pub,
+                                           &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  if (GNUNET_OK ==
+      TALER_merchant_wire_signature_check (
+        "payto://x-taler-bank/localhost/Other",
+        salt,
+        &pub,
+        &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  if (GNUNET_OK ==
+      TALER_merchant_wire_signature_check (pt,
+                                           "other salt",
+                                           &pub,
+                                           &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  return 0;
+}
+
+
 int
 main (int argc,
       const char *const argv[])
@@ -132,7 +211,11 @@ main (int argc,
   if (0 != test_high_level ())
     return 1;
   if (0 != test_planchets ())
-    return 1;
+    return 2;
+  if (0 != test_exchange_sigs ())
+    return 3;
+  if (0 != test_merchant_sigs ())
+    return 4;
   return 0;
 }
 
